@@ -11,15 +11,18 @@
 const uint32_t BambooTracker::CHIP_CLOCK = 3993600 * 2;
 
 BambooTracker::BambooTracker(std::weak_ptr<Configuration> config)
-	: instMan_(std::make_shared<InstrumentsManager>()),
+	: instMan_(std::make_shared<InstrumentsManager>(config.lock()->getOverwriteUnusedUneditedPropety())),
 	  tickCounter_(std::make_shared<TickCounter>()),
 	  mod_(std::make_shared<Module>()),
-	  octave_(4),
+	  curOctave_(4),
 	  curSongNum_(0),
 	  curTrackNum_(0),
 	  curOrderNum_(0),
 	  curStepNum_(0),
 	  curInstNum_(-1),
+	  curVolume_(127),
+	  mkOrder_(-1),
+	  mkStep_(-1),
 	  isFollowPlay_(true)
 {
 	opnaCtrl_ = std::make_shared<OPNAController>(
@@ -36,6 +39,8 @@ BambooTracker::BambooTracker(std::weak_ptr<Configuration> config)
 
 	playback_ = std::make_unique<PlaybackManager>(
 					opnaCtrl_, instMan_, tickCounter_, mod_, config.lock()->getRetrieveChannelState());
+
+	storeOnlyUsedSamples_ = config.lock()->getWriteOnlyUsedSamples();
 }
 
 /********** Change configuration **********/
@@ -49,17 +54,31 @@ void BambooTracker::changeConfiguration(std::weak_ptr<Configuration> config)
 		setMasterVolumeSSG(config.lock()->getMixerVolumeSSG());
 	}
 	playback_->setChannelRetrieving(config.lock()->getRetrieveChannelState());
+	instMan_->setPropertyFindMode(config.lock()->getOverwriteUnusedUneditedPropety());
+	storeOnlyUsedSamples_ = config.lock()->getWriteOnlyUsedSamples();
+	volFMReversed_ = config.lock()->getReverseFMVolumeOrder();
 }
 
-/********** Change octave **********/
+/********** Current octave **********/
 void BambooTracker::setCurrentOctave(int octave)
 {
-	octave_ = octave;
+	curOctave_ = octave;
 }
 
 int BambooTracker::getCurrentOctave() const
 {
-	return octave_;
+	return curOctave_;
+}
+
+/********** Current volume **********/
+void BambooTracker::setCurrentVolume(int volume)
+{
+	curVolume_ = volume;
+}
+
+int BambooTracker::getCurrentVolume() const
+{
+	return curVolume_;
 }
 
 /********** Current track **********/
@@ -86,10 +105,9 @@ int BambooTracker::getCurrentInstrumentNumber() const
 }
 
 /********** Instrument edit **********/
-void BambooTracker::addInstrument(int num, std::string name)
+void BambooTracker::addInstrument(int num, InstrumentType type, std::string name)
 {
-	comMan_.invoke(std::make_unique<AddInstrumentCommand>(
-					   instMan_, num, songStyle_.trackAttribs[static_cast<size_t>(curTrackNum_)].source, name));
+	comMan_.invoke(std::make_unique<AddInstrumentCommand>(instMan_, num, type, name));
 }
 
 void BambooTracker::removeInstrument(int num)
@@ -114,6 +132,11 @@ void BambooTracker::deepCloneInstrument(int num, int refNum)
 	comMan_.invoke(std::make_unique<DeepCloneInstrumentCommand>(instMan_, num, refNum));
 }
 
+void BambooTracker::swapInstruments(int a, int b, bool patternChange)
+{
+	comMan_.invoke(std::make_unique<SwapInstrumentsCommand>(instMan_, mod_, a, b, curSongNum_, patternChange));
+}
+
 void BambooTracker::loadInstrument(BinaryContainer& container, std::string path, int instNum)
 {
 	auto inst = InstrumentIO::loadInstrument(container, path, instMan_, instNum);
@@ -373,46 +396,46 @@ void BambooTracker::setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorT
 }
 
 //--- SSG
-void BambooTracker::addWaveFormSSGSequenceCommand(int wfNum, int type, int data)
+void BambooTracker::addWaveformSSGSequenceCommand(int wfNum, int type, int data)
 {
-	instMan_->addWaveFormSSGSequenceCommand(wfNum, type, data);
+	instMan_->addWaveformSSGSequenceCommand(wfNum, type, data);
 }
 
-void BambooTracker::removeWaveFormSSGSequenceCommand(int wfNum)
+void BambooTracker::removeWaveformSSGSequenceCommand(int wfNum)
 {
-	instMan_->removeWaveFormSSGSequenceCommand(wfNum);
+	instMan_->removeWaveformSSGSequenceCommand(wfNum);
 }
 
-void BambooTracker::setWaveFormSSGSequenceCommand(int wfNum, int cnt, int type, int data)
+void BambooTracker::setWaveformSSGSequenceCommand(int wfNum, int cnt, int type, int data)
 {
-	instMan_->setWaveFormSSGSequenceCommand(wfNum, cnt, type, data);
+	instMan_->setWaveformSSGSequenceCommand(wfNum, cnt, type, data);
 }
 
-void BambooTracker::setWaveFormSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times)
+void BambooTracker::setWaveformSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times)
 {
-	instMan_->setWaveFormSSGLoops(wfNum, std::move(begins), std::move(ends), std::move(times));
+	instMan_->setWaveformSSGLoops(wfNum, std::move(begins), std::move(ends), std::move(times));
 }
 
-void BambooTracker::setWaveFormSSGRelease(int wfNum, ReleaseType type, int begin)
+void BambooTracker::setWaveformSSGRelease(int wfNum, ReleaseType type, int begin)
 {
-	instMan_->setWaveFormSSGRelease(wfNum, type, begin);
+	instMan_->setWaveformSSGRelease(wfNum, type, begin);
 }
 
-void BambooTracker::setInstrumentSSGWaveForm(int instNum, int wfNum)
+void BambooTracker::setInstrumentSSGWaveform(int instNum, int wfNum)
 {
-	instMan_->setInstrumentSSGWaveForm(instNum, wfNum);
+	instMan_->setInstrumentSSGWaveform(instNum, wfNum);
 	opnaCtrl_->updateInstrumentSSG(instNum);
 }
 
-void BambooTracker::setInstrumentSSGWaveFormEnabled(int instNum, bool enabled)
+void BambooTracker::setInstrumentSSGWaveformEnabled(int instNum, bool enabled)
 {
-	instMan_->setInstrumentSSGWaveFormEnabled(instNum, enabled);
+	instMan_->setInstrumentSSGWaveformEnabled(instNum, enabled);
 	opnaCtrl_->updateInstrumentSSG(instNum);
 }
 
-std::vector<int> BambooTracker::getWaveFormSSGUsers(int wfNum) const
+std::vector<int> BambooTracker::getWaveformSSGUsers(int wfNum) const
 {
-	return instMan_->getWaveFormSSGUsers(wfNum);
+	return instMan_->getWaveformSSGUsers(wfNum);
 }
 
 void BambooTracker::addToneNoiseSSGSequenceCommand(int tnNum, int type, int data)
@@ -593,6 +616,255 @@ std::vector<int> BambooTracker::getPitchSSGUsers(int ptNum) const
 	return instMan_->getPitchSSGUsers(ptNum);
 }
 
+//--- ADPCM
+size_t BambooTracker::getADPCMLimit() const
+{
+	return opnaCtrl_->getDRAMSize();
+}
+
+
+size_t BambooTracker::getADPCMStoredSize() const
+{
+	return opnaCtrl_->getADPCMStoredSize();
+}
+
+void BambooTracker::setSampleADPCMRootKeyNumber(int sampNum, int n)
+{
+	instMan_->setSampleADPCMRootKeyNumber(sampNum, n);
+	// opnaCtrl is changed through refInstADPCM (shared_ptr)
+}
+
+int BambooTracker::getSampleADPCMRootKeyNumber(int sampNum) const
+{
+	return instMan_->getSampleADPCMRootKeyNumber(sampNum);
+}
+
+void BambooTracker::setSampleADPCMRootDeltaN(int sampNum, int dn)
+{
+	instMan_->setSampleADPCMRootDeltaN(sampNum, dn);
+	// opnaCtrl is changed through refInstADPCM (shared_ptr)
+}
+
+int BambooTracker::getSampleADPCMRootDeltaN(int sampNum) const
+{
+	return instMan_->getSampleADPCMRootDeltaN(sampNum);
+}
+
+void BambooTracker::setSampleADPCMRepeatEnabled(int sampNum, bool enabled)
+{
+	instMan_->setSampleADPCMRepeatEnabled(sampNum, enabled);
+	// opnaCtrl is changed through refInstADPCM (shared_ptr)
+}
+
+bool BambooTracker::getSampleADPCMRepeatEnabled(int sampNum) const
+{
+	return instMan_->isSampleADPCMRepeatable(sampNum);
+}
+
+void BambooTracker::storeSampleADPCMRawSample(int sampNum, std::vector<uint8_t> sample)
+{
+	instMan_->storeSampleADPCMRawSample(sampNum, sample);
+}
+
+std::vector<uint8_t> BambooTracker::getSampleADPCMRawSample(int sampNum) const
+{
+	return instMan_->getSampleADPCMRawSample(sampNum);
+}
+
+void BambooTracker::clearSampleADPCMRawSample(int sampNum)
+{
+	instMan_->clearSampleADPCMRawSample(sampNum);
+}
+
+void BambooTracker::assignSampleADPCMRawSamples()
+{
+	opnaCtrl_->clearSamplesADPCM();
+	std::vector<int> idcs = storeOnlyUsedSamples_ ? instMan_->getSampleADPCMValidIndices()
+												  : instMan_->getSampleADPCMEntriedIndices();
+	for (auto sampNum : idcs) {
+		std::vector<size_t> addresses
+				= opnaCtrl_->storeSampleADPCM(instMan_->getSampleADPCMRawSample(sampNum));
+		instMan_->setSampleADPCMStartAddress(sampNum, addresses[0]);
+		instMan_->setSampleADPCMStopAddress(sampNum, addresses[1]);
+	}
+}
+
+size_t BambooTracker::getSampleADPCMStartAddress(int sampNum) const
+{
+	return instMan_->getSampleADPCMStartAddress(sampNum);
+}
+
+size_t BambooTracker::getSampleADPCMStopAddress(int sampNum) const
+{
+	return instMan_->getSampleADPCMStopAddress(sampNum);
+}
+
+void BambooTracker::setInstrumentADPCMSample(int instNum, int sampNum)
+{
+	instMan_->setInstrumentADPCMSample(instNum, sampNum);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+std::vector<int> BambooTracker::getSampleADPCMUsers(int sampNum) const
+{
+	return instMan_->getSampleADPCMUsers(sampNum);
+}
+
+void BambooTracker::addEnvelopeADPCMSequenceCommand(int envNum, int type, int data)
+{
+	instMan_->addEnvelopeADPCMSequenceCommand(envNum, type, data);
+}
+
+void BambooTracker::removeEnvelopeADPCMSequenceCommand(int envNum)
+{
+	instMan_->removeEnvelopeADPCMSequenceCommand(envNum);
+}
+
+void BambooTracker::setEnvelopeADPCMSequenceCommand(int envNum, int cnt, int type, int data)
+{
+	instMan_->setEnvelopeADPCMSequenceCommand(envNum, cnt, type, data);
+}
+
+void BambooTracker::setEnvelopeADPCMLoops(int envNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times)
+{
+	instMan_->setEnvelopeADPCMLoops(envNum, std::move(begins), std::move(ends), std::move(times));
+}
+
+void BambooTracker::setEnvelopeADPCMRelease(int envNum, ReleaseType type, int begin)
+{
+	instMan_->setEnvelopeADPCMRelease(envNum, type, begin);
+}
+
+void BambooTracker::setInstrumentADPCMEnvelope(int instNum, int envNum)
+{
+	instMan_->setInstrumentADPCMEnvelope(instNum, envNum);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+void BambooTracker::setInstrumentADPCMEnvelopeEnabled(int instNum, bool enabled)
+{
+	instMan_->setInstrumentADPCMEnvelopeEnabled(instNum, enabled);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+std::vector<int> BambooTracker::getEnvelopeADPCMUsers(int envNum) const
+{
+	return instMan_->getEnvelopeADPCMUsers(envNum);
+}
+
+void BambooTracker::setArpeggioADPCMType(int arpNum, SequenceType type)
+{
+	instMan_->setArpeggioADPCMType(arpNum, type);
+}
+
+void BambooTracker::addArpeggioADPCMSequenceCommand(int arpNum, int type, int data)
+{
+	instMan_->addArpeggioADPCMSequenceCommand(arpNum, type, data);
+}
+
+void BambooTracker::removeArpeggioADPCMSequenceCommand(int arpNum)
+{
+	instMan_->removeArpeggioADPCMSequenceCommand(arpNum);
+}
+
+void BambooTracker::setArpeggioADPCMSequenceCommand(int arpNum, int cnt, int type, int data)
+{
+	instMan_->setArpeggioADPCMSequenceCommand(arpNum, cnt, type, data);
+}
+
+void BambooTracker::setArpeggioADPCMLoops(int arpNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times)
+{
+	instMan_->setArpeggioADPCMLoops(arpNum, std::move(begins), std::move(ends), std::move(times));
+}
+
+void BambooTracker::setArpeggioADPCMRelease(int arpNum, ReleaseType type, int begin)
+{
+	instMan_->setArpeggioADPCMRelease(arpNum, type, begin);
+}
+
+void BambooTracker::setInstrumentADPCMArpeggio(int instNum, int arpNum)
+{
+	instMan_->setInstrumentADPCMArpeggio(instNum, arpNum);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+void BambooTracker::setInstrumentADPCMArpeggioEnabled(int instNum, bool enabled)
+{
+	instMan_->setInstrumentADPCMArpeggioEnabled(instNum, enabled);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+std::vector<int> BambooTracker::getArpeggioADPCMUsers(int arpNum) const
+{
+	return instMan_->getArpeggioADPCMUsers(arpNum);
+}
+
+void BambooTracker::setPitchADPCMType(int ptNum, SequenceType type)
+{
+	instMan_->setPitchADPCMType(ptNum, type);
+}
+
+void BambooTracker::addPitchADPCMSequenceCommand(int ptNum, int type, int data)
+{
+	instMan_->addPitchADPCMSequenceCommand(ptNum, type, data);
+}
+
+void BambooTracker::removePitchADPCMSequenceCommand(int ptNum)
+{
+	instMan_->removePitchADPCMSequenceCommand(ptNum);
+}
+
+void BambooTracker::setPitchADPCMSequenceCommand(int ptNum, int cnt, int type, int data)
+{
+	instMan_->setPitchADPCMSequenceCommand(ptNum, cnt, type, data);
+}
+
+void BambooTracker::setPitchADPCMLoops(int ptNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times)
+{
+	instMan_->setPitchADPCMLoops(ptNum, std::move(begins), std::move(ends), std::move(times));
+}
+
+void BambooTracker::setPitchADPCMRelease(int ptNum, ReleaseType type, int begin)
+{
+	instMan_->setPitchADPCMRelease(ptNum, type, begin);
+}
+
+void BambooTracker::setInstrumentADPCMPitch(int instNum, int ptNum)
+{
+	instMan_->setInstrumentADPCMPitch(instNum, ptNum);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+void BambooTracker::setInstrumentADPCMPitchEnabled(int instNum, bool enabled)
+{
+	instMan_->setInstrumentADPCMPitchEnabled(instNum, enabled);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+std::vector<int> BambooTracker::getPitchADPCMUsers(int ptNum) const
+{
+	return instMan_->getPitchADPCMUsers(ptNum);
+}
+
+//--- Drumkit
+void BambooTracker::setInstrumentDrumkitSample(int instNum, int key, int sampNum)
+{
+	instMan_->setInstrumentDrumkitSamples(instNum, key, sampNum);
+	opnaCtrl_->updateInstrumentDrumkit(instNum, key);
+}
+
+void BambooTracker::setInstrumentDrumkitSampleEnabled(int instNum, int key, bool enabled)
+{
+	instMan_->setInstrumentDrumkitSamplesEnabled(instNum, key, enabled);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+void BambooTracker::setInstrumentDrumkitPitch(int instNum, int key, int pitch)
+{
+	instMan_->setInstrumentDrumkitPitch(instNum, key, pitch);
+	opnaCtrl_->updateInstrumentDrumkit(instNum, key);
+}
+
 /********** Song edit **********/
 int BambooTracker::getCurrentSongNumber() const
 {
@@ -605,6 +877,8 @@ void BambooTracker::setCurrentSongNumber(int num)
 	curTrackNum_ = 0;
 	curOrderNum_ = 0;
 	curStepNum_ = 0;
+	mkOrder_ = -1;
+	mkStep_ = -1;
 
 	auto& song = mod_->getSong(curSongNum_);
 	songStyle_ = song.getStyle();
@@ -623,13 +897,16 @@ void BambooTracker::setCurrentSongNumber(int num)
 	tickCounter_->setGrooveTrigger(song.isUsedTempo() ? GrooveTrigger::Invalid
 													  : GrooveTrigger::ValidByGlobal);
 
-	size_t fmch = getFMChannelCount(songStyle_.type);
-	muteStateFM_ = std::vector<bool>(fmch, false);
-	for (int i = 0; i < static_cast<int>(fmch); ++i) opnaCtrl_->setMuteFMState(i, false);
-	muteStateSSG_ = std::vector<bool>(3, false);
-	for (int i = 0; i < 3; ++i) opnaCtrl_->setMuteSSGState(i, false);
-	muteStateDrum_ = std::vector<bool>(6, false);
-	for (int i = 0; i < 6; ++i) opnaCtrl_->setMuteDrumState(i, false);
+	std::unordered_map<SoundSource, int> pairs = {
+		{ SoundSource::FM, getFMChannelCount(songStyle_.type) },
+		{ SoundSource::SSG, 3 },
+		{ SoundSource::DRUM, 6 },
+		{ SoundSource::ADPCM, 1 },
+	};
+	for (auto& pair : pairs) {
+		muteState_[pair.first] = std::vector<bool>(pair.second, false);
+		for (int i = 0; i < pair.second; ++i) opnaCtrl_->setMuteState(pair.first, i, false);
+	}
 }
 
 /********** Order edit **********/
@@ -686,37 +963,42 @@ bool BambooTracker::isJamMode() const
 	return jamMan_->isJamMode();
 }
 
-void BambooTracker::jamKeyOn(JamKey key)
+void BambooTracker::jamKeyOn(JamKey key, bool volumeSet)
 {
-	int keyNum = octaveAndNoteToNoteNumber(octave_, JamManager::jamKeyToNote(key));
+	int keyNum = octaveAndNoteToNoteNumber(curOctave_, JamManager::jamKeyToNote(key));
 	const TrackAttribute& attrib = songStyle_.trackAttribs[static_cast<size_t>(curTrackNum_)];
-	funcJamKeyOn(key, keyNum, attrib);
+	funcJamKeyOn(key, keyNum, attrib, volumeSet);
 }
 
-void BambooTracker::jamKeyOn(int keyNum)
+void BambooTracker::jamKeyOn(int keyNum, bool volumeSet)
 {
 	const TrackAttribute& attrib = songStyle_.trackAttribs[static_cast<size_t>(curTrackNum_)];
-	funcJamKeyOn(JamKey::MidiKey, keyNum, attrib);
+	funcJamKeyOn(JamKey::MidiKey, keyNum, attrib, volumeSet);
 }
 
-void BambooTracker::jamKeyOnForced(JamKey key, SoundSource src)
+void BambooTracker::jamKeyOnForced(JamKey key, SoundSource src, bool volumeSet, std::shared_ptr<AbstractInstrument> inst)
 {
-	int keyNum = octaveAndNoteToNoteNumber(octave_, JamManager::jamKeyToNote(key));
+	int keyNum = octaveAndNoteToNoteNumber(curOctave_, JamManager::jamKeyToNote(key));
 	auto it = std::find_if(songStyle_.trackAttribs.begin(), songStyle_.trackAttribs.end(),
 						   [src](TrackAttribute& attrib) { return attrib.source == src; });
-	funcJamKeyOn(key, keyNum, *it);
+	funcJamKeyOn(key, keyNum, *it, volumeSet, inst);
 }
 
-void BambooTracker::jamKeyOnForced(int keyNum, SoundSource src)
+void BambooTracker::jamKeyOnForced(int keyNum, SoundSource src, bool volumeSet, std::shared_ptr<AbstractInstrument> inst)
 {
 	auto it = std::find_if(songStyle_.trackAttribs.begin(), songStyle_.trackAttribs.end(),
 						   [src](TrackAttribute& attrib) { return attrib.source == src; });
-	funcJamKeyOn(JamKey::MidiKey, keyNum, *it);
+	funcJamKeyOn(JamKey::MidiKey, keyNum, *it, volumeSet, inst);
 }
 
-void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& attrib)
+void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& attrib, bool volumeSet,
+								 std::shared_ptr<AbstractInstrument> inst)
 {
+	if (playback_->isPlayingStep()) playback_->stopPlaySong();	// Reset
+
 	if (attrib.source == SoundSource::DRUM) {
+		if (volumeSet)
+			opnaCtrl_->setVolumeDrum(attrib.channelInSource, std::min(curVolume_, 0x1f));
 		opnaCtrl_->setKeyOnFlagDrum(attrib.channelInSource);
 		opnaCtrl_->updateRegisterStates();
 	}
@@ -739,12 +1021,17 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 			case SoundSource::SSG:
 				opnaCtrl_->keyOffSSG(offData.channelInSource, true);
 				break;
+			case SoundSource::ADPCM:
+				opnaCtrl_->keyOffADPCM(true);
+				break;
 			default:
 				break;
 			}
 		}
 
-		std::shared_ptr<AbstractInstrument> tmpInst = instMan_->getInstrumentSharedPtr(curInstNum_);
+		if (!inst) {	// Use current instrument if not specified
+			inst = instMan_->getInstrumentSharedPtr(curInstNum_);
+		}
 		JamKeyData& onData = list.front();
 
 		Note note;
@@ -756,7 +1043,7 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 		}
 		else {
 			note = JamManager::jamKeyToNote(onData.key);
-			octave = JamManager::calcOctave(octave_, onData.key);
+			octave = JamManager::calcOctave(curOctave_, onData.key);
 			if (octave > 7) {	// Tone range check
 				octave = 7;
 				note = Note::B;
@@ -766,8 +1053,14 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 
 		switch (onData.source) {
 		case SoundSource::FM:
-			if (auto fm = std::dynamic_pointer_cast<InstrumentFM>(tmpInst))
+			if (auto fm = std::dynamic_pointer_cast<InstrumentFM>(inst))
 				opnaCtrl_->setInstrumentFM(onData.channelInSource, fm);
+			if (volumeSet) {
+				int vol;
+				if (volFMReversed_) vol = (curVolume_ < 0x80) ? (0x7f - curVolume_) : 0;
+				else vol = std::min(curVolume_, 0x7f);
+				opnaCtrl_->setVolumeFM(onData.channelInSource, vol);
+			}
 			if (songStyle_.type == SongType::FM3chExpanded && onData.channelInSource == 2) {
 				opnaCtrl_->keyOnFM(2, note, octave, pitch, true);
 				opnaCtrl_->keyOnFM(6, note, octave, pitch, true);
@@ -779,10 +1072,20 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 			}
 			break;
 		case SoundSource::SSG:
-			if (auto ssg = std::dynamic_pointer_cast<InstrumentSSG>(tmpInst))
+			if (auto ssg = std::dynamic_pointer_cast<InstrumentSSG>(inst))
 				opnaCtrl_->setInstrumentSSG(onData.channelInSource, ssg);
+			if (volumeSet)
+				opnaCtrl_->setVolumeSSG(onData.channelInSource, std::min(curVolume_, 0xf));
 			opnaCtrl_->keyOnSSG(onData.channelInSource, note, octave, pitch, true);
 			break;
+		case SoundSource::ADPCM:
+			if (auto adpcm = std::dynamic_pointer_cast<InstrumentADPCM>(inst))
+				opnaCtrl_->setInstrumentADPCM(adpcm);
+			else if (auto kit = std::dynamic_pointer_cast<InstrumentDrumkit>(inst))
+				opnaCtrl_->setInstrumentDrumkit(kit);
+			if (volumeSet) opnaCtrl_->setVolumeADPCM(curVolume_);
+			opnaCtrl_->keyOnADPCM(note, octave, pitch, true);
+			break;
 		default:
 			break;
 		}
@@ -791,7 +1094,7 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 
 void BambooTracker::jamKeyOff(JamKey key)
 {
-	int keyNum = octaveAndNoteToNoteNumber(octave_, JamManager::jamKeyToNote(key));
+	int keyNum = octaveAndNoteToNoteNumber(curOctave_, JamManager::jamKeyToNote(key));
 	const TrackAttribute& attrib = songStyle_.trackAttribs[static_cast<size_t>(curTrackNum_)];
 	funcJamKeyOff(key, keyNum, attrib);
 }
@@ -804,7 +1107,7 @@ void BambooTracker::jamKeyOff(int keyNum)
 
 void BambooTracker::jamKeyOffForced(JamKey key, SoundSource src)
 {
-	int keyNum = octaveAndNoteToNoteNumber(octave_, JamManager::jamKeyToNote(key));
+	int keyNum = octaveAndNoteToNoteNumber(curOctave_, JamManager::jamKeyToNote(key));
 	auto it = std::find_if(songStyle_.trackAttribs.begin(), songStyle_.trackAttribs.end(),
 						   [src](TrackAttribute& attrib) { return attrib.source == src; });
 	funcJamKeyOff(key, keyNum, *it);
@@ -842,6 +1145,9 @@ void BambooTracker::funcJamKeyOff(JamKey key, int keyNum, const TrackAttribute&
 			case SoundSource::SSG:
 				opnaCtrl_->keyOffSSG(data.channelInSource, true);
 				break;
+			case SoundSource::ADPCM:
+				opnaCtrl_->keyOffADPCM(true);
+				break;
 			default:
 				break;
 			}
@@ -849,6 +1155,32 @@ void BambooTracker::funcJamKeyOff(JamKey key, int keyNum, const TrackAttribute&
 	}
 }
 
+std::vector<std::vector<size_t>> BambooTracker::assignADPCMBeforeForcedJamKeyOn(std::shared_ptr<AbstractInstrument> inst)
+{
+	switch (inst->getType()) {
+	case InstrumentType::ADPCM:
+	{
+		opnaCtrl_->clearSamplesADPCM();
+		return { opnaCtrl_->storeSampleADPCM(
+						std::dynamic_pointer_cast<InstrumentADPCM>(inst)->getRawSample()) };
+	}
+	case InstrumentType::Drumkit:
+	{
+		opnaCtrl_->clearSamplesADPCM();
+		std::vector<std::vector<size_t>> addrs;
+		auto kit = std::dynamic_pointer_cast<InstrumentDrumkit>(inst);
+		for (const int& key : kit->getAssignedKeys()) {
+			int samp = kit->getSampleNumber(key);
+			if (addrs.size() <= static_cast<size_t>(samp)) addrs.resize(samp + 1);
+			if (addrs[samp].empty()) addrs[samp] = opnaCtrl_->storeSampleADPCM(kit->getRawSample(key));
+		}
+		return addrs;
+	}
+	default:
+		return {};
+	}
+}
+
 /********** Play song **********/
 void BambooTracker::startPlaySong()
 {
@@ -879,17 +1211,41 @@ void BambooTracker::startPlayPattern()
 
 void BambooTracker::startPlayFromCurrentStep()
 {
-	playback_->startPlayFromCurrentStep(curOrderNum_, curStepNum_);
+	playback_->startPlayFromPosition(curOrderNum_, curStepNum_);
 	startPlay();
 }
 
+bool BambooTracker::startPlayFromMarker()
+{
+	Song& song = mod_->getSong(curSongNum_);
+	if (mkOrder_ != -1 && mkOrder_ < static_cast<int>(song.getOrderSize())
+			&& mkStep_ != -1 && mkStep_ < static_cast<int>(song.getPatternSizeFromOrderNumber(mkOrder_))) {
+		playback_->startPlayFromPosition(mkOrder_, mkStep_);
+		startPlay();
+		return true;
+	}
+	return false;
+}
+
+void BambooTracker::playStep()
+{
+	playback_->playStep(curOrderNum_, curStepNum_);
+	for (auto& pair : muteState_) {
+		for (size_t i = 0; i < pair.second.size(); ++i) {
+			opnaCtrl_->setMuteState(pair.first, static_cast<int>(i), pair.second[i]);
+		}
+	}
+}
+
 void BambooTracker::startPlay()
 {
 	jamMan_->polyphonic(false);
 
-	for (size_t i = 0; i < muteStateFM_.size(); ++i) opnaCtrl_->setMuteFMState(static_cast<int>(i), muteStateFM_[i]);
-	for (size_t i = 0; i < muteStateSSG_.size(); ++i) opnaCtrl_->setMuteSSGState(static_cast<int>(i), muteStateSSG_[i]);
-	for (size_t i = 0; i < muteStateDrum_.size(); ++i) opnaCtrl_->setMuteDrumState(static_cast<int>(i), muteStateDrum_[i]);
+	for (auto& pair : muteState_) {
+		for (size_t i = 0; i < pair.second.size(); ++i) {
+			opnaCtrl_->setMuteState(pair.first, static_cast<int>(i), pair.second[i]);
+		}
+	}
 }
 
 void BambooTracker::stopPlaySong()
@@ -897,9 +1253,11 @@ void BambooTracker::stopPlaySong()
 	playback_->stopPlaySong();
 	jamMan_->polyphonic(true);
 
-	for (size_t i = 0; i < muteStateFM_.size(); ++i) opnaCtrl_->setMuteFMState(static_cast<int>(i), false);
-	for (size_t i = 0; i < muteStateSSG_.size(); ++i) opnaCtrl_->setMuteSSGState(static_cast<int>(i), false);
-	for (size_t i = 0; i < muteStateDrum_.size(); ++i) opnaCtrl_->setMuteDrumState(static_cast<int>(i), false);
+	for (auto& pair : muteState_) {
+		for (size_t i = 0; i < pair.second.size(); ++i) {
+			opnaCtrl_->setMuteState(pair.first, static_cast<int>(i), false);
+		}
+	}
 }
 
 bool BambooTracker::isPlaySong() const
@@ -907,41 +1265,17 @@ bool BambooTracker::isPlaySong() const
 	return playback_->isPlaySong();
 }
 
-PlaybackState BambooTracker::getPlaybackState() const
-{
-	return playback_->getPlaybackState();
-}
-
 void BambooTracker::setTrackMuteState(int trackNum, bool isMute)
 {
 	auto& ta = songStyle_.trackAttribs[static_cast<size_t>(trackNum)];
-	int ch = ta.channelInSource;
-	switch (ta.source) {
-	case SoundSource::FM:
-		muteStateFM_.at(static_cast<size_t>(ch)) = isMute;
-		if (isPlaySong()) opnaCtrl_->setMuteFMState(ch, isMute);
-		break;
-	case SoundSource::SSG:
-		muteStateSSG_.at(static_cast<size_t>(ch)) = isMute;
-		if (isPlaySong()) opnaCtrl_->setMuteSSGState(ch, isMute);
-		break;
-	case SoundSource::DRUM:
-		muteStateDrum_.at(static_cast<size_t>(ch)) = isMute;
-		if (isPlaySong()) opnaCtrl_->setMuteDrumState(ch, isMute);
-		break;
-	}
+	muteState_.at(ta.source).at(static_cast<size_t>(ta.channelInSource)) = isMute;
+	if (isPlaySong()) opnaCtrl_->setMuteState(ta.source, ta.channelInSource, isMute);
 }
 
 bool BambooTracker::isMute(int trackNum)
 {
 	auto& ta = songStyle_.trackAttribs[static_cast<size_t>(trackNum)];
-	size_t ch = static_cast<size_t>(ta.channelInSource);
-	switch (ta.source) {
-	case SoundSource::FM:	return muteStateFM_.at(ch);
-	case SoundSource::SSG:	return muteStateSSG_.at(ch);
-	case SoundSource::DRUM:	return muteStateDrum_.at(ch);
-	default:	return false;	// Dummy
-	}
+	return muteState_.at(ta.source).at(ta.channelInSource);
 }
 
 void BambooTracker::setFollowPlay(bool isFollowed)
@@ -971,11 +1305,33 @@ int BambooTracker::getPlayingStepNumber() const
 	return playback_->getPlayingStepNumber();
 }
 
+void BambooTracker::setMarker(int order, int step)
+{
+	if (mkOrder_ == order && mkStep_ == step) {
+		mkOrder_ = -1;
+		mkStep_ = -1;
+	}
+	else {
+		mkOrder_ = order;
+		mkStep_ = step;
+	}
+}
+
+int BambooTracker::getMarkerOrder() const
+{
+	return mkOrder_;
+}
+
+int BambooTracker::getMarkerStep() const
+{
+	return mkStep_;
+}
+
 /********** Export **********/
-bool BambooTracker::exportToWav(BinaryContainer& container, int rate, int loopCnt, std::function<bool()> bar)
+bool BambooTracker::exportToWav(WavContainer& container, int loopCnt, std::function<bool()> bar)
 {
 	int tmpRate = opnaCtrl_->getRate();
-	opnaCtrl_->setRate(rate);
+	opnaCtrl_->setRate(static_cast<int>(container.getSampleRate()));
 	size_t sampCnt = static_cast<size_t>(opnaCtrl_->getRate() * opnaCtrl_->getDuration() / 1000);
 	size_t intrCnt = static_cast<size_t>(opnaCtrl_->getRate()) / mod_->getTickFrequency();
 	size_t intrCntRest = 0;
@@ -1028,13 +1384,9 @@ bool BambooTracker::exportToWav(BinaryContainer& container, int rate, int loopCn
 	isFollowPlay_ = tmpFollow;
 	opnaCtrl_->setRate(tmpRate);
 
-	try {
-		ExportHandler::writeWave(container, exCntr->getStream(), static_cast<uint32_t>(rate));
-		return true;
-	}
-	catch (...) {
-		throw;
-	}
+	container.storeSample(exCntr->getStream());
+
+	return true;
 }
 
 bool BambooTracker::exportToVgm(BinaryContainer& container, int target, bool gd3TagEnabled,
@@ -1056,8 +1408,24 @@ bool BambooTracker::exportToVgm(BinaryContainer& container, int target, bool gd3
 	bool tmpFollow = std::exchange(isFollowPlay_, false);
 	uint32_t loopPoint = 0;
 	uint32_t loopPointSamples = 0;
+
 	std::shared_ptr<chip::VgmExportContainer> exCntr
 			= std::make_shared<chip::VgmExportContainer>(target, mod_->getTickFrequency());
+
+	// Set ADPCM
+	opnaCtrl_->clearSamplesADPCM();
+	std::vector<uint8_t> rom;
+	for (auto sampNum : instMan_->getSampleADPCMValidIndices()) {
+		std::vector<uint8_t> sample = instMan_->getSampleADPCMRawSample(sampNum);
+		std::vector<size_t> addresses = opnaCtrl_->storeSampleADPCM(sample);
+		instMan_->setSampleADPCMStartAddress(sampNum, addresses[0]);
+		instMan_->setSampleADPCMStopAddress(sampNum, addresses[1]);
+
+		rom.resize((addresses[1] + 1) << 5);
+		std::copy(sample.begin(), sample.end(), rom.begin() + static_cast<int>(addresses[0] << 5));
+	}
+	exCntr->setDataBlock(std::move(rom));
+
 	opnaCtrl_->setExportContainer(exCntr);
 	startPlayFromStart();
 	exCntr->forceMoveLoopPoint();
@@ -1122,6 +1490,7 @@ bool BambooTracker::exportToS98(BinaryContainer& container, int target, bool tag
 	std::shared_ptr<chip::S98ExportContainer> exCntr = std::make_shared<chip::S98ExportContainer>(target);
 	opnaCtrl_->setExportContainer(exCntr);
 	startPlayFromStart();
+	assignSampleADPCMRawSamples();
 	exCntr->forceMoveLoopPoint();
 
 	while (true) {
@@ -1219,7 +1588,7 @@ RealChipInterface BambooTracker::getRealChipinterface() const
 int BambooTracker::streamCountUp()
 {
 	int state = playback_->streamCountUp();
-	if (!state && isFollowPlay_) {	// Step
+	if (!state && isFollowPlay_ && !playback_->isPlayingStep()) {	// Step
 		int odr = playback_->getPlayingOrderNumber();
 		if (odr >= 0) {
 			curOrderNum_ = odr;
@@ -1483,6 +1852,11 @@ void BambooTracker::replaceDuplicateInstrumentsInPatterns(std::vector<std::vecto
 	mod_->replaceDuplicateInstrumentsInPatterns(map);
 }
 
+void BambooTracker::clearUnusedADPCMSamples()
+{
+	instMan_->clearUnusedSamplesADPCM();
+}
+
 /*----- Song -----*/
 void BambooTracker::setSongTitle(int songNum, std::string title)
 {
@@ -1584,6 +1958,76 @@ size_t BambooTracker::getAllStepCount(int songNum, int loopCnt) const
 	}
 }
 
+void BambooTracker::transposeSong(int songNum, int seminotes, std::vector<int> excludeInsts)
+{
+	mod_->getSong(songNum).transpose(seminotes, excludeInsts);
+}
+
+void BambooTracker::swapTracks(int songNum, int track1, int track2)
+{
+	mod_->getSong(songNum).swapTracks(track1, track2);
+}
+
+/*----- Bookmark -----*/
+void BambooTracker::addBookmark(int songNum, std::string name, int order, int step)
+{
+	mod_->getSong(songNum).addBookmark(name, order, step);
+}
+
+void BambooTracker::changeBookmark(int songNum, int i, std::string name, int order, int step)
+{
+	mod_->getSong(songNum).changeBookmark(i, name, order, step);
+}
+
+void BambooTracker::removeBookmark(int songNum, int i)
+{
+	mod_->getSong(songNum).removeBookmark(i);
+}
+
+void BambooTracker::clearBookmark(int songNum)
+{
+	mod_->getSong(songNum).clearBookmark();
+}
+
+void BambooTracker::swapBookmarks(int songNum, int a, int b)
+{
+	mod_->getSong(songNum).swapBookmarks(a, b);
+}
+
+void BambooTracker::sortBookmarkByPosition(int songNum)
+{
+	mod_->getSong(songNum).sortBookmarkByPosition();
+}
+void BambooTracker::sortBookmarkByName(int songNum)
+{
+	mod_->getSong(songNum).sortBookmarkByName();
+}
+
+Bookmark BambooTracker::getBookmark(int songNum, int i) const
+{
+	return mod_->getSong(songNum).getBookmark(i);
+}
+
+std::vector<int> BambooTracker::findBookmarks(int songNum, int order, int step)
+{
+	return mod_->getSong(songNum).findBookmarks(order, step);
+}
+
+Bookmark BambooTracker::getPreviousBookmark(int songNum, int order, int step)
+{
+	return mod_->getSong(songNum).getPreviousBookmark(order, step);
+}
+
+Bookmark BambooTracker::getNextBookmark(int songNum, int order, int step)
+{
+	return mod_->getSong(songNum).getNextBookmark(order, step);
+}
+
+size_t BambooTracker::getBookmarkSize(int songNum) const
+{
+	return mod_->getSong(songNum).getBookmarkSize();
+}
+
 /*----- Track -----*/
 void BambooTracker::setEffectDisplayWidth(int songNum, int trackNum, size_t w)
 {
@@ -1674,17 +2118,19 @@ int BambooTracker::getStepNoteNumber(int songNum, int trackNum, int orderNum, in
 			.getStep(stepNum).getNoteNumber();
 }
 
-void BambooTracker::setStepNote(int songNum, int trackNum, int orderNum, int stepNum, int octave, Note note, bool autosetInst)
+void BambooTracker::setStepNote(int songNum, int trackNum, int orderNum, int stepNum, int octave, Note note, bool instMask, bool volMask)
 {
 	int nn = octaveAndNoteToNoteNumber(octave, note);
+	SoundSource src = songStyle_.trackAttribs.at(static_cast<size_t>(trackNum)).source;
 
 	int in = -1;
-	if (autosetInst && curInstNum_ != -1
-			&& (songStyle_.trackAttribs.at(static_cast<size_t>(trackNum)).source
-				== instMan_->getInstrumentSharedPtr(curInstNum_)->getSoundSource()))
+	if (!instMask && curInstNum_ != -1
+			&& (src == instMan_->getInstrumentSharedPtr(curInstNum_)->getSoundSource())) {
 		in = curInstNum_;
+	}
+	bool fmReversed = (volFMReversed_ && src == SoundSource::FM);
 
-	comMan_.invoke(std::make_unique<SetKeyOnToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum, nn, autosetInst, in));
+	comMan_.invoke(std::make_unique<SetKeyOnToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum, nn, instMask, in, volMask, curVolume_, fmReversed));
 }
 
 void BambooTracker::setStepKeyOff(int songNum, int trackNum, int orderNum, int stepNum)
@@ -1724,9 +2170,14 @@ int BambooTracker::getStepVolume(int songNum, int trackNum, int orderNum, int st
 			.getStep(stepNum).getVolume();
 }
 
-void BambooTracker::setStepVolumeDigit(int songNum, int trackNum, int orderNum, int stepNum, int volume, bool isFMReversed, bool secondEntry)
+int BambooTracker::setStepVolumeDigit(int songNum, int trackNum, int orderNum, int stepNum, int volume, bool secondEntry)
 {	
-	comMan_.invoke(std::make_unique<SetVolumeToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum, volume, isFMReversed, secondEntry));
+	bool fmReversed = (volFMReversed_
+					   && songStyle_.trackAttribs.at(static_cast<size_t>(trackNum)).source == SoundSource::FM);
+	comMan_.invoke(std::make_unique<SetVolumeToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum, volume, fmReversed, secondEntry));
+	curVolume_ = mod_->getSong(songNum).getTrack(trackNum).getPatternFromOrderNumber(orderNum).getStep(stepNum).getVolume();
+	if (fmReversed && curVolume_ < 0x80) curVolume_ = 0x7f - curVolume_;
+	return curVolume_;
 }
 
 void BambooTracker::eraseStepVolume(int songNum, int trackNum, int orderNum, int stepNum)
@@ -1806,6 +2257,16 @@ void BambooTracker::pasteOverwritePatternCells(int songNum, int beginTrack, int
 					   mod_, songNum, beginTrack, beginColmn, beginOrder, beginStep, std::move(d)));
 }
 
+void BambooTracker::pasteInsertPatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder,
+											int beginStep, std::vector<std::vector<std::string>> cells)
+{
+	std::vector<std::vector<std::string>> d
+			= arrangePatternDataCells(songNum, beginTrack, beginColmn, beginOrder, beginStep, std::move(cells));
+
+	comMan_.invoke(std::make_unique<PasteInsertCopiedDataToPatternCommand>(
+					   mod_, songNum, beginTrack, beginColmn, beginOrder, beginStep, std::move(d)));
+}
+
 std::vector<std::vector<std::string>> BambooTracker::arrangePatternDataCells(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 																			 std::vector<std::vector<std::string>> cells)
 {
@@ -1834,31 +2295,19 @@ void BambooTracker::erasePatternCells(int songNum, int beginTrack, int beginColm
 					   mod_, songNum, beginTrack, beginColmn, beginOrder, beginStep, endTrack, endColmn, endStep));
 }
 
-void BambooTracker::increaseNoteKeyInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-											 int endTrack, int endStep)
+void BambooTracker::transposeNoteInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
+										   int endTrack, int endStep, int seminote)
 {
-	comMan_.invoke(std::make_unique<IncreaseNoteKeyInPatternCommand>(
-					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep));
-}
-void BambooTracker::decreaseNoteKeyInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-											 int endTrack, int endStep)
-{
-	comMan_.invoke(std::make_unique<DecreaseNoteKeyInPatternCommand>(
-					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep));
+	comMan_.invoke(std::make_unique<TransposeNoteInPatternCommand>(
+					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep, seminote));
 }
 
-void BambooTracker::increaseNoteOctaveInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-												int endTrack, int endStep)
+void BambooTracker::changeValuesInPattern(int songNum, int beginTrack, int beginColumn, int beginOrder,
+										  int beginStep, int endTrack, int endColumn, int endStep, int value)
 {
-	comMan_.invoke(std::make_unique<IncreaseNoteOctaveInPatternCommand>(
-					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep));
-}
-
-void BambooTracker::decreaseNoteOctaveInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-												int endTrack, int endStep)
-{
-	comMan_.invoke(std::make_unique<DecreaseNoteOctaveInPatternCommand>(
-					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep));
+	comMan_.invoke(std::make_unique<ChangeValuesInPatternCommand>(
+					   mod_, songNum, beginTrack, beginColumn, beginOrder, beginStep,
+					   endTrack, endColumn, endStep, value, volFMReversed_));
 }
 
 void BambooTracker::expandPattern(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
