@@ -5,6 +5,7 @@
 #include <memory>
 #include <vector>
 #include <functional>
+#include <unordered_map>
 #include "configuration.hpp"
 #include "opna_controller.hpp"
 #include "jam_manager.hpp"
@@ -21,6 +22,8 @@
 #include "effect.hpp"
 #include "playback.hpp"
 #include "binary_container.hpp"
+#include "wav_container.hpp"
+#include "enum_hash.hpp"
 #include "misc.hpp"
 
 class AbstractBank;
@@ -33,10 +36,14 @@ public:
 	// Change confuguration
 	void changeConfiguration(std::weak_ptr<Configuration> config);
 
-	// Change octave
+	// Current octave
 	void setCurrentOctave(int octave);
 	int getCurrentOctave() const;
 
+	// Current volume
+	void setCurrentVolume(int volume);
+	int getCurrentVolume() const;
+
 	// Current track
 	void setCurrentTrack(int num);
 	TrackAttribute getCurrentTrackAttribute() const;
@@ -46,11 +53,12 @@ public:
 	int getCurrentInstrumentNumber() const;
 
 	// Instrument edit
-	void addInstrument(int num, std::string name);
+	void addInstrument(int num, InstrumentType type, std::string name);
 	void removeInstrument(int num);
 	std::unique_ptr<AbstractInstrument> getInstrument(int num);
 	void cloneInstrument(int num, int refNum);
 	void deepCloneInstrument(int num, int refNum);
+	void swapInstruments(int a, int b, bool patternChange);
 	void loadInstrument(BinaryContainer& container, std::string path, int instNum);
 	void saveInstrument(BinaryContainer& container, int instNum);
 	void importInstrument(const AbstractBank &bank, size_t index, int instNum);
@@ -107,14 +115,14 @@ public:
 	void setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorType op, bool enabled);
 
 	//--- SSG
-	void addWaveFormSSGSequenceCommand(int wfNum, int type, int data);
-	void removeWaveFormSSGSequenceCommand(int wfNum);
-	void setWaveFormSSGSequenceCommand(int wfNum, int cnt, int type, int data);
-	void setWaveFormSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
-	void setWaveFormSSGRelease(int wfNum, ReleaseType type, int begin);
-	void setInstrumentSSGWaveForm(int instNum, int wfNum);
-	void setInstrumentSSGWaveFormEnabled(int instNum, bool enabled);
-	std::vector<int> getWaveFormSSGUsers(int wfNum) const;
+	void addWaveformSSGSequenceCommand(int wfNum, int type, int data);
+	void removeWaveformSSGSequenceCommand(int wfNum);
+	void setWaveformSSGSequenceCommand(int wfNum, int cnt, int type, int data);
+	void setWaveformSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	void setWaveformSSGRelease(int wfNum, ReleaseType type, int begin);
+	void setInstrumentSSGWaveform(int instNum, int wfNum);
+	void setInstrumentSSGWaveformEnabled(int instNum, bool enabled);
+	std::vector<int> getWaveformSSGUsers(int wfNum) const;
 
 	void addToneNoiseSSGSequenceCommand(int tnNum, int type, int data);
 	void removeToneNoiseSSGSequenceCommand(int tnNum);
@@ -154,6 +162,59 @@ public:
 	void setInstrumentSSGPitchEnabled(int instNum, bool enabled);
 	std::vector<int> getPitchSSGUsers(int ptNum) const;
 
+	//--- ADPCM
+	size_t getADPCMLimit() const;
+	size_t getADPCMStoredSize() const;
+
+	void setSampleADPCMRootKeyNumber(int sampNum, int n);
+	int getSampleADPCMRootKeyNumber(int sampNum) const;
+	void setSampleADPCMRootDeltaN(int sampNum, int dn);
+	int getSampleADPCMRootDeltaN(int sampNum) const;
+	void setSampleADPCMRepeatEnabled(int sampNum, bool enabled);
+	bool getSampleADPCMRepeatEnabled(int sampNum) const;
+	void storeSampleADPCMRawSample(int sampNum, std::vector<uint8_t> sample);
+	std::vector<uint8_t> getSampleADPCMRawSample(int sampNum) const;
+	void clearSampleADPCMRawSample(int sampNum);
+	void assignSampleADPCMRawSamples();
+	size_t getSampleADPCMStartAddress(int sampNum) const;
+	size_t getSampleADPCMStopAddress(int sampNum) const;
+	void setInstrumentADPCMSample(int instNum, int sampNum);
+	std::vector<int> getSampleADPCMUsers(int sampNum) const;
+
+	void addEnvelopeADPCMSequenceCommand(int envNum, int type, int data);
+	void removeEnvelopeADPCMSequenceCommand(int envNum);
+	void setEnvelopeADPCMSequenceCommand(int envNum, int cnt, int type, int data);
+	void setEnvelopeADPCMLoops(int envNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	void setEnvelopeADPCMRelease(int envNum, ReleaseType type, int begin);
+	void setInstrumentADPCMEnvelope(int instNum, int envNum);
+	void setInstrumentADPCMEnvelopeEnabled(int instNum, bool enabled);
+	std::vector<int> getEnvelopeADPCMUsers(int envNum) const;
+
+	void setArpeggioADPCMType(int arpNum, SequenceType type);
+	void addArpeggioADPCMSequenceCommand(int arpNum, int type, int data);
+	void removeArpeggioADPCMSequenceCommand(int arpNum);
+	void setArpeggioADPCMSequenceCommand(int arpNum, int cnt, int type, int data);
+	void setArpeggioADPCMLoops(int arpNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	void setArpeggioADPCMRelease(int arpNum, ReleaseType type, int begin);
+	void setInstrumentADPCMArpeggio(int instNum, int arpNum);
+	void setInstrumentADPCMArpeggioEnabled(int instNum, bool enabled);
+	std::vector<int> getArpeggioADPCMUsers(int arpNum) const;
+
+	void setPitchADPCMType(int ptNum, SequenceType type);
+	void addPitchADPCMSequenceCommand(int ptNum, int type, int data);
+	void removePitchADPCMSequenceCommand(int ptNum);
+	void setPitchADPCMSequenceCommand(int ptNum, int cnt, int type, int data);
+	void setPitchADPCMLoops(int ptNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	void setPitchADPCMRelease(int ptNum, ReleaseType type, int begin);
+	void setInstrumentADPCMPitch(int instNum, int ptNum);
+	void setInstrumentADPCMPitchEnabled(int instNum, bool enabled);
+	std::vector<int> getPitchADPCMUsers(int ptNum) const;
+
+	//--- Drumkit
+	void setInstrumentDrumkitSample(int instNum, int key, int sampNum);
+	void setInstrumentDrumkitSampleEnabled(int instNum, int key, bool enabled);
+	void setInstrumentDrumkitPitch(int instNum, int key, int pitch);
+
 	// Song edit
 	void setCurrentSongNumber(int num);
 	int getCurrentSongNumber() const;
@@ -174,32 +235,37 @@ public:
 	// Jam mode
 	void toggleJamMode();
 	bool isJamMode() const;
-	void jamKeyOn(JamKey key);
-	void jamKeyOn(int keyNum);
+	void jamKeyOn(JamKey key, bool volumeSet);
+	void jamKeyOn(int keyNum, bool volumeSet);
 	void jamKeyOff(JamKey key);
 	void jamKeyOff(int keyNum);
-	void jamKeyOnForced(JamKey key, SoundSource src);
-	void jamKeyOnForced(int keyNum, SoundSource src);
+	void jamKeyOnForced(JamKey key, SoundSource src, bool volumeSet,  std::shared_ptr<AbstractInstrument> inst = nullptr);
+	void jamKeyOnForced(int keyNum, SoundSource src, bool volumeSet, std::shared_ptr<AbstractInstrument> inst = nullptr);
 	void jamKeyOffForced(JamKey key, SoundSource src);
 	void jamKeyOffForced(int keyNum, SoundSource src);
+	std::vector<std::vector<size_t>> assignADPCMBeforeForcedJamKeyOn(std::shared_ptr<AbstractInstrument> inst);
 
 	// Play song
 	void startPlaySong();
 	void startPlayFromStart();
 	void startPlayPattern();
 	void startPlayFromCurrentStep();
+	bool startPlayFromMarker();
+	void playStep();
 	void stopPlaySong();
 	bool isPlaySong() const;
-	PlaybackState getPlaybackState() const;
 	void setTrackMuteState(int trackNum, bool isMute);
 	bool isMute(int trackNum);
 	void setFollowPlay(bool isFollowed);
 	bool isFollowPlay() const;
 	int getPlayingOrderNumber() const;
 	int getPlayingStepNumber() const;
+	void setMarker(int order, int step);
+	int getMarkerOrder() const;
+	int getMarkerStep() const;
 
 	// Export
-	bool exportToWav(BinaryContainer& container, int rate, int loopCnt, std::function<bool()> bar);
+	bool exportToWav(WavContainer& container, int loopCnt, std::function<bool()> bar);
 	bool exportToVgm(BinaryContainer& container, int target, bool gd3TagEnabled,
 					 GD3Tag tag, std::function<bool()> bar);
 	bool exportToS98(BinaryContainer& container, int target, bool tagEnabled, S98Tag tag,
@@ -263,6 +329,7 @@ public:
 	std::vector<int> getGroove(int num) const;
 	void clearUnusedPatterns();
 	void replaceDuplicateInstrumentsInPatterns(std::vector<std::vector<int>> list);
+	void clearUnusedADPCMSamples();
 	/*----- Song -----*/
 	void setSongTitle(int songNum, std::string title);
 	std::string getSongTitle(int songNum) const;
@@ -279,6 +346,21 @@ public:
 	void addSong(SongType songType, std::string title);
 	void sortSongs(std::vector<int> numbers);
 	size_t getAllStepCount(int songNum, int loopCnt) const;
+	void transposeSong(int songNum, int seminotes, std::vector<int> excludeInsts);
+	void swapTracks(int songNum, int track1, int track2);
+	/*----- Bookmark -----*/
+	void addBookmark(int songNum, std::string name, int order, int step);
+	void changeBookmark(int songNum, int i, std::string name, int order, int step);
+	void removeBookmark(int songNum, int i);
+	void clearBookmark(int songNum);
+	void swapBookmarks(int songNum, int a, int b);
+	void sortBookmarkByPosition(int songNum);
+	void sortBookmarkByName(int songNum);
+	Bookmark getBookmark(int songNum, int i) const;
+	std::vector<int> findBookmarks(int songNum, int order, int step);
+	Bookmark getPreviousBookmark(int songNum, int order, int step);
+	Bookmark getNextBookmark(int songNum, int order, int step);
+	size_t getBookmarkSize(int songNum) const;
 	/*----- Track -----*/
 	void setEffectDisplayWidth(int songNum, int trackNum, size_t w);
 	size_t getEffectDisplayWidth(int songNum, int trackNum) const;
@@ -288,7 +370,7 @@ public:
 	void insertOrderBelow(int songNum, int orderNum);
 	void deleteOrder(int songNum, int orderNum);
 	void pasteOrderCells(int songNum, int beginTrack, int beginOrder,
-						   std::vector<std::vector<std::string>> cells);
+						 std::vector<std::vector<std::string>> cells);
 	void duplicateOrder(int songNum, int orderNum);
 	void MoveOrder(int songNum, int orderNum, bool isUp);
 	void clonePatterns(int songNum, int beginOrder, int beginTrack, int endOrder, int endTrack);
@@ -297,7 +379,7 @@ public:
 	bool canAddNewOrder(int songNum) const;
 	/*----- Pattern -----*/
 	int getStepNoteNumber(int songNum, int trackNum, int orderNum, int stepNum) const;
-	void setStepNote(int songNum, int trackNum, int orderNum, int stepNum, int octave, Note note, bool autosetInst);
+	void setStepNote(int songNum, int trackNum, int orderNum, int stepNum, int octave, Note note, bool instMask, bool volMask);
 	void setStepKeyOff(int songNum, int trackNum, int orderNum, int stepNum);
 	void setEchoBufferAccess(int songNum, int trackNum, int orderNum, int stepNum, int bufNum);
 	void eraseStepNote(int songNum, int trackNum, int orderNum, int stepNum);
@@ -305,7 +387,7 @@ public:
 	void setStepInstrumentDigit(int songNum, int trackNum, int orderNum, int stepNum, int instNum, bool secondEntry);
 	void eraseStepInstrument(int songNum, int trackNum, int orderNum, int stepNum);
 	int getStepVolume(int songNum, int trackNum, int orderNum, int stepNum) const;
-	void setStepVolumeDigit(int songNum, int trackNum, int orderNum, int stepNum, int volume, bool isFMReversed, bool secondEntry);
+	int setStepVolumeDigit(int songNum, int trackNum, int orderNum, int stepNum, int volume, bool secondEntry);
 	void eraseStepVolume(int songNum, int trackNum, int orderNum, int stepNum);
 	std::string getStepEffectID(int songNum, int trackNum, int orderNum, int stepNum, int n) const;
 	void setStepEffectIDCharacter(int songNum, int trackNum, int orderNum, int stepNum, int n, std::string id, bool fillValue00, bool secondEntry);
@@ -327,18 +409,16 @@ public:
 							  std::vector<std::vector<std::string>> cells);
 	void pasteOverwritePatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder,
 									int beginStep, std::vector<std::vector<std::string>> cells);
+	void pasteInsertPatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder,
+								 int beginStep, std::vector<std::vector<std::string>> cells);
 	std::vector<std::vector<std::string>> arrangePatternDataCells(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 																  std::vector<std::vector<std::string>> cells);
 	void erasePatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 						   int endTrack, int endColmn, int endStep);
-	void increaseNoteKeyInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-								  int endTrack, int endStep);
-	void decreaseNoteKeyInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-								  int endTrack, int endStep);
-	void increaseNoteOctaveInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-									 int endTrack, int endStep);
-	void decreaseNoteOctaveInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-									 int endTrack, int endStep);
+	void transposeNoteInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
+								int endTrack, int endStep, int seminote);
+	void changeValuesInPattern(int songNum, int beginTrack, int beginColumn, int beginOrder,
+							   int beginStep, int endTrack, int endColumn, int endStep, int value);
 	void expandPattern(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 					   int endTrack, int endColmn, int endStep);
 	void shrinkPattern(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
@@ -365,21 +445,26 @@ private:
 	std::shared_ptr<Module> mod_;
 
 	// Current status
-	int octave_;	// 0-7
+	int curOctave_;	// 0-7
 	int curSongNum_;
 	SongStyle songStyle_;
 	int curTrackNum_;
 	int curOrderNum_, curStepNum_;
 	///	-1: not set
 	int curInstNum_;
-	std::vector<bool> muteStateFM_, muteStateSSG_, muteStateDrum_;
+	int curVolume_;
+	bool volFMReversed_;
+	std::unordered_map<SoundSource, std::vector<bool>> muteState_;
+	int mkOrder_, mkStep_;
 
 	bool isFollowPlay_;
+	bool storeOnlyUsedSamples_;
 
 	static const uint32_t CHIP_CLOCK;
 
 	// Jam mode
-	void funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& attrib);
+	void funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& attrib, bool volumeSet,
+					  std::shared_ptr<AbstractInstrument> inst = nullptr);
 	void funcJamKeyOff(JamKey key, int keyNum, const TrackAttribute& attrib);
 
 	// Play song
