@@ -14,7 +14,7 @@ namespace chip
 	public:
 		// [rate]
 		// 0 = rate is 55466 (FM synthesis rate when clock is 3993600 * 2)
-		OPNA(Emu emu, int clock, int rate, size_t maxDuration,
+		OPNA(Emu emu, int clock, int rate, size_t maxDuration, size_t dramSize,
 			 std::unique_ptr<AbstractResampler> fmResampler = std::make_unique<LinearResampler>(),
 			 std::unique_ptr<AbstractResampler> ssgResampler = std::make_unique<LinearResampler>(),
 			 std::shared_ptr<ExportContainerInterface> exportContainer = nullptr);
@@ -30,6 +30,7 @@ namespace chip
 		bool isUsedSCCI() const;
 		void useC86CTL(C86ctlBase* base);
 		bool isUsedC86CTL() const;
+		size_t getDRAMSize() const;
 
 	private:
 		static size_t count_;
@@ -45,7 +46,7 @@ namespace chip
 		std::unique_ptr<C86ctlRealChip> c86ctlRC_;
 		std::unique_ptr<C86ctlGimic> c86ctlGm_;
 
-		static const double VOL_REDUC;
+		static constexpr double VOL_REDUC_ = 7.5;
 
 		enum SoundSource : int
 		{
