@@ -8,6 +8,7 @@ enum CommandId : int
 	ChangeInstrumentName	= 0x12,
 	CloneInstrument			= 0x13,
 	DeepCloneInstrument		= 0x14,
+	SwapInstruments			= 0x15,
 
 	// 0x2*, 0x3*: Pattern editor
 	SetKeyOnToStep						= 0x20,
@@ -26,10 +27,8 @@ enum CommandId : int
 	PasteCopiedDataToPattern			= 0x2d,
 	EraseCellsInPattern					= 0x2e,
 	PasteMixCopiedDataToPattern			= 0x2f,
-	IncreaseNoteKeyInPattern			= 0x30,
-	DecreaseNoteKeyInPattern			= 0x31,
-	IncreaseNoteOctaveInPattern			= 0x32,
-	DecreaseNoteOctaveInPattern			= 0x33,
+	TransposeNoteInPattern				= 0x30,
+	ChangeValuesInPattern				= 0x31,
 	ExpandPattern						= 0x34,
 	ShrinkPattern						= 0x35,
 	SetEchoBufferAccess					= 0x36,
@@ -37,6 +36,7 @@ enum CommandId : int
 	ReversePattern						= 0x38,
 	ReplaceInstrumentInPattern			= 0x39,
 	PasteOverwriteCopiedDataToPattern	= 0x3a,
+	PasteInsertCopiedDataToPattern		= 0x3b,
 
 	// 0x4*: Order list
 	SetPatternToOrder		= 0x40,
