@@ -1,10 +1,10 @@
 #include "add_instrument_command.hpp"
 #include <utility>
 
-AddInstrumentCommand::AddInstrumentCommand(std::weak_ptr<InstrumentsManager> manager, int num, SoundSource source, std::string name)
+AddInstrumentCommand::AddInstrumentCommand(std::weak_ptr<InstrumentsManager> manager, int num, InstrumentType type, std::string name)
 	: manager_(manager),
 	  num_(num),
-	  source_(source),
+	  type_(type),
 	  name_(name)
 {}
 
@@ -18,7 +18,7 @@ AddInstrumentCommand::AddInstrumentCommand(std::weak_ptr<InstrumentsManager> man
 void AddInstrumentCommand::redo()
 {
 	if (inst_) manager_.lock()->addInstrument(inst_->clone());
-	else manager_.lock()->addInstrument(num_, source_, name_);
+	else manager_.lock()->addInstrument(num_, type_, name_);
 }
 
 void AddInstrumentCommand::undo()
