@@ -5,12 +5,11 @@
 #include "abstract_command.hpp"
 #include "instrument.hpp"
 #include "instruments_manager.hpp"
-#include "misc.hpp"
 
 class AddInstrumentCommand : public AbstractCommand
 {
 public:
-	AddInstrumentCommand(std::weak_ptr<InstrumentsManager> manager, int num, SoundSource source, std::string name);
+	AddInstrumentCommand(std::weak_ptr<InstrumentsManager> manager, int num, InstrumentType type, std::string name);
 	AddInstrumentCommand(std::weak_ptr<InstrumentsManager> manager, std::unique_ptr<AbstractInstrument> inst);
 
 	void redo() override;
@@ -20,7 +19,7 @@ public:
 private:
 	std::weak_ptr<InstrumentsManager> manager_;
 	int num_;
-	SoundSource source_;
+	InstrumentType type_;
 	std::string name_;
 	std::unique_ptr<AbstractInstrument> inst_;
 };
