@@ -10,7 +10,7 @@ class InterpolatePatternCommand : public AbstractCommand
 {
 public:
 	InterpolatePatternCommand(std::weak_ptr<Module> mod, int songNum,
-							  int beginTrack, int beginColmn, int beginOrder, int beginStep,
+							  int beginTrack, int beginColumn, int beginOrder, int beginStep,
 							  int endTrack, int endColumn, int endStep);
 	void redo() override;
 	void undo() override;
@@ -21,4 +21,6 @@ private:
 	int song_, bTrack_, bCol_, order_, bStep_;
 	int eStep_;
 	std::vector<std::vector<std::string>> prevCells_;
+
+	inline int interp(int a, int b, size_t t, int div) { return a + (b - a) * static_cast<int>(t) / div; }
 };
