@@ -9,8 +9,8 @@
 class PasteCopiedDataToPatternCommand : public AbstractCommand
 {
 public:
-	PasteCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
-									std::vector<std::vector<std::string>> cells);
+	PasteCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColmn,
+									int beginOrder, int beginStep, std::vector<std::vector<std::string>> cells);
 	void redo() override;
 	void undo() override;
 	CommandId getID() const override;
@@ -19,6 +19,4 @@ private:
 	std::weak_ptr<Module> mod_;
 	int song_, track_, col_, order_, step_;
 	std::vector<std::vector<std::string>> cells_, prevCells_;
-
-	void setCells(std::vector<std::vector<std::string>>& cells);
 };
