@@ -9,8 +9,8 @@
 class PasteMixCopiedDataToPatternCommand : public AbstractCommand
 {
 public:
-	PasteMixCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
-									   std::vector<std::vector<std::string>> cells);
+	PasteMixCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColumn,
+									   int beginOrder, int beginStep, std::vector<std::vector<std::string>> cells);
 	void redo() override;
 	void undo() override;
 	CommandId getID() const override;
