@@ -9,7 +9,8 @@
 class PasteOverwriteCopiedDataToPatternCommand : public AbstractCommand
 {
 public:
-	PasteOverwriteCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
+	PasteOverwriteCopiedDataToPatternCommand(std::weak_ptr<Module> mod, int songNum, int beginTrack, int beginColumn,
+											 int beginOrder, int beginStep,
 											 std::vector<std::vector<std::string>> cells);
 	void redo() override;
 	void undo() override;
