@@ -4,20 +4,46 @@
 Configuration::Configuration()
 {
 	// Internal //
+	followMode_ = true;
+	workDir_ = "";
+	instOpenFormat_ = 0;
+	bankOpenFormat_ = 0;
+	instMask_ = false;
+	volMask_ = true;
+	visibleToolbar_ = true;
+	visibleStatusBar_ = true;
+	visibleWaveView_ = true;
+	pasteMode_ = PasteMode::CURSOR;
+
+	// Mainwindow state
 	mainW_ = 930;
 	mainH_= 780;
 	mainMax_ = false;
 	mainX_ = -1;	// Dummy
 	mainY_ = -1;	// Dummy
 	mainVSplit_ = -1;	// Dummy
+
+	// Instrument editor state
 	instFMW_ = 570;
 	instFMH_ = 750;
-	instSSGW_ = 500;
+	instSSGW_ = 510;
 	instSSGH_ = 390;
-	followMode_ = true;
-	workDir_ = "";
-	instOpenFormat_ = 0;
-	bankOpenFormat_ = 0;
+	instADPCMW_ = 510;
+	instADPCMH_ = 390;
+	instKitW_ = 590;
+	instKitH_ = 390;
+
+	// Toolbar state
+	mainTb_.setPosition(ToolbarConfiguration::TOP_POS);
+	mainTb_.setNumber(0);
+	mainTb_.setBreakBefore(false);
+	mainTb_.setX(-1);	// Dummy
+	mainTb_.setY(-1);	// Dummy
+	subTb_.setPosition(ToolbarConfiguration::TOP_POS);
+	subTb_.setNumber(1);
+	subTb_.setBreakBefore(false);
+	subTb_.setX(-1);	// Dummy
+	subTb_.setY(-1);	// Dummy
 
 	// General //
 	// General settings
@@ -32,10 +58,12 @@ Configuration::Configuration()
 	retrieveChannelState_ = false;
 	enableTranslation_ = true;
 	showFMDetuneSigned_ = false;
-	showWaveVisual_ = true;
 	fill00ToEffectValue_ = true;
-	autosetInstrument_ = true;
 	moveCursorHScroll_ = true;
+	overwriteUnusedUnedited_ = false;
+	writeOnlyUsedSamples_ = false;
+	reflectInstNumChange_ = false;
+	fixJamVol_ = true;
 
 	// Edit settings
 	pageJumpLength_ = 4;
@@ -43,10 +71,77 @@ Configuration::Configuration()
 	keyRepetision_ = true;
 
 	// Keys
-	keyOffKey_ = u8"-";
-	octUpKey_ = u8"Num+*";
-	octDownKey_ = u8"Num+/";
-	echoKey_ = u8"^";
+	shortcuts_ = {
+		{ KeyOff, u8"-" },
+		{ OctaveUp, u8"Num+*" },
+		{ OctaveDown, u8"Num+/" },
+		{ EchoBuffer, u8"^" },
+		{ PlayAndStop, u8"Return" },
+		{ Play, u8"" },
+		{ PlayFromStart, u8"F5" },
+		{ PlayPattern, u8"F6" },
+		{ PlayFromCursor, u8"F7" },
+		{ PlayFromMarker, u8"Ctrl+F7" },
+		{ PlayStep, u8"Ctrl+Return" },
+		{ Stop, u8"F8" },
+		{ FocusOnPattern, u8"F2" },
+		{ FocusOnOrder, u8"F3" },
+		{ FocusOnInstrument, u8"F4" },
+		{ ToggleEditJam, u8"Space" },
+		{ SetMarker, u8"Ctrl+B" },
+		{ PasteMix, u8"Ctrl+M" },
+		{ PasteOverwrite, u8"" },
+		{ PasteInsert, u8"" },
+		{ SelectAll, u8"Ctrl+A" },
+		{ Deselect, u8"Esc" },
+		{ SelectRow, u8"" },
+		{ SelectColumn, u8"" },
+		{ SelectPattern, u8"" },
+		{ SelectOrder, u8"" },
+		{ GoToStep, u8"Alt+G" },
+		{ ToggleTrack, u8"Alt+F9" },
+		{ SoloTrack, u8"Alt+F10" },
+		{ Interpolate, u8"Ctrl+G" },
+		{ Reverse, u8"Ctrl+R" },
+		{ GoToPrevOrder, u8"Ctrl+Left" },
+		{ GoToNextOrder, u8"Ctrl+Right" },
+		{ ToggleBookmark, u8"Ctrl+K" },
+		{ PrevBookmark, u8"Ctrl+PgUp" },
+		{ NextBookmark, u8"Ctrl+PgDown" },
+		{ DecreaseNote, u8"Ctrl+F1" },
+		{ IncreaseNote, u8"Ctrl+F2" },
+		{ DecreaseOctave, u8"Ctrl+F3" },
+		{ IncreaseOctave, u8"Ctrl+F4" },
+		{ PrevInstrument, u8"Alt+Left" },
+		{ NextInstrument, u8"Alt+Right" },
+		{ MaskInstrument, u8"" },
+		{ MaskVolume, u8"" },
+		{ EditInstrument, u8"Ctrl+I" },
+		{ FollowMode, u8"ScrollLock" },
+		{ DuplicateOrder, u8"Ctrl+D" },
+		{ ClonePatterns, u8"Alt+D" },
+		{ CloneOrder, u8"" },
+		{ ReplaceInstrument, u8"Alt+S" },
+		{ ExpandPattern, u8"" },
+		{ ShrinkPattern, u8"" },
+		{ FineDecreaseValues, u8"Shift+F1" },
+		{ FineIncreaseValues, u8"Shift+F2" },
+		{ CoarseDecreaseValues, u8"Shift+F3" },
+		{ CoarseIncreaseValuse, u8"Shift+F4" },
+		{ ExpandEffect, u8"Alt+L" },
+		{ ShrinkEffect, u8"Alt+K" },
+		{ PrevHighlighted, u8"Ctrl+Up" },
+		{ NextHighlighted, u8"Ctrl+Down" },
+		{ IncreasePatternSize, u8"" },
+		{ DecreasePatternSize, u8"" },
+		{ IncreaseEditStep, u8"" },
+		{ DecreaseEditStep, u8"" },
+		{ DisplayEffectList, u8"F1" },
+		{ PreviousSong, u8"" },
+		{ NextSong, u8"" },
+		{ JamVolumeUp, u8"" },
+		{ JamVolumeDown, u8"" }
+	};
 	noteEntryLayout_ = QWERTY;
 
 	// Sound //
@@ -601,378 +696,221 @@ Configuration::Configuration()
 }
 
 // Internal //
-void Configuration::setMainWindowWidth(int w)
-{
-	mainW_ = w;
-}
+void Configuration::setFollowMode(bool enabled) { followMode_ = enabled; }
 
-int Configuration::getMainWindowWidth() const
-{
-	return mainW_;
-}
+bool Configuration::getFollowMode() const { return followMode_; }
 
-void Configuration::setMainWindowHeight(int h)
-{
-	mainH_ = h;
-}
+void Configuration::setWorkingDirectory(std::string path) { workDir_ = path; }
 
-int Configuration::getMainWindowHeight() const
-{
-	return mainH_;
-}
+std::string Configuration::getWorkingDirectory() const { return workDir_; }
 
-void Configuration::setMainWindowMaximized(bool isMax)
-{
-	mainMax_ = isMax;
-}
+void Configuration::setInstrumentOpenFormat(int i) { instOpenFormat_ = i; }
 
-bool Configuration::getMainWindowMaximized() const
-{
-	return mainMax_;
-}
+int Configuration::getInstrumentOpenFormat() const { return instOpenFormat_; }
 
-void Configuration::setMainWindowX(int x)
-{
-	mainX_ = x;
-}
+void Configuration::setBankOpenFormat(int i) { bankOpenFormat_ = i; }
 
-int Configuration::getMainWindowX() const
-{
-	return mainX_;
-}
+int Configuration::getBankOpenFormat() const { return bankOpenFormat_; }
 
-void Configuration::setMainWindowY(int y)
-{
-	mainY_ = y;
-}
+void Configuration::setInstrumentMask(bool enabled) { instMask_ = enabled; }
 
-int Configuration::getMainWindowY() const
-{
-	return mainY_;
-}
+bool Configuration::getInstrumentMask() const { return instMask_; }
 
-void Configuration::setMainWindowVerticalSplit(int y)
-{
-	mainVSplit_ = y;
-}
+void Configuration::setVolumeMask(bool enabled) { volMask_ = enabled; }
 
-int Configuration::getMainWindowVerticalSplit() const
-{
-	return mainVSplit_;
-}
+bool Configuration::getVolumeMask() const { return volMask_; }
 
-void Configuration::setInstrumentFMWindowWidth(int w)
-{
-	instFMW_ = w;
-}
+void Configuration::setVisibleToolbar(bool visible) { visibleToolbar_ = visible; }
 
-int Configuration::getInstrumentFMWindowWidth() const
-{
-	return instFMW_;
-}
+bool Configuration::getVisibleToolbar() const { return visibleToolbar_; }
 
-void Configuration::setInstrumentFMWindowHeight(int h)
-{
-	instFMH_ = h;
-}
+void Configuration::setVisibleStatusBar(bool visible) { visibleStatusBar_ = visible; }
 
-int Configuration::getInstrumentFMWindowHeight() const
-{
-	return instFMH_;
-}
+bool Configuration::getVisibleStatusBar() const { return visibleStatusBar_; }
 
-void Configuration::setInstrumentSSGWindowWidth(int w)
-{
-	instSSGW_ = w;
-}
+void Configuration::setVisibleWaveView(bool visible) { visibleWaveView_ = visible; }
 
-int Configuration::getInstrumentSSGWindowWidth() const
-{
-	return instSSGW_;
-}
+bool Configuration::getVisibleWaveView() const { return visibleWaveView_; }
 
-void Configuration::setInstrumentSSGWindowHeight(int h)
-{
-	instSSGH_ = h;
-}
+void Configuration::setPasteMode(PasteMode mode) { pasteMode_ = mode; }
 
-int Configuration::getInstrumentSSGWindowHeight() const
-{
-	return instSSGH_;
-}
+Configuration::PasteMode Configuration::getPasteMode() const { return pasteMode_; }
 
-void Configuration::setFollowMode(bool enabled)
-{
-	followMode_ = enabled;
-}
+// Mainwindow state
+void Configuration::setMainWindowWidth(int w) { mainW_ = w; }
 
-bool Configuration::getFollowMode() const
-{
-	return followMode_;
-}
+int Configuration::getMainWindowWidth() const { return mainW_; }
 
-void Configuration::setWorkingDirectory(std::string path)
-{
-	workDir_ = path;
-}
+void Configuration::setMainWindowHeight(int h) { mainH_ = h; }
 
-std::string Configuration::getWorkingDirectory() const
-{
-	return workDir_;
-}
+int Configuration::getMainWindowHeight() const { return mainH_; }
 
-void Configuration::setInstrumentOpenFormat(int i)
-{
-	instOpenFormat_ = i;
-}
+void Configuration::setMainWindowMaximized(bool isMax) { mainMax_ = isMax; }
 
-int Configuration::getInstrumentOpenFormat() const
-{
-	return instOpenFormat_;
-}
+bool Configuration::getMainWindowMaximized() const { return mainMax_; }
 
-void Configuration::setBankOpenFormat(int i)
-{
-	bankOpenFormat_ = i;
-}
+void Configuration::setMainWindowX(int x) { mainX_ = x; }
 
-int Configuration::getBankOpenFormat() const
-{
-	return bankOpenFormat_;
-}
+int Configuration::getMainWindowX() const { return mainX_; }
+
+void Configuration::setMainWindowY(int y) { mainY_ = y; }
+
+int Configuration::getMainWindowY() const { return mainY_; }
+
+void Configuration::setMainWindowVerticalSplit(int y) { mainVSplit_ = y; }
+
+int Configuration::getMainWindowVerticalSplit() const { return mainVSplit_; }
+
+// Instrument editor state
+void Configuration::setInstrumentFMWindowWidth(int w) { instFMW_ = w; }
+
+int Configuration::getInstrumentFMWindowWidth() const { return instFMW_; }
+
+void Configuration::setInstrumentFMWindowHeight(int h) { instFMH_ = h; }
+
+int Configuration::getInstrumentFMWindowHeight() const { return instFMH_; }
+
+void Configuration::setInstrumentSSGWindowWidth(int w) { instSSGW_ = w; }
+
+int Configuration::getInstrumentSSGWindowWidth() const { return instSSGW_; }
+
+void Configuration::setInstrumentSSGWindowHeight(int h) { instSSGH_ = h; }
+
+int Configuration::getInstrumentSSGWindowHeight() const { return instSSGH_; }
+
+void Configuration::setInstrumentADPCMWindowWidth(int w) { instADPCMW_ = w; }
+
+int Configuration::getInstrumentADPCMWindowWidth() const { return instADPCMW_; }
+
+void Configuration::setInstrumentADPCMWindowHeight(int h) { instADPCMH_ = h; }
+
+int Configuration::getInstrumentADPCMWindowHeight() const { return instADPCMH_; }
+
+void Configuration::setInstrumentDrumkitWindowWidth(int w) { instKitW_ = w; }
+
+int Configuration::getInstrumentDrumkitWindowWidth() const { return instKitW_; }
+
+void Configuration::setInstrumentDrumkitWindowHeight(int h) { instKitH_ = h; }
+
+int Configuration::getInstrumentDrumkitWindowHeight() const { return instKitH_; }
+
+// Toolbar state
+using TBConfig = Configuration::ToolbarConfiguration;
+void TBConfig::setPosition(ToolbarPosition pos) { pos_ = pos; }
+
+TBConfig::ToolbarPosition TBConfig::getPosition() const { return pos_; }
+
+void TBConfig::setNumber(int n) { num_ = n; }
+
+int TBConfig::getNumber() const { return num_; }
+
+void TBConfig::setBreakBefore(bool enabled) { hasBreakBefore_ = enabled; }
+
+bool TBConfig::hasBreakBefore() const { return hasBreakBefore_; }
+
+void TBConfig::setX(int x) { x_ = x; }
+
+int TBConfig::getX() const { return x_; }
+
+void TBConfig::setY(int y) { y_ = y; }
+
+int TBConfig::getY() const { return y_; }
+
+TBConfig& Configuration::getMainToolbarConfiguration() { return mainTb_; }
+
+TBConfig& Configuration::getSubToolbarConfiguration() { return subTb_; }
 
 // General //
 // General settings
-void Configuration::setWarpCursor(bool enabled)
-{
-	warpCursor_ = enabled;
-}
+void Configuration::setWarpCursor(bool enabled) { warpCursor_ = enabled; }
 
-bool Configuration::getWarpCursor() const
-{
-	return warpCursor_;
-}
+bool Configuration::getWarpCursor() const { return warpCursor_; }
 
-void Configuration::setWarpAcrossOrders(bool enabled)
-{
-	warpAcrossOrders_ = enabled;
-}
+void Configuration::setWarpAcrossOrders(bool enabled) { warpAcrossOrders_ = enabled; }
 
-bool Configuration::getWarpAcrossOrders() const
-{
-	return warpAcrossOrders_;
-}
+bool Configuration::getWarpAcrossOrders() const { return warpAcrossOrders_; }
 
-void Configuration::setShowRowNumberInHex(bool enabled)
-{
-	showRowNumHex_ = enabled;
-}
+void Configuration::setShowRowNumberInHex(bool enabled) { showRowNumHex_ = enabled; }
 
-bool Configuration::getShowRowNumberInHex() const
-{
-	return showRowNumHex_;
-}
+bool Configuration::getShowRowNumberInHex() const { return showRowNumHex_; }
 
-void Configuration::setShowPreviousNextOrders(bool enabled)
-{
-	showPrevNextOrders_ = enabled;
-}
+void Configuration::setShowPreviousNextOrders(bool enabled) { showPrevNextOrders_ = enabled; }
 
-bool Configuration::getShowPreviousNextOrders() const
-{
-	return showPrevNextOrders_;
-}
+bool Configuration::getShowPreviousNextOrders() const { return showPrevNextOrders_; }
 
-void Configuration::setBackupModules(bool enabled)
-{
-	backupModules_ = enabled;
-}
+void Configuration::setBackupModules(bool enabled) { backupModules_ = enabled; }
 
-bool Configuration::getBackupModules() const
-{
-	return backupModules_;
-}
+bool Configuration::getBackupModules() const { return backupModules_; }
 
-void Configuration::setDontSelectOnDoubleClick(bool enabled)
-{
-	dontSelectOnDoubleClick_ = enabled;
-}
+void Configuration::setDontSelectOnDoubleClick(bool enabled) { dontSelectOnDoubleClick_ = enabled; }
 
-bool Configuration::getDontSelectOnDoubleClick() const
-{
-	return dontSelectOnDoubleClick_;
-}
+bool Configuration::getDontSelectOnDoubleClick() const { return dontSelectOnDoubleClick_; }
 
-void Configuration::setReverseFMVolumeOrder(bool enabled)
-{
-	reverseFMVolumeOrder_= enabled;
-}
+void Configuration::setReverseFMVolumeOrder(bool enabled) { reverseFMVolumeOrder_= enabled; }
 
-bool Configuration::getReverseFMVolumeOrder() const
-{
-	return reverseFMVolumeOrder_;
-}
+bool Configuration::getReverseFMVolumeOrder() const { return reverseFMVolumeOrder_; }
 
-void Configuration::setMoveCursorToRight(bool enabled)
-{
-	moveCursorToRight_ = enabled;
-}
+void Configuration::setMoveCursorToRight(bool enabled) { moveCursorToRight_ = enabled; }
 
-bool Configuration::getMoveCursorToRight() const
-{
-	return moveCursorToRight_;
-}
+bool Configuration::getMoveCursorToRight() const { return moveCursorToRight_; }
 
-void Configuration::setRetrieveChannelState(bool enabled)
-{
-	retrieveChannelState_ = enabled;
-}
+void Configuration::setRetrieveChannelState(bool enabled) { retrieveChannelState_ = enabled; }
 
-bool Configuration::getRetrieveChannelState() const
-{
-	return retrieveChannelState_;
-}
+bool Configuration::getRetrieveChannelState() const { return retrieveChannelState_; }
 
-void Configuration::setEnableTranslation(bool enabled)
-{
-	enableTranslation_ = enabled;
-}
-bool Configuration::getEnableTranslation() const
-{
-	return enableTranslation_;
-}
+void Configuration::setEnableTranslation(bool enabled) { enableTranslation_ = enabled; }
 
-void Configuration::setShowFMDetuneAsSigned(bool enabled)
-{
-	showFMDetuneSigned_ = enabled;
-}
+bool Configuration::getEnableTranslation() const { return enableTranslation_; }
 
-bool Configuration::getShowFMDetuneAsSigned() const
-{
-	return showFMDetuneSigned_;
-}
+void Configuration::setShowFMDetuneAsSigned(bool enabled) { showFMDetuneSigned_ = enabled; }
 
-void Configuration::setShowWaveVisual(bool enabled)
-{
-	showWaveVisual_ = enabled;
-}
+bool Configuration::getShowFMDetuneAsSigned() const { return showFMDetuneSigned_; }
 
-bool Configuration::getShowWaveVisual() const
-{
-	return showWaveVisual_;
-}
+void Configuration::setFill00ToEffectValue(bool enabled) { fill00ToEffectValue_ = enabled; }
 
-void Configuration::setFill00ToEffectValue(bool enabled)
-{
-	fill00ToEffectValue_ = enabled;
-}
+bool Configuration::getFill00ToEffectValue() const { return fill00ToEffectValue_; }
 
-bool Configuration::getFill00ToEffectValue() const
-{
-	return fill00ToEffectValue_;
-}
+void Configuration::setMoveCursorByHorizontalScroll(bool enabled) { moveCursorHScroll_ = enabled; }
 
-void Configuration::setAutosetInstrument(bool enabled)
-{
-	autosetInstrument_ = enabled;
-}
+bool Configuration::getMoveCursorByHorizontalScroll() const { return moveCursorHScroll_; }
 
-bool Configuration::getAutosetInstrument() const
-{
-	return autosetInstrument_;
-}
+void Configuration::setOverwriteUnusedUneditedPropety(bool enabled) { overwriteUnusedUnedited_ = enabled; }
 
-void Configuration::setMoveCursorByHorizontalScroll(bool enabled)
-{
-	moveCursorHScroll_ = enabled;
-}
+bool Configuration::getOverwriteUnusedUneditedPropety() const { return overwriteUnusedUnedited_; }
 
-bool Configuration::getMoveCursorByHorizontalScroll() const
-{
-	return moveCursorHScroll_;
-}
+void Configuration::setWriteOnlyUsedSamples(bool enabled) { writeOnlyUsedSamples_ = enabled; }
+
+bool Configuration::getWriteOnlyUsedSamples() const { return writeOnlyUsedSamples_; }
+
+void Configuration::setReflectInstrumentNumberChange(bool enabled) { reflectInstNumChange_ = enabled; }
+
+bool Configuration::getReflectInstrumentNumberChange() const { return reflectInstNumChange_; }
+
+void Configuration::setFixJammingVolume(bool enabled) { fixJamVol_ = enabled; }
+
+bool Configuration::getFixJammingVolume() const { return fixJamVol_; }
 
 // Edit settings
-void Configuration::setPageJumpLength(size_t length)
-{
-	pageJumpLength_ = length;
-}
+void Configuration::setPageJumpLength(size_t length) { pageJumpLength_ = length; }
 
-size_t Configuration::getPageJumpLength() const
-{
-	return pageJumpLength_;
-}
+size_t Configuration::getPageJumpLength() const { return pageJumpLength_; }
 
-void Configuration::setEditableStep(size_t step)
-{
-	editableStep_ = step;
-}
+void Configuration::setEditableStep(size_t step) { editableStep_ = step; }
 
-size_t Configuration::getEditableStep() const
-{
-	return editableStep_;
-}
+size_t Configuration::getEditableStep() const { return editableStep_; }
 
-void Configuration::setKeyRepetition(bool enabled)
-{
-	keyRepetision_ = enabled;
-}
+void Configuration::setKeyRepetition(bool enabled) { keyRepetision_ = enabled; }
 
-bool Configuration::getKeyRepetition() const
-{
-	return keyRepetision_;
-}
+bool Configuration::getKeyRepetition() const { return keyRepetision_; }
 
 // Keys
-void Configuration::setKeyOffKey(std::string key)
-{
-	keyOffKey_ = key;
-}
+void Configuration::setShortcuts(std::unordered_map<ShortcutAction, std::string> shortcuts) { shortcuts_ = shortcuts; }
 
-std::string Configuration::getKeyOffKey() const
-{
-	return keyOffKey_;
-}
+std::unordered_map<Configuration::ShortcutAction, std::string> Configuration::getShortcuts() const { return shortcuts_; }
 
-void Configuration::setOctaveUpKey(std::string key)
-{
-	octUpKey_ = key;
-}
+void Configuration::setNoteEntryLayout(KeyboardLayout layout) { noteEntryLayout_ = layout; }
 
-std::string Configuration::getOctaveUpKey() const
-{
-	return octUpKey_;
-}
-
-void Configuration::setOctaveDownKey(std::string key)
-{
-	octDownKey_ = key;
-}
-
-std::string Configuration::getOctaveDownKey() const
-{
-	return octDownKey_;
-}
-
-void Configuration::setEchoBufferKey(std::string key)
-{
-	echoKey_ = key;
-}
-
-std::string Configuration::getEchoBufferKey() const
-{
-	return echoKey_;
-}
-
-void Configuration::setNoteEntryLayout(KeyboardLayout layout)
-{
-	noteEntryLayout_ = layout;
-}
-
-Configuration::KeyboardLayout Configuration::getNoteEntryLayout() const
-{
-	return noteEntryLayout_;
-}
+Configuration::KeyboardLayout Configuration::getNoteEntryLayout() const { return noteEntryLayout_; }
 
 void Configuration::setCustomLayoutKeys(std::unordered_map<std::string, JamKey> mapping)
 {
@@ -985,196 +923,82 @@ std::unordered_map<std::string, JamKey> Configuration::getCustomLayoutKeys() con
 }
 
 // Sound //
-void Configuration::setSoundAPI(std::string api)
-{
-	sndAPI_ = api;
-}
+void Configuration::setSoundAPI(std::string api) { sndAPI_ = api; }
 
-std::string Configuration::getSoundAPI() const
-{
-	return sndAPI_;
-}
+std::string Configuration::getSoundAPI() const { return sndAPI_; }
 
-void Configuration::setSoundDevice(std::string device)
-{
-	sndDevice_ = device;
-}
+void Configuration::setSoundDevice(std::string device) { sndDevice_ = device; }
 
-std::string Configuration::getSoundDevice() const
-{
-	return sndDevice_;
-}
+std::string Configuration::getSoundDevice() const { return sndDevice_; }
 
-void Configuration::setRealChipInterface(RealChipInterface type)
-{
-	realChip_ = type;
-}
+void Configuration::setRealChipInterface(RealChipInterface type) { realChip_ = type; }
 
-RealChipInterface Configuration::getRealChipInterface() const
-{
-	return realChip_;
-}
+RealChipInterface Configuration::getRealChipInterface() const { return realChip_; }
 
-void Configuration::setEmulator(int emulator)
-{
-	emulator_ = emulator;
-}
+void Configuration::setEmulator(int emulator) { emulator_ = emulator; }
 
-int Configuration::getEmulator() const
-{
-	return emulator_;
-}
+int Configuration::getEmulator() const { return emulator_; }
 
-void Configuration::setSampleRate(uint32_t rate)
-{
-	sampleRate_ = rate;
-}
+void Configuration::setSampleRate(uint32_t rate) { sampleRate_ = rate; }
 
-uint32_t Configuration::getSampleRate() const
-{
-	return sampleRate_;
-}
+uint32_t Configuration::getSampleRate() const { return sampleRate_; }
 
-void Configuration::setBufferLength(size_t length)
-{
-	bufferLength_ = length;
-}
+void Configuration::setBufferLength(size_t length) { bufferLength_ = length; }
 
-size_t Configuration::getBufferLength() const
-{
-	return bufferLength_;
-}
+size_t Configuration::getBufferLength() const { return bufferLength_; }
 
 // Midi //
-void Configuration::setMidiInputPort(const std::string &port)
-{
-	midiInPort_ = port;
-}
+void Configuration::setMidiInputPort(const std::string &port) { midiInPort_ = port; }
 
-std::string Configuration::getMidiInputPort() const
-{
-	return midiInPort_;
-}
+std::string Configuration::getMidiInputPort() const { return midiInPort_; }
 
 // Mixer //
-void Configuration::setMixerVolumeMaster(int percentage)
-{
-	mixerVolumeMaster_ = percentage;
-}
+void Configuration::setMixerVolumeMaster(int percentage) { mixerVolumeMaster_ = percentage; }
 
-int Configuration::getMixerVolumeMaster() const
-{
-	return mixerVolumeMaster_;
-}
+int Configuration::getMixerVolumeMaster() const { return mixerVolumeMaster_; }
 
-void Configuration::setMixerVolumeFM(double dB)
-{
-	mixerVolumeFM_ = dB;
-}
+void Configuration::setMixerVolumeFM(double dB) { mixerVolumeFM_ = dB; }
 
-double Configuration::getMixerVolumeFM() const
-{
-	return mixerVolumeFM_;
-}
+double Configuration::getMixerVolumeFM() const { return mixerVolumeFM_; }
 
-void Configuration::setMixerVolumeSSG(double dB)
-{
-	mixerVolumeSSG_ = dB;
-}
+void Configuration::setMixerVolumeSSG(double dB) { mixerVolumeSSG_ = dB; }
 
-double Configuration::getMixerVolumeSSG() const
-{
-	return mixerVolumeSSG_;
-}
+double Configuration::getMixerVolumeSSG() const { return mixerVolumeSSG_; }
 
 // Input //
-void Configuration::setFMEnvelopeTexts(std::vector<FMEnvelopeText> texts)
-{
-	fmEnvelopeTexts_ = texts;
-}
+void Configuration::setFMEnvelopeTexts(std::vector<FMEnvelopeText> texts) { fmEnvelopeTexts_ = texts; }
 
-std::vector<FMEnvelopeText> Configuration::getFMEnvelopeTexts() const
-{
-	return fmEnvelopeTexts_;
-}
+std::vector<FMEnvelopeText> Configuration::getFMEnvelopeTexts() const { return fmEnvelopeTexts_; }
 
 // Appearrance
-void Configuration::setPatternEditorHeaderFont(std::string font)
-{
-	ptnHdFont_ = font;
-}
+void Configuration::setPatternEditorHeaderFont(std::string font) { ptnHdFont_ = font; }
 
-std::string Configuration::getPatternEditorHeaderFont() const
-{
-	return ptnHdFont_;
-}
+std::string Configuration::getPatternEditorHeaderFont() const { return ptnHdFont_; }
 
-void Configuration::setPatternEditorHeaderFontSize(int size)
-{
-	ptnHdFontSize_ = size;
-}
+void Configuration::setPatternEditorHeaderFontSize(int size) { ptnHdFontSize_ = size; }
 
-int Configuration::getPatternEditorHeaderFontSize() const
-{
-	return ptnHdFontSize_;
-}
+int Configuration::getPatternEditorHeaderFontSize() const { return ptnHdFontSize_; }
 
-void Configuration::setPatternEditorRowsFont(std::string font)
-{
-	ptnRowFont_ = font;
-}
+void Configuration::setPatternEditorRowsFont(std::string font) { ptnRowFont_ = font; }
 
-std::string Configuration::getPatternEditorRowsFont() const
-{
-	return ptnRowFont_;
-}
+std::string Configuration::getPatternEditorRowsFont() const { return ptnRowFont_; }
 
-void Configuration::setPatternEditorRowsFontSize(int size)
-{
-	ptnRowFontSize_ = size;
-}
+void Configuration::setPatternEditorRowsFontSize(int size) { ptnRowFontSize_ = size; }
 
-int Configuration::getPatternEditorRowsFontSize() const
-{
-	return ptnRowFontSize_;
-}
+int Configuration::getPatternEditorRowsFontSize() const { return ptnRowFontSize_; }
 
-void Configuration::setOrderListHeaderFont(std::string font)
-{
-	odrHdFont_ = font;
-}
+void Configuration::setOrderListHeaderFont(std::string font) { odrHdFont_ = font; }
 
-std::string Configuration::getOrderListHeaderFont() const
-{
-	return odrHdFont_;
-}
+std::string Configuration::getOrderListHeaderFont() const { return odrHdFont_; }
 
-void Configuration::setOrderListHeaderFontSize(int size)
-{
-	odrHdFontSize_ = size;
-}
+void Configuration::setOrderListHeaderFontSize(int size) { odrHdFontSize_ = size; }
 
-int Configuration::getOrderListHeaderFontSize() const
-{
-	return odrHdFontSize_;
-}
+int Configuration::getOrderListHeaderFontSize() const { return odrHdFontSize_; }
 
-void Configuration::setOrderListRowsFont(std::string font)
-{
-	odrRowFont_ = font;
-}
+void Configuration::setOrderListRowsFont(std::string font) { odrRowFont_ = font; }
 
-std::string Configuration::getOrderListRowsFont() const
-{
-	return odrRowFont_;
-}
+std::string Configuration::getOrderListRowsFont() const { return odrRowFont_; }
 
-void Configuration::setOrderListRowsFontSize(int size)
-{
-	odrRowFontSize_ = size;
-}
+void Configuration::setOrderListRowsFontSize(int size) { odrRowFontSize_ = size; }
 
-int Configuration::getOrderListRowsFontSize() const
-{
-	return odrRowFontSize_;
-}
+int Configuration::getOrderListRowsFontSize() const { return odrRowFontSize_; }
