@@ -22,6 +22,37 @@ public:
 	Configuration();
 
 	// Internal //
+public:
+	void setFollowMode(bool enabled);
+	bool getFollowMode() const;
+	void setWorkingDirectory(std::string path);
+	std::string getWorkingDirectory() const;
+	void setInstrumentOpenFormat(int i);
+	int getInstrumentOpenFormat() const;
+	void setBankOpenFormat(int i);
+	int getBankOpenFormat() const;
+	void setInstrumentMask(bool enabled);
+	bool getInstrumentMask() const;
+	void setVolumeMask(bool enabled);
+	bool getVolumeMask() const;
+	void setVisibleToolbar(bool visible);
+	bool getVisibleToolbar() const;
+	void setVisibleStatusBar(bool visible);
+	bool getVisibleStatusBar() const;
+	void setVisibleWaveView(bool visible);
+	bool getVisibleWaveView() const;
+	enum PasteMode { CURSOR = 0, SELECTION, FILL };
+	void setPasteMode(PasteMode mode);
+	PasteMode getPasteMode() const;
+private:
+	bool followMode_;
+	std::string workDir_;
+	int instOpenFormat_, bankOpenFormat_;
+	bool instMask_, volMask_;
+	bool visibleToolbar_, visibleStatusBar_, visibleWaveView_;
+	PasteMode pasteMode_;
+
+	// Mainwindow state
 public:
 	void setMainWindowWidth(int w);
 	int getMainWindowWidth() const;
@@ -35,6 +66,14 @@ public:
 	int getMainWindowY() const;
 	void setMainWindowVerticalSplit(int y);
 	int getMainWindowVerticalSplit() const;
+private:
+	int mainW_, mainH_;
+	bool mainMax_;
+	int mainX_, mainY_;
+	int mainVSplit_;
+
+	// Instrument editor state
+public:
 	void setInstrumentFMWindowWidth(int w);
 	int getInstrumentFMWindowWidth() const;
 	void setInstrumentFMWindowHeight(int h);
@@ -43,24 +82,46 @@ public:
 	int getInstrumentSSGWindowWidth() const;
 	void setInstrumentSSGWindowHeight(int h);
 	int getInstrumentSSGWindowHeight() const;
-	void setFollowMode(bool enabled);
-	bool getFollowMode() const;
-	void setWorkingDirectory(std::string path);
-	std::string getWorkingDirectory() const;
-	void setInstrumentOpenFormat(int i);
-	int getInstrumentOpenFormat() const;
-	void setBankOpenFormat(int i);
-	int getBankOpenFormat() const;
+	void setInstrumentADPCMWindowWidth(int w);
+	int getInstrumentADPCMWindowWidth() const;
+	void setInstrumentADPCMWindowHeight(int h);
+	int getInstrumentADPCMWindowHeight() const;
+	void setInstrumentDrumkitWindowWidth(int w);
+	int getInstrumentDrumkitWindowWidth() const;
+	void setInstrumentDrumkitWindowHeight(int h);
+	int getInstrumentDrumkitWindowHeight() const;
 private:
-	int mainW_, mainH_;
-	bool mainMax_;
-	int mainX_, mainY_;
-	int mainVSplit_;
 	int instFMW_, instFMH_;
 	int instSSGW_, instSSGH_;
-	bool followMode_;
-	std::string workDir_;
-	int instOpenFormat_, bankOpenFormat_;
+	int instADPCMW_, instADPCMH_;
+	int instKitW_, instKitH_;
+
+	// Toolbar state
+public:
+	class ToolbarConfiguration
+	{
+	public:
+		enum ToolbarPosition : int { TOP_POS = 0, BOTTOM_POS, LEFT_POS, RIGHT_POS, FLOAT_POS };
+		void setPosition(ToolbarPosition pos);
+		ToolbarPosition getPosition() const;
+		void setNumber(int n);
+		int getNumber() const;
+		void setBreakBefore(bool enabled);
+		bool hasBreakBefore() const;
+		void setX(int x);
+		int getX() const;
+		void setY(int y);
+		int getY() const;
+	private:
+		ToolbarPosition pos_;
+		int num_;
+		bool hasBreakBefore_;
+		int x_, y_;
+	};
+	ToolbarConfiguration& getMainToolbarConfiguration();
+	ToolbarConfiguration& getSubToolbarConfiguration();
+private:
+	ToolbarConfiguration mainTb_, subTb_;
 
 	// General //
 	// General settings
@@ -87,23 +148,23 @@ public:
 	bool getEnableTranslation() const;
 	void setShowFMDetuneAsSigned(bool enabled);
 	bool getShowFMDetuneAsSigned() const;
-	void setShowWaveVisual(bool enabled);
-	bool getShowWaveVisual() const;
 	void setFill00ToEffectValue(bool enabled);
 	bool getFill00ToEffectValue() const;
-	void setAutosetInstrument(bool enabled);
-	bool getAutosetInstrument() const;
 	void setMoveCursorByHorizontalScroll(bool enabled);
 	bool getMoveCursorByHorizontalScroll() const;
+	void setOverwriteUnusedUneditedPropety(bool enabled);
+	bool getOverwriteUnusedUneditedPropety() const;
+	void setWriteOnlyUsedSamples(bool enabled);
+	bool getWriteOnlyUsedSamples() const;
+	void setReflectInstrumentNumberChange(bool enabled);
+	bool getReflectInstrumentNumberChange() const;
+	void setFixJammingVolume(bool enabled);
+	bool getFixJammingVolume() const;
 private:
-	bool warpCursor_, warpAcrossOrders_;
-	bool showRowNumHex_, showPrevNextOrders_;
-	bool backupModules_, dontSelectOnDoubleClick_;
-	bool reverseFMVolumeOrder_, moveCursorToRight_;
-	bool retrieveChannelState_, enableTranslation_;
-	bool showFMDetuneSigned_, showWaveVisual_;
-	bool fill00ToEffectValue_, autosetInstrument_;
-	bool moveCursorHScroll_;
+	bool warpCursor_, warpAcrossOrders_, showRowNumHex_, showPrevNextOrders_, backupModules_;
+	bool dontSelectOnDoubleClick_, reverseFMVolumeOrder_, moveCursorToRight_, retrieveChannelState_;
+	bool enableTranslation_, showFMDetuneSigned_, fill00ToEffectValue_, moveCursorHScroll_;
+	bool overwriteUnusedUnedited_, writeOnlyUsedSamples_, reflectInstNumChange_, fixJamVol_;
 
 	// Edit settings
 public:
@@ -119,14 +180,22 @@ private:
 
 	// Keys
 public:
-	void setKeyOffKey(std::string key);
-	std::string getKeyOffKey() const;
-	void setOctaveUpKey(std::string key);
-	std::string getOctaveUpKey() const;
-	void setOctaveDownKey(std::string key);
-	std::string getOctaveDownKey() const;
-	void setEchoBufferKey(std::string key);
-	std::string getEchoBufferKey() const;
+	enum ShortcutAction : int
+	{
+		KeyOff, OctaveUp, OctaveDown, EchoBuffer, PlayAndStop, Play, PlayFromStart, PlayPattern,
+		PlayFromCursor, PlayFromMarker, PlayStep, Stop, FocusOnPattern, FocusOnOrder, FocusOnInstrument,
+		ToggleEditJam, SetMarker, PasteMix, PasteOverwrite, PasteInsert, SelectAll, Deselect, SelectRow, SelectColumn,
+		SelectPattern, SelectOrder, GoToStep, ToggleTrack, SoloTrack, Interpolate, Reverse, GoToPrevOrder,
+		GoToNextOrder, ToggleBookmark, PrevBookmark, NextBookmark, DecreaseNote, IncreaseNote,
+		DecreaseOctave, IncreaseOctave, PrevInstrument, NextInstrument, MaskInstrument, MaskVolume,
+		EditInstrument, FollowMode, DuplicateOrder, ClonePatterns, CloneOrder, ReplaceInstrument,
+		ExpandPattern, ShrinkPattern, FineDecreaseValues, FineIncreaseValues, CoarseDecreaseValues,
+		CoarseIncreaseValuse, ExpandEffect, ShrinkEffect, PrevHighlighted, NextHighlighted,
+		IncreasePatternSize, DecreasePatternSize, IncreaseEditStep, DecreaseEditStep, DisplayEffectList,
+		PreviousSong, NextSong, JamVolumeUp, JamVolumeDown
+	};
+	void setShortcuts(std::unordered_map<ShortcutAction, std::string> shortcuts);
+	std::unordered_map<ShortcutAction, std::string> getShortcuts() const;
 	enum KeyboardLayout : int
 	{
 		// at the top, so new layouts can easily be added in after it
@@ -145,7 +214,7 @@ public:
 	std::unordered_map<std::string, JamKey> getCustomLayoutKeys() const;
 
 private:
-	std::string keyOffKey_, octUpKey_, octDownKey_, echoKey_;
+	std::unordered_map<ShortcutAction, std::string> shortcuts_;
 	KeyboardLayout noteEntryLayout_;
 
 	// Sound //
