@@ -34,6 +34,14 @@ ColorPalette::ColorPalette()
 	instSeqMaskColor = QColor::fromRgb(0, 0, 0, 128);
 	instSeqOddColColor = QColor::fromRgb(255, 255, 255, 31);
 
+	// ADPCM sample editor
+	instADPCMMemAllColor = QColor::fromRgb(210, 40, 180, 150);
+	instADPCMMemCurColor = QColor::fromRgb(210, 40, 180, 255);
+	instADPCMMemBackColor = QColor::fromRgb(0, 0, 0, 255);
+	instADPCMSampViewForeColor = QColor::fromRgb(38, 183, 173, 255);
+	instADPCMSampViewBackColor = QColor::fromRgb(0, 0, 0, 255);
+	instADPCMSampViewCenterColor = QColor::fromRgb(63, 63, 63, 255);
+
 	// Tone/Noise editor
 	tnToneCellColor = QColor::fromRgb(225, 209, 47, 255);
 	tnToneTextColor = QColor::fromRgb(255, 255, 126, 255);
@@ -85,6 +93,7 @@ ColorPalette::ColorPalette()
 	ptnMuteColor = QColor::fromRgb(255, 0, 0, 255);
 	ptnUnmuteColor = QColor::fromRgb(0, 255, 0, 255);
 	ptnBackColor = QColor::fromRgb(0, 0, 0, 255);
+	ptnMarkerColor = QColor::fromRgb(255, 255, 255, 128);
 
 	// Wave visual
 	wavBackColor = QColor::fromRgb(0, 0, 33, 255);
