@@ -29,6 +29,10 @@ public:
 	QColor instSeqMaskColor;
 	QColor instSeqOddColColor;
 
+	// ADPCM sample editor
+	QColor instADPCMMemAllColor, instADPCMMemCurColor, instADPCMMemBackColor;
+	QColor instADPCMSampViewForeColor, instADPCMSampViewBackColor, instADPCMSampViewCenterColor;
+
 	// Tone/Noise editor
 	QColor tnToneCellColor, tnToneTextColor;
 	QColor tnNoiseCellColor, tnNoiseTextColor;
@@ -61,6 +65,7 @@ public:
 	QColor ptnBorderColor;
 	QColor ptnMuteColor, ptnUnmuteColor;
 	QColor ptnBackColor;
+	QColor ptnMarkerColor;
 
 	// Wave visual
 	QColor wavBackColor;
