@@ -6,14 +6,18 @@
 #include <QWidget>
 #include <QListWidget>
 #include <QString>
+#include "gui/mainwindow.hpp"
 #include "gui/instrument_editor/instrument_form_manager.hpp"
-#include "misc.hpp"
+
+enum class InstrumentType;
 
 class AddInstrumentQtCommand : public QUndoCommand
 {
 public:
-	AddInstrumentQtCommand(QListWidget *list, int num, QString name, SoundSource source,
-						   std::weak_ptr<InstrumentFormManager> formMan, QUndoCommand *parent = nullptr);
+	AddInstrumentQtCommand(QListWidget *list, int num, QString name, InstrumentType type,
+						   std::weak_ptr<InstrumentFormManager> formMan,
+						   MainWindow* mainwin, bool onlyUsed, bool preventFirstStore = false,
+						   QUndoCommand *parent = nullptr);
 	void undo() Q_DECL_OVERRIDE;
 	void redo() Q_DECL_OVERRIDE;
 	int id() const Q_DECL_OVERRIDE;
@@ -22,8 +26,10 @@ private:
 	QListWidget *list_;
 	int num_;
 	QString name_;
-	SoundSource source_;
+	InstrumentType type_;
 	std::weak_ptr<InstrumentFormManager> formMan_;
+	MainWindow* mainwin_;
+	bool onlyUsed_, hasDone_;
 };
 
 #endif // ADD_INSTRUMENT_QT_COMMAND_HPP
