@@ -6,7 +6,6 @@
 #include <QString>
 #include <QListWidget>
 #include "gui/instrument_editor/instrument_form_manager.hpp"
-#include "misc.hpp"
 
 class ChangeInstrumentNameQtCommand : public QUndoCommand
 {
@@ -24,7 +23,6 @@ private:
 	int row_;
 	std::weak_ptr<InstrumentFormManager> formMan_;
 	QString oldName_, newName_;
-	SoundSource source_;
 };
 
 #endif // CHANGE_INSTRUMENT_NAME_QT_COMMAND_HPP
