@@ -5,23 +5,25 @@
 #include <QListWidget>
 #include <memory>
 #include "gui/instrument_editor/instrument_form_manager.hpp"
-#include "misc.hpp"
+
+enum class InstrumentType;
 
 class CloneInstrumentQtCommand : public QUndoCommand
 {
 public:
-	CloneInstrumentQtCommand(QListWidget *list, int num, int refNum,
+	CloneInstrumentQtCommand(QListWidget *list, int num, InstrumentType type, QString name,
 							 std::weak_ptr<InstrumentFormManager> formMan,
 							 QUndoCommand* parent = nullptr);
-    void undo() Q_DECL_OVERRIDE;
-    void redo() Q_DECL_OVERRIDE;
-    int id() const Q_DECL_OVERRIDE;
+	void undo() Q_DECL_OVERRIDE;
+	void redo() Q_DECL_OVERRIDE;
+	int id() const Q_DECL_OVERRIDE;
 
 private:
 	QListWidget* list_;
-	int cloneNum_, refNum_;
+	int cloneNum_;
 	std::weak_ptr<InstrumentFormManager> formMan_;
-	SoundSource source_;
+	InstrumentType type_;
+	QString name_;
 };
 
 #endif // CLONE_INSTRUMENT_QT_COMMAND_H
