@@ -4,15 +4,17 @@
 #include <QUndoCommand>
 #include <QListWidget>
 #include <memory>
+#include "gui/mainwindow.hpp"
 #include "gui/instrument_editor/instrument_form_manager.hpp"
-#include "misc.hpp"
+
+enum class InstrumentType;
 
 class DeepCloneInstrumentQtCommand : public QUndoCommand
 {
 public:
-	DeepCloneInstrumentQtCommand(QListWidget *list, int num, int refNum,
-							 std::weak_ptr<InstrumentFormManager> formMan,
-							 QUndoCommand* parent = nullptr);
+	DeepCloneInstrumentQtCommand(QListWidget *list, int num, InstrumentType type, QString name,
+								 std::weak_ptr<InstrumentFormManager> formMan, MainWindow* mainwin, bool onlyUsed,
+								 QUndoCommand* parent = nullptr);
 
 	void undo() Q_DECL_OVERRIDE;
 	void redo() Q_DECL_OVERRIDE;
@@ -20,9 +22,12 @@ public:
 
 private:
 	QListWidget* list_;
-	int cloneNum_, refNum_;
+	int cloneNum_;
 	std::weak_ptr<InstrumentFormManager> formMan_;
-	SoundSource source_;
+	InstrumentType type_;
+	QString name_;
+	MainWindow* mainwin_;
+	bool onlyUsed_;
 };
 
 #endif // DEEP_CLONE_INSTRUMENT_QT_COMMAND_HPP
