@@ -6,14 +6,17 @@
 #include <QWidget>
 #include <QListWidget>
 #include <QString>
+#include "gui/mainwindow.hpp"
 #include "gui/instrument_editor/instrument_form_manager.hpp"
-#include "misc.hpp"
+
+enum class InstrumentType;
 
 class RemoveInstrumentQtCommand : public QUndoCommand
 {
 public:
-	RemoveInstrumentQtCommand(QListWidget *list, int num, int row,
-							  std::weak_ptr<InstrumentFormManager> formMan, QUndoCommand *parent = nullptr);
+	RemoveInstrumentQtCommand(QListWidget *list, int num, int row, QString name, InstrumentType type,
+							  std::weak_ptr<InstrumentFormManager> formMan,
+							  MainWindow* mainwin, bool updateRequested, QUndoCommand *parent = nullptr);
 	void undo() Q_DECL_OVERRIDE;
 	void redo() Q_DECL_OVERRIDE;
 	int id() const Q_DECL_OVERRIDE;
@@ -23,8 +26,10 @@ private:
 	int num_;
 	QString name_;
 	int row_;
-	SoundSource source_;
+	InstrumentType type_;
 	std::weak_ptr<InstrumentFormManager> formMan_;
+	MainWindow* mainwin_;
+	bool updateRequested_;
 };
 
 #endif // REMOVE_INSTRUMENT_QT_COMMAND_HPP
