@@ -1,4 +1,5 @@
 #include "erase_cells_in_pattern_qt_command.hpp"
+#include "command_id.hpp"
 
 EraseCellsInPatternQtCommand::EraseCellsInPatternQtCommand(PatternEditorPanel* panel, QUndoCommand* parent)
 	: QUndoCommand(parent),
@@ -8,15 +9,15 @@ EraseCellsInPatternQtCommand::EraseCellsInPatternQtCommand(PatternEditorPanel* p
 
 void EraseCellsInPatternQtCommand::redo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 void EraseCellsInPatternQtCommand::undo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 int EraseCellsInPatternQtCommand::id() const
 {
-	return 0x2e;
+	return CommandId::EraseCellsInPattern;
 }
