@@ -1,4 +1,5 @@
 #include "paste_copied_data_to_pattern_qt_command.hpp"
+#include "command_id.hpp"
 
 PasteCopiedDataToPatternQtCommand::PasteCopiedDataToPatternQtCommand(PatternEditorPanel* panel, QUndoCommand* parent)
 	: QUndoCommand(parent),
@@ -8,15 +9,15 @@ PasteCopiedDataToPatternQtCommand::PasteCopiedDataToPatternQtCommand(PatternEdit
 
 void PasteCopiedDataToPatternQtCommand::redo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 void PasteCopiedDataToPatternQtCommand::undo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 int PasteCopiedDataToPatternQtCommand::id() const
 {
-	return 0x2d;
+	return CommandId::PasteCopiedDataToPattern;
 }
