@@ -18,10 +18,7 @@
 #include "paste_copied_data_to_pattern_qt_command.hpp"
 #include "erase_cells_in_pattern_qt_command.hpp"
 #include "paste_mix_copied_data_to_pattern_qt_command.hpp"
-#include "increase_note_key_in_pattern_qt_command.hpp"
-#include "decrease_note_key_in_pattern_qt_command.hpp"
-#include "increase_note_octave_in_pattern_qt_command.hpp"
-#include "decrease_note_octave_in_pattern_qt_command.hpp"
+#include "transpose_note_in_pattern_qt_command.hpp"
 #include "expand_pattern_qt_command.hpp"
 #include "shrink_pattern_qt_command.hpp"
 #include "set_echo_buffer_access_qt_command.hpp"
@@ -29,5 +26,7 @@
 #include "reverse_pattern_qt_command.hpp"
 #include "replace_instrument_in_pattern_qt_command.hpp"
 #include "paste_overwrite_copied_data_to_pattern_qt_command.hpp"
+#include "change_values_in_pattern_qt_command.hpp"
+#include "paste_insert_copied_data_to_pattern_qt_command.hpp"
 
 #endif // PATTERN_COMMANDS_QT_HPP
