@@ -1,4 +1,5 @@
 #include "shrink_pattern_qt_command.hpp"
+#include "command_id.hpp"
 
 ShrinkPatternQtCommand::ShrinkPatternQtCommand(PatternEditorPanel* panel, QUndoCommand* parent)
 	: QUndoCommand(parent),
@@ -8,15 +9,15 @@ ShrinkPatternQtCommand::ShrinkPatternQtCommand(PatternEditorPanel* panel, QUndoC
 
 void ShrinkPatternQtCommand::redo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 void ShrinkPatternQtCommand::undo()
 {
-	panel_->update();
+	panel_->redrawByPatternChanged(true);
 }
 
 int ShrinkPatternQtCommand::id() const
 {
-	return 0x35;
+	return CommandId::ShrinkPattern;
 }
