@@ -2,6 +2,7 @@
 #include "ui_configuration_dialog.h"
 #include <algorithm>
 #include <functional>
+#include <map>
 #include <QPushButton>
 #include <QMenu>
 #include <QMessageBox>
@@ -11,12 +12,17 @@
 #include <QColorDialog>
 #include <QFileDialog>
 #include <QApplication>
+#include <QListWidgetItem>
+#include <QTreeWidgetItem>
+#include <QToolButton>
+#include <QHBoxLayout>
 #include "slider_style.hpp"
 #include "fm_envelope_set_edit_dialog.hpp"
 #include "midi/midi.hpp"
 #include "jam_manager.hpp"
 #include "chips/chip_misc.h"
 #include "color_palette_handler.hpp"
+#include "gui/gui_util.hpp"
 
 ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, std::weak_ptr<ColorPalette> palette,
 										 std::string curApi, std::vector<std::string> apis, QWidget *parent)
@@ -37,41 +43,148 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	std::shared_ptr<Configuration> configLocked = config.lock();
 	// General //
 	// General settings
-	ui->generalSettingsListWidget->item(0)->setCheckState(toCheckState(configLocked->getWarpCursor()));
-	ui->generalSettingsListWidget->item(1)->setCheckState(toCheckState(configLocked->getWarpAcrossOrders()));
-	ui->generalSettingsListWidget->item(2)->setCheckState(toCheckState(configLocked->getShowRowNumberInHex()));
-	ui->generalSettingsListWidget->item(3)->setCheckState(toCheckState(configLocked->getShowPreviousNextOrders()));
-	ui->generalSettingsListWidget->item(4)->setCheckState(toCheckState(configLocked->getBackupModules()));
-	ui->generalSettingsListWidget->item(5)->setCheckState(toCheckState(configLocked->getDontSelectOnDoubleClick()));
-	ui->generalSettingsListWidget->item(6)->setCheckState(toCheckState(configLocked->getReverseFMVolumeOrder()));
-	ui->generalSettingsListWidget->item(7)->setCheckState(toCheckState(configLocked->getMoveCursorToRight()));
-	ui->generalSettingsListWidget->item(8)->setCheckState(toCheckState(configLocked->getRetrieveChannelState()));
-	ui->generalSettingsListWidget->item(9)->setCheckState(toCheckState(configLocked->getEnableTranslation()));
-	ui->generalSettingsListWidget->item(10)->setCheckState(toCheckState(configLocked->getShowFMDetuneAsSigned()));
-	ui->generalSettingsListWidget->item(11)->setCheckState(toCheckState(configLocked->getShowWaveVisual()));
-	ui->generalSettingsListWidget->item(12)->setCheckState(toCheckState(configLocked->getFill00ToEffectValue()));
-	ui->generalSettingsListWidget->item(13)->setCheckState(toCheckState(configLocked->getAutosetInstrument()));
-	ui->generalSettingsListWidget->item(14)->setCheckState(toCheckState(configLocked->getMoveCursorByHorizontalScroll()));
+	auto glfunc = [&](int i, bool enabled, QString desc) {
+		QListWidgetItem* item = ui->generalSettingsListWidget->item(i);
+		item->setCheckState(toCheckState(enabled));
+		item->setData(Qt::UserRole, desc);
+	};
+	glfunc(0, configLocked->getWarpCursor(),
+		   tr("Warp the cursor around the edges of the pattern editor."));
+	glfunc(1, configLocked->getWarpAcrossOrders(),
+		   tr("Move to previous or next order when reaching top or bottom in the pattern editor."));
+	glfunc(2, configLocked->getShowRowNumberInHex(),
+		   tr("Display row numbers and the playback position on the status bar in hexadecimal."));
+	glfunc(3, configLocked->getShowPreviousNextOrders(),
+		   tr("Preview previous and next orders in the pattern editor."));
+	glfunc(4, configLocked->getBackupModules(),
+		   tr("Create a backup copy of the existing file when saving a module."));
+	glfunc(5, configLocked->getDontSelectOnDoubleClick(),
+		   tr("Don't select the whole track when double-clicking in the pattern editor."));
+	glfunc(6, configLocked->getReverseFMVolumeOrder(),
+		   tr("Reverse the order of FM volume so that 00 is the quietest in the pattern editor."));
+	glfunc(7, configLocked->getMoveCursorToRight(),
+		   tr("Move the cursor to right after entering effects in the pattern editor."));
+	glfunc(8, configLocked->getRetrieveChannelState(),
+		   tr("Reconstruct the current channel's state from previous orders upon playing."));
+	glfunc(9, configLocked->getEnableTranslation(),
+		   tr("Translate to your language from the next launch. See readme to check supported languages."));
+	glfunc(10, configLocked->getShowFMDetuneAsSigned(),
+		   tr("Display FM detune values as signed numbers in the FM envelope editor."));
+	glfunc(11, configLocked->getFill00ToEffectValue(),
+		   tr("Fill 00 to effect value column upon entering effect id."));
+	glfunc(12, configLocked->getMoveCursorByHorizontalScroll(),
+		   tr("Move the cursor position by cell with horizontal scroll bar in the order list and the pattern editor."));
+	glfunc(13, configLocked->getOverwriteUnusedUneditedPropety(),
+		   tr("Overwrite unused and unedited instrument properties on creating new properties. "
+			  "When disabled, override unused properties regardless of editing."));
+	glfunc(14, configLocked->getWriteOnlyUsedSamples(),
+		   tr("Send only ADPCM samples used by instruments to the ADPCM memory. "
+			  "Recommend to turn off if you change ADPCM samples frequently due to take the high rewriting cost."));
+	glfunc(15, configLocked->getReflectInstrumentNumberChange(),
+		   tr("Correspond the instrument number in patterns when the instrument changes its number."));
+	glfunc(16, configLocked->getFixJammingVolume(),
+		   tr("Set maximum volume during jam mode. When unchecked, the volume is changed by the volume spinbox."));
 
 	// Edit settings
 	ui->pageJumpLengthSpinBox->setValue(static_cast<int>(configLocked->getPageJumpLength()));
 
 	// Keys
-	ui->keyOffKeySequenceEdit->setKeySequence(
-				QString::fromUtf8(configLocked->getKeyOffKey().c_str(),
-								  static_cast<int>(configLocked->getKeyOffKey().length())));
-	ui->octaveUpKeySequenceEdit->setKeySequence(
-				QString::fromUtf8(configLocked->getOctaveUpKey().c_str(),
-								  static_cast<int>(configLocked->getOctaveUpKey().length())));
-	ui->octaveDownKeySequenceEdit->setKeySequence(
-				QString::fromUtf8(configLocked->getOctaveDownKey().c_str(),
-								  static_cast<int>(configLocked->getOctaveDownKey().length())));
-	ui->echoBufferKeySequenceEdit->setKeySequence(
-				QString::fromUtf8(configLocked->getEchoBufferKey().c_str(),
-								  static_cast<int>(configLocked->getEchoBufferKey().length())));
-	ui->keyboardTypeComboBox->setCurrentIndex(static_cast<int>(configLocked->getNoteEntryLayout()));
+	ui->shortcutsTreeWidget->header()->setSectionResizeMode(0, QHeaderView::Stretch);
+	ui->shortcutsTreeWidget->header()->setSectionResizeMode(1, QHeaderView::Fixed);
+	std::map<Configuration::ShortcutAction, QString> shortcutsActions = {
+		{ Configuration::KeyOff, tr("Key off") },
+		{ Configuration::OctaveUp, tr("Octave up") },
+		{ Configuration::OctaveDown, tr("Octave down") },
+		{ Configuration::EchoBuffer, tr("Echo buffer") },
+		{ Configuration::PlayAndStop, tr("Play and stop") },
+		{ Configuration::Play, tr("Play") },
+		{ Configuration::PlayFromStart, tr("Play from start") },
+		{ Configuration::PlayPattern, tr("Play pattern") },
+		{ Configuration::PlayFromCursor, tr("Play from cursor") },
+		{ Configuration::PlayFromMarker, tr("Play from marker") },
+		{ Configuration::PlayStep, tr("Play step") },
+		{ Configuration::Stop, tr("Stop") },
+		{ Configuration::FocusOnPattern, tr("Focus on pattern editor") },
+		{ Configuration::FocusOnOrder, tr("Focus on order list") },
+		{ Configuration::FocusOnInstrument, tr("Focus on instrument list") },
+		{ Configuration::ToggleEditJam, tr("Toggle edit/jam mode") },
+		{ Configuration::SetMarker, tr("Set marker") },
+		{ Configuration::PasteMix, tr("Paste and mix") },
+		{ Configuration::PasteOverwrite, tr("Paste and overwrite") },
+		{ Configuration::PasteInsert, tr("Paste and insert") },
+		{ Configuration::SelectAll, tr("Select all") },
+		{ Configuration::Deselect, tr("Deselect") },
+		{ Configuration::SelectRow, tr("Select row") },
+		{ Configuration::SelectColumn, tr("Select column") },
+		{ Configuration::SelectPattern, tr("Select pattern") },
+		{ Configuration::SelectOrder, tr("Select order") },
+		{ Configuration::GoToStep, tr("Go to step") },
+		{ Configuration::ToggleTrack, tr("Toggle track") },
+		{ Configuration::SoloTrack, tr("Solo track") },
+		{ Configuration::Interpolate, tr("Interpolate") },
+		{ Configuration::Reverse, tr("Reverse") },
+		{ Configuration::GoToPrevOrder, tr("Go to previous order") },
+		{ Configuration::GoToNextOrder, tr("Go to next order") },
+		{ Configuration::ToggleBookmark, tr("Toggle bookmark") },
+		{ Configuration::PrevBookmark, tr("Previous bookmark") },
+		{ Configuration::NextBookmark, tr("Next bookmark") },
+		{ Configuration::DecreaseNote, tr("Transpose, decrease note") },
+		{ Configuration::IncreaseNote, tr("Transpose, increase note") },
+		{ Configuration::DecreaseOctave, tr("Transpose, decrease octave") },
+		{ Configuration::IncreaseOctave, tr("Transpose, increase octave") },
+		{ Configuration::PrevInstrument, tr("Previous instrument") },
+		{ Configuration::NextInstrument, tr("Next instrument") },
+		{ Configuration::MaskInstrument, tr("Mask instrument") },
+		{ Configuration::MaskVolume, tr("Mask volume") },
+		{ Configuration::EditInstrument, tr("Edit instrument") },
+		{ Configuration::FollowMode, tr("Follow mode") },
+		{ Configuration::DuplicateOrder, tr("Duplicate order") },
+		{ Configuration::ClonePatterns, tr("Clone patterns") },
+		{ Configuration::CloneOrder, tr("Clone order") },
+		{ Configuration::ReplaceInstrument, tr("Replace instrument") },
+		{ Configuration::ExpandPattern, tr("Expand pattern") },
+		{ Configuration::ShrinkPattern, tr("Shrink pattern") },
+		{ Configuration::FineDecreaseValues, tr("Fine decrease values") },
+		{ Configuration::FineIncreaseValues, tr("Fine increase values") },
+		{ Configuration::CoarseDecreaseValues, tr("Coarse decrease values") },
+		{ Configuration::CoarseIncreaseValuse, tr("Coarse increase valuse") },
+		{ Configuration::ExpandEffect, tr("Expand effect column") },
+		{ Configuration::ShrinkEffect, tr("Shrink effect column") },
+		{ Configuration::PrevHighlighted, tr("Previous highlighted step") },
+		{ Configuration::NextHighlighted, tr("Next highlighted step") },
+		{ Configuration::IncreasePatternSize, tr("Increase pattern size") },
+		{ Configuration::DecreasePatternSize, tr("Decrease pattern size") },
+		{ Configuration::IncreaseEditStep, tr("Increase edit step") },
+		{ Configuration::DecreaseEditStep, tr("Decrease edit step") },
+		{ Configuration::DisplayEffectList, tr("Display effect list") },
+		{ Configuration::PreviousSong, tr("Previous song") },
+		{ Configuration::NextSong, tr("Next song") },
+		{ Configuration::JamVolumeUp, tr("Jam volume up") },
+		{ Configuration::JamVolumeDown, tr("Jam volume down") }
+	};
+	std::unordered_map<Configuration::ShortcutAction, std::string> shortcuts = configLocked->getShortcuts();
+	for (const auto& pair : shortcutsActions) {
+		int row = ui->shortcutsTreeWidget->topLevelItemCount();
+		auto item = new QTreeWidgetItem();
+		item->setText(0, pair.second);
+		ui->shortcutsTreeWidget->insertTopLevelItem(row, item);
+		auto widget = new QWidget();
+		widget->setLayout(new QHBoxLayout());
+		auto seq = new QKeySequenceEdit(utf8ToQString(shortcuts.at(pair.first)));
+		shortcutsMap_[pair.first] = seq;
+		auto button = new QToolButton();
+		button->setIcon(QIcon(":/icon/remove_inst"));
+		QObject::connect(button, &QToolButton::clicked, seq, &QKeySequenceEdit::clear);
+		auto layout = widget->layout();
+		layout->setSpacing(0);
+		layout->setMargin(0);
+		layout->addWidget(seq);
+		layout->addWidget(button);
+		ui->shortcutsTreeWidget->setItemWidget(item, 1, widget);
+	}
 
-	customLayoutKeysMap = {
+	ui->keyboardTypeComboBox->setCurrentIndex(static_cast<int>(configLocked->getNoteEntryLayout()));
+	customLayoutKeysMap_ = {
 		{ JamKey::LowC,     ui->lowCEdit },
 		{ JamKey::LowCS,    ui->lowCSEdit },
 		{ JamKey::LowD,     ui->lowDEdit },
@@ -104,11 +217,8 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 		{ JamKey::HighCS2, ui->highHighCSEdit },
 		{ JamKey::HighD2,  ui->highHighDEdit }
 	};
-	std::unordered_map<std::string, JamKey> customLayoutMapping = configLocked->getCustomLayoutKeys();
-	std::unordered_map<std::string, JamKey>::const_iterator customLayoutMappingIterator = customLayoutMapping.begin();
-	while (customLayoutMappingIterator != customLayoutMapping.end()) {
-		customLayoutKeysMap.at(customLayoutMappingIterator->second)->setKeySequence(QKeySequence(QString::fromStdString(customLayoutMappingIterator->first)));
-		customLayoutMappingIterator++;
+	for (const auto& pair : configLocked->getCustomLayoutKeys()) {
+		customLayoutKeysMap_.at(pair.second)->setKeySequence(QKeySequence(utf8ToQString(pair.first)));
 	}
 
 	// Emulation //
@@ -120,7 +230,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	int apiRow = -1;
 	int defApiRow = 0;
 	for (auto& name : apis) {
-		ui->soundAPIComboBox->addItem(QString::fromUtf8(name.c_str(), static_cast<int>(name.length())));
+		ui->soundAPIComboBox->addItem(utf8ToQString(name));
 		if (name == configLocked->getSoundAPI()) apiRow = ui->soundAPIComboBox->count() - 1;
 		if (name == curApi) defApiRow = apiRow = ui->soundAPIComboBox->count() - 1;
 	}
@@ -130,8 +240,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	int defDevRow = 0;
 	for (auto& info : QAudioDeviceInfo::availableDevices(QAudio::AudioOutput)) {
 		ui->soundDeviceComboBox->addItem(info.deviceName());
-		if (info.deviceName() == QString::fromUtf8(configLocked->getSoundDevice().c_str(),
-												   static_cast<int>(configLocked->getSoundDevice().length())))
+		if (info.deviceName() == utf8ToQString(configLocked->getSoundDevice()))
 			devRow = ui->soundDeviceComboBox->count() - 1;
 		if (info.deviceName() == QAudioDeviceInfo::defaultOutputDevice().deviceName()) {
 			defDevRow = ui->soundDeviceComboBox->count() - 1;
@@ -149,6 +258,11 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	case RealChipInterface::C86CTL:	ui->realChipComboBox->setCurrentIndex(2);	break;
 	}
 
+	MidiInterface &midiIntf = MidiInterface::instance();
+	if (midiIntf.supportsVirtualPort())
+		ui->midiInputNameLine->setPlaceholderText(tr("Virtual port"));
+	ui->midiInputNameLine->setText(QString::fromStdString(configLocked->getMidiInputPort()));
+
 	ui->sampleRateComboBox->addItem("44100Hz", 44100);
 	ui->sampleRateComboBox->addItem("48000Hz", 48000);
 	ui->sampleRateComboBox->addItem("55466Hz", 55466);
@@ -165,11 +279,6 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	});
 	ui->bufferLengthHorizontalSlider->setValue(static_cast<int>(configLocked->getBufferLength()));
 
-	// Midi //
-	MidiInterface &midiIntf = MidiInterface::instance();
-	if (midiIntf.supportsVirtualPort())
-		ui->midiInputNameLine->setPlaceholderText(tr("Virtual port"));
-	ui->midiInputNameLine->setText(QString::fromStdString(configLocked->getMidiInputPort()));
 
 	// Mixer //
 	ui->masterMixerSlider->setText(tr("Master"));
@@ -208,21 +317,13 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	ui->colorsTreeWidget->setColumnWidth(0, 250);
 	updateColorTree();
 
-	ui->ptnHdFontComboBox->setCurrentFont(
-				QFont(QString::fromUtf8(configLocked->getPatternEditorHeaderFont().c_str(),
-										static_cast<int>(configLocked->getPatternEditorHeaderFont().size()))));
+	ui->ptnHdFontComboBox->setCurrentFont(QFont(utf8ToQString(configLocked->getPatternEditorHeaderFont())));
 	ui->ptnHdFontSizeComboBox->setCurrentText(QString::number(configLocked->getPatternEditorHeaderFontSize()));
-	ui->ptnRowFontComboBox->setCurrentFont(
-				QFont(QString::fromUtf8(configLocked->getPatternEditorRowsFont().c_str(),
-										static_cast<int>(configLocked->getPatternEditorRowsFont().size()))));
+	ui->ptnRowFontComboBox->setCurrentFont(QFont(utf8ToQString(configLocked->getPatternEditorRowsFont())));
 	ui->ptnRowFontSizeComboBox->setCurrentText(QString::number(configLocked->getPatternEditorRowsFontSize()));
-	ui->odrHdFontComboBox->setCurrentFont(
-				QFont(QString::fromUtf8(configLocked->getOrderListHeaderFont().c_str(),
-										static_cast<int>(configLocked->getOrderListHeaderFont().size()))));
+	ui->odrHdFontComboBox->setCurrentFont(QFont(utf8ToQString(configLocked->getOrderListHeaderFont())));
 	ui->odrHdFontSizeComboBox->setCurrentText(QString::number(configLocked->getOrderListHeaderFontSize()));
-	ui->odrRowFontComboBox->setCurrentFont(
-				QFont(QString::fromUtf8(configLocked->getOrderListRowsFont().c_str(),
-										static_cast<int>(configLocked->getOrderListRowsFont().size()))));
+	ui->odrRowFontComboBox->setCurrentFont(QFont(utf8ToQString(configLocked->getOrderListRowsFont())));
 	ui->odrRowFontSizeComboBox->setCurrentText(QString::number(configLocked->getOrderListRowsFontSize()));
 }
 
@@ -247,25 +348,27 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	configLocked->setRetrieveChannelState(fromCheckState(ui->generalSettingsListWidget->item(8)->checkState()));
 	configLocked->setEnableTranslation(fromCheckState(ui->generalSettingsListWidget->item(9)->checkState()));
 	configLocked->setShowFMDetuneAsSigned(fromCheckState(ui->generalSettingsListWidget->item(10)->checkState()));
-	configLocked->setShowWaveVisual(fromCheckState(ui->generalSettingsListWidget->item(11)->checkState()));
-	configLocked->setFill00ToEffectValue(fromCheckState(ui->generalSettingsListWidget->item(12)->checkState()));
-	configLocked->setAutosetInstrument(fromCheckState(ui->generalSettingsListWidget->item(13)->checkState()));
-	configLocked->setMoveCursorByHorizontalScroll(fromCheckState(ui->generalSettingsListWidget->item(14)->checkState()));
+	configLocked->setFill00ToEffectValue(fromCheckState(ui->generalSettingsListWidget->item(11)->checkState()));
+	configLocked->setMoveCursorByHorizontalScroll(fromCheckState(ui->generalSettingsListWidget->item(12)->checkState()));
+	configLocked->setOverwriteUnusedUneditedPropety(fromCheckState(ui->generalSettingsListWidget->item(13)->checkState()));
+	configLocked->setWriteOnlyUsedSamples(fromCheckState(ui->generalSettingsListWidget->item(14)->checkState()));
+	configLocked->setReflectInstrumentNumberChange(fromCheckState(ui->generalSettingsListWidget->item(15)->checkState()));
+	configLocked->setFixJammingVolume(fromCheckState(ui->generalSettingsListWidget->item(16)->checkState()));
 
 	// Edit settings
 	configLocked->setPageJumpLength(static_cast<size_t>(ui->pageJumpLengthSpinBox->value()));
 
 	// Keys
-	configLocked->setKeyOffKey(ui->keyOffKeySequenceEdit->keySequence().toString().toStdString());
-	configLocked->setOctaveUpKey(ui->octaveUpKeySequenceEdit->keySequence().toString().toStdString());
-	configLocked->setOctaveDownKey(ui->octaveDownKeySequenceEdit->keySequence().toString().toStdString());
-	configLocked->setEchoBufferKey(ui->echoBufferKeySequenceEdit->keySequence().toString().toStdString());
+	std::unordered_map<Configuration::ShortcutAction, std::string> shortcuts;
+	for (const auto& pair: shortcutsMap_) {
+		shortcuts[pair.first] = pair.second->keySequence().toString().toStdString();
+	}
+	configLocked->setShortcuts(shortcuts);
+
 	configLocked->setNoteEntryLayout(static_cast<Configuration::KeyboardLayout>(ui->keyboardTypeComboBox->currentIndex()));
 	std::unordered_map<std::string, JamKey> customLayoutNewKeys;
-	std::unordered_map<JamKey, QKeySequenceEdit *>::const_iterator customLayoutKeysMapIterator = customLayoutKeysMap.begin();
-	while (customLayoutKeysMapIterator != customLayoutKeysMap.end()) {
-		customLayoutNewKeys[customLayoutKeysMapIterator->second->keySequence().toString().toStdString()] = customLayoutKeysMapIterator->first;
-		customLayoutKeysMapIterator++;
+	for (const auto& pair : customLayoutKeysMap_) {
+		customLayoutNewKeys[pair.second->keySequence().toString().toStdString()] = pair.first;
 	}
 	configLocked->setCustomLayoutKeys(customLayoutNewKeys);
 
@@ -282,12 +385,10 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	configLocked->setSoundAPI(ui->soundAPIComboBox->currentText().toUtf8().toStdString());
 	configLocked->setRealChipInterface(static_cast<RealChipInterface>(
 										   ui->realChipComboBox->currentData(Qt::UserRole).toInt()));
+	configLocked->setMidiInputPort(ui->midiInputNameLine->text().toStdString());
 	configLocked->setSampleRate(ui->sampleRateComboBox->currentData(Qt::UserRole).toUInt());
 	configLocked->setBufferLength(static_cast<size_t>(ui->bufferLengthHorizontalSlider->value()));
 
-	// Midi //
-	configLocked->setMidiInputPort(ui->midiInputNameLine->text().toStdString());
-
 	// Mixer //
 	configLocked->setMixerVolumeMaster(ui->masterMixerSlider->value());
 	configLocked->setMixerVolumeFM(ui->fmMixerSlider->value() * 0.1);
@@ -327,6 +428,7 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	pl->ptnMuteColor = ptnColors->child(23)->data(1, Qt::BackgroundRole).value<QColor>();
 	pl->ptnUnmuteColor = ptnColors->child(24)->data(1, Qt::BackgroundRole).value<QColor>();
 	pl->ptnBackColor = ptnColors->child(25)->data(1, Qt::BackgroundRole).value<QColor>();
+	pl->ptnMarkerColor = ptnColors->child(26)->data(1, Qt::BackgroundRole).value<QColor>();
 
 	QTreeWidgetItem* odrColors = ui->colorsTreeWidget->topLevelItem(1);
 	pl->odrDefTextColor = odrColors->child(0)->data(1, Qt::BackgroundRole).value<QColor>();
@@ -372,57 +474,9 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 /***** General *****/
 void ConfigurationDialog::on_generalSettingsListWidget_itemSelectionChanged()
 {
-	QString text;
-	switch (ui->generalSettingsListWidget->currentRow()) {
-	case 0:		// Warp cursor
-		text = tr("Warp the cursor around the edges of the pattern editor.");
-		break;
-	case 1:		// Warp across orders
-		text = tr("Move to previous or next order when reaching top or bottom in the pattern editor.");
-		break;
-	case 2:		// Show row numbers in hex
-		text = tr("Display row numbers and the playback position on the status bar in hexadecimal.");
-		break;
-	case 3:		// Preview previous/next orders
-		text = tr("Preview previous and next orders in the pattern editor.");
-		break;
-	case 4:		// Backup modeles
-		text = tr("Create a backup copy of the existing file when saving a module.");
-		break;
-	case 5:		// Don't select on double click
-		text = tr("Don't select the whole track when double-clicking in the pattern editor.");
-		break;
-	case 6:		// Reverse FM volume order
-		text = tr("Reverse the order of FM volume so that 00 is the quietest in the pattern editor.");
-		break;
-	case 7:		// Move cursor to right
-		text = tr("Move the cursor to right after entering effects in the pattern editor.");
-		break;
-	case 8:		// Retrieve channel state
-		text = tr("Reconstruct the current channel's state from previous orders upon playing.");
-		break;
-	case 9:		// Enable translation
-		text = tr("Translate to your language from the next launch. "
-				  "See readme to check supported languages.");
-		break;
-	case 10:	// Show FM detune as signed
-		text = tr("Display FM detune values as signed numbers in the FM envelope editor.");
-		break;
-	case 11:	// Show wave visual
-		text = tr("Enable an oscilloscope which displays a waveform of the sound output.");
-		break;
-	case 12:	// Fill 00 to effect value
-		text = tr("Fill 00 to effect value column upon entering effect id.");
-		break;
-	case 13:	// Autoset instrument
-		text = tr("Set current instrument upon entering note.");
-		break;
-	case 14:	// Move cursor by horizontal scroll
-		text = tr("Move the cursor position by cell with horizontal scroll bar in the order list and the pattern editor.");
-		break;
-	default:
-		text = "";
-		break;
+	QString text("");
+	if (QListWidgetItem* item = ui->generalSettingsListWidget->currentItem()) {
+		text = item->data(Qt::UserRole).toString();
 	}
 	ui->descPlainTextEdit->setPlainText(tr("Description: %1").arg(text));
 }
@@ -434,7 +488,7 @@ void ConfigurationDialog::on_mixerResetPushButton_clicked()
 	ui->ssgMixerSlider->setValue(0);
 }
 
-/***** MIDI *****/
+/***** Sound *****/
 void ConfigurationDialog::on_midiInputChoiceButton_clicked()
 {
 	QToolButton *button = ui->midiInputChoiceButton;
@@ -524,8 +578,7 @@ void ConfigurationDialog::updateEnvelopeSetUi()
 
 	ui->envelopeTypeListWidget->clear();
 	for (auto& texts : fmEnvelopeTexts_)
-		ui->envelopeTypeListWidget->addItem(
-					QString::fromUtf8(texts.name.c_str(), static_cast<int>(texts.name.length())));
+		ui->envelopeTypeListWidget->addItem(utf8ToQString(texts.name));
 }
 
 /***** Keys *****/
@@ -542,11 +595,27 @@ void ConfigurationDialog::on_customLayoutResetButton_clicked()
 	std::unordered_map<std::string, JamKey> QWERTYLayoutMapping = config_.lock()->mappingLayouts.at (Configuration::KeyboardLayout::QWERTY);
 	std::unordered_map<std::string, JamKey>::const_iterator QWERTYLayoutMappingIterator = QWERTYLayoutMapping.begin();
 	while (QWERTYLayoutMappingIterator != QWERTYLayoutMapping.end()) {
-		customLayoutKeysMap.at(QWERTYLayoutMappingIterator->second)->setKeySequence(QKeySequence(QString::fromStdString(QWERTYLayoutMappingIterator->first)));
+		customLayoutKeysMap_.at(QWERTYLayoutMappingIterator->second)->setKeySequence(QKeySequence(QString::fromStdString(QWERTYLayoutMappingIterator->first)));
 		QWERTYLayoutMappingIterator++;
 	}
 }
 
+void ConfigurationDialog::addShortcutItem(QString action, std::string shortcut)
+{
+	int row = ui->shortcutsTreeWidget->topLevelItemCount();
+	auto titem = new QTreeWidgetItem();
+	titem->setText(0, action);
+	ui->shortcutsTreeWidget->insertTopLevelItem(row, titem);
+	ui->shortcutsTreeWidget->setItemWidget(titem, 1, new QKeySequenceEdit(utf8ToQString(shortcut)));
+}
+
+std::string ConfigurationDialog::getShortcutString(int row) const
+{
+	return qobject_cast<QKeySequenceEdit*>(
+				ui->shortcutsTreeWidget->itemWidget(ui->shortcutsTreeWidget->topLevelItem(row), 1)
+				)->keySequence().toString().toStdString();
+}
+
 /***** Appearance *****/
 void ConfigurationDialog::on_colorEditPushButton_clicked()
 {
@@ -615,6 +684,7 @@ void ConfigurationDialog::updateColorTree()
 	ptnColors->child(23)->setData(1, Qt::BackgroundRole, pl->ptnMuteColor);
 	ptnColors->child(24)->setData(1, Qt::BackgroundRole, pl->ptnUnmuteColor);
 	ptnColors->child(25)->setData(1, Qt::BackgroundRole, pl->ptnBackColor);
+	ptnColors->child(26)->setData(1, Qt::BackgroundRole, pl->ptnMarkerColor);
 
 	QTreeWidgetItem* odrColors = ui->colorsTreeWidget->topLevelItem(1);
 	odrColors->child(0)->setData(1, Qt::BackgroundRole, pl->odrDefTextColor);
