@@ -1,20 +1,125 @@
 #include "configuration_handler.hpp"
 #include <vector>
 #include <unordered_map>
-#include <QSettings>
 #include "jam_manager.hpp"
 #include"enum_hash.hpp"
 
 // config path (*nix): ~/.config/<organization>/<application>.ini
-const QString ConfigurationHandler::organization = "BambooTracker";
-const QString ConfigurationHandler::application = "BambooTracker";
+const QString ConfigurationHandler::ORGANIZATION_ = "BambooTracker";
+const QString ConfigurationHandler::APPLICATION_ = "BambooTracker";
+
+const std::unordered_map<Configuration::ShortcutAction, QString> ConfigurationHandler::SHORTCUTS_NAME_MAP_ = {
+	{ Configuration::KeyOff, "keyOff" },
+	{ Configuration::OctaveUp, "octaveUp" },
+	{ Configuration::OctaveDown, "octaveDown" },
+	{ Configuration::EchoBuffer, "echoBuffer" },
+	{ Configuration::PlayAndStop, "playAndStop" },
+	{ Configuration::Play, "play" },
+	{ Configuration::PlayFromStart, "playFromStart" },
+	{ Configuration::PlayPattern, "playPattern" },
+	{ Configuration::PlayFromCursor, "playFromCursor" },
+	{ Configuration::PlayFromMarker, "playFromMarker" },
+	{ Configuration::PlayStep, "playStep" },
+	{ Configuration::Stop, "stop" },
+	{ Configuration::FocusOnPattern, "ocusOnPattern" },
+	{ Configuration::FocusOnOrder, "focusOnOrder" },
+	{ Configuration::FocusOnInstrument, "focusOnInstrument" },
+	{ Configuration::ToggleEditJam, "toggleEditJam" },
+	{ Configuration::SetMarker, "setMarker" },
+	{ Configuration::PasteMix, "pasteMix" },
+	{ Configuration::PasteOverwrite, "pasteOverwrite" },
+	{ Configuration::PasteInsert, "pasteInsert" },
+	{ Configuration::SelectAll, "selectAll" },
+	{ Configuration::Deselect, "deselect" },
+	{ Configuration::SelectRow, "selectRow" },
+	{ Configuration::SelectColumn, "selectColumn" },
+	{ Configuration::SelectPattern, "selectPattern" },
+	{ Configuration::SelectOrder, "selectOrder" },
+	{ Configuration::GoToStep, "goToStep" },
+	{ Configuration::ToggleTrack, "toggleTrack" },
+	{ Configuration::SoloTrack, "soloTrack" },
+	{ Configuration::Interpolate, "interpolate" },
+	{ Configuration::Reverse, "reverse" },
+	{ Configuration::GoToPrevOrder, "goToPrevOrder" },
+	{ Configuration::GoToNextOrder, "goToNextOrder" },
+	{ Configuration::ToggleBookmark, "toggleBookmark" },
+	{ Configuration::PrevBookmark, "prevBookmark" },
+	{ Configuration::NextBookmark, "nextBookmark" },
+	{ Configuration::DecreaseNote, "decreaseNote" },
+	{ Configuration::IncreaseNote, "increaseNote" },
+	{ Configuration::DecreaseOctave, "decreaseOctave" },
+	{ Configuration::IncreaseOctave, "increaseOctave" },
+	{ Configuration::PrevInstrument, "prevInstrument" },
+	{ Configuration::NextInstrument, "nextInstrument" },
+	{ Configuration::MaskInstrument, "maskInstrument" },
+	{ Configuration::MaskVolume, "maskVolume" },
+	{ Configuration::EditInstrument, "editInstrument" },
+	{ Configuration::FollowMode, "followMode" },
+	{ Configuration::DuplicateOrder, "duplicateOrder" },
+	{ Configuration::ClonePatterns, "clonePatterns" },
+	{ Configuration::CloneOrder, "cloneOrder" },
+	{ Configuration::ReplaceInstrument, "replaceInstrument" },
+	{ Configuration::ExpandPattern, "expandPattern" },
+	{ Configuration::ShrinkPattern, "shrinkPattern" },
+	{ Configuration::FineDecreaseValues, "fineDecreaseValues" },
+	{ Configuration::FineIncreaseValues, "fineIncreaseValues" },
+	{ Configuration::CoarseDecreaseValues, "coarseDecreaseValues" },
+	{ Configuration::CoarseIncreaseValuse, "coarseIncreaseValuse" },
+	{ Configuration::ExpandEffect, "expandEffect" },
+	{ Configuration::ShrinkEffect, "shrinkEffect" },
+	{ Configuration::PrevHighlighted, "prevHighlightedStep" },
+	{ Configuration::NextHighlighted, "nextHighlightedStep" },
+	{ Configuration::IncreasePatternSize, "incPtnSize" },
+	{ Configuration::DecreasePatternSize, "decPtnSize" },
+	{ Configuration::IncreaseEditStep, "incEditStep" },
+	{ Configuration::DecreaseEditStep, "decEditStep" },
+	{ Configuration::DisplayEffectList, "dispEffectList" },
+	{ Configuration::PreviousSong, "prevSong" },
+	{ Configuration::NextSong, "nextSong" },
+	{ Configuration::JamVolumeUp, "jamVolumeUp" },
+	{ Configuration::JamVolumeDown, "jamVolumeDown" }
+};
+
+const std::unordered_map<JamKey, QString> ConfigurationHandler::JAM_KEY_NAME_MAP_ = {
+	{JamKey::LowC,     "lowC"},
+	{JamKey::LowCS,    "lowCS"},
+	{JamKey::LowD,     "lowD"},
+	{JamKey::LowDS,    "lowDS"},
+	{JamKey::LowE,     "lowE"},
+	{JamKey::LowF,     "lowF"},
+	{JamKey::LowFS,    "lowFS"},
+	{JamKey::LowG,     "lowG"},
+	{JamKey::LowGS,    "lowGS"},
+	{JamKey::LowA,     "lowA"},
+	{JamKey::LowAS,    "lowAS"},
+	{JamKey::LowB,     "lowB"},
+	{JamKey::LowC2,   "lowHighC"},
+	{JamKey::LowCS2,  "lowHighCS"},
+	{JamKey::LowD2,   "lowHighD"},
+
+	{JamKey::HighC,    "highC"},
+	{JamKey::HighCS,   "highCS"},
+	{JamKey::HighD,    "highD"},
+	{JamKey::HighDS,   "highDS"},
+	{JamKey::HighE,    "highE"},
+	{JamKey::HighF,    "highF"},
+	{JamKey::HighFS,   "highFS"},
+	{JamKey::HighG,    "highG"},
+	{JamKey::HighGS,   "highGS"},
+	{JamKey::HighA,    "highA"},
+	{JamKey::HighAS,   "highAS"},
+	{JamKey::HighB,    "highB"},
+	{JamKey::HighC2,  "highHighC"},
+	{JamKey::HighCS2, "highHighCS"},
+	{JamKey::HighD2,  "highHighD"}
+};
 
 ConfigurationHandler::ConfigurationHandler() {}
 
 bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config)
 {
 	try {
-		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ConfigurationHandler::organization, ConfigurationHandler::application);
+		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ConfigurationHandler::ORGANIZATION_, ConfigurationHandler::APPLICATION_);
 		std::shared_ptr<Configuration> configLocked = config.lock();
 
 		// Internal //
@@ -29,10 +134,32 @@ bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config
 		settings.setValue("instrumentFMWindowHeight",  configLocked->getInstrumentFMWindowHeight());
 		settings.setValue("instrumentSSGWindowWidth",  configLocked->getInstrumentSSGWindowWidth());
 		settings.setValue("instrumentSSGWindowHeight", configLocked->getInstrumentSSGWindowHeight());
+		settings.setValue("instrumentADPCMWindowWidth",  configLocked->getInstrumentADPCMWindowWidth());
+		settings.setValue("instrumentADPCMWindowHeight", configLocked->getInstrumentADPCMWindowHeight());
+		settings.setValue("instrumentDrumkitWindowWidth",  configLocked->getInstrumentDrumkitWindowWidth());
+		settings.setValue("instrumentDrumkitWindowHeight", configLocked->getInstrumentDrumkitWindowHeight());
 		settings.setValue("followMode",		configLocked->getFollowMode());
 		settings.setValue("workingDirectory",          QString::fromStdString(configLocked->getWorkingDirectory()));
 		settings.setValue("instrumentOpenFormat",		configLocked->getInstrumentOpenFormat());
 		settings.setValue("bankOpenFormat",				configLocked->getBankOpenFormat());
+		settings.setValue("instrumentMask",				configLocked->getInstrumentMask());
+		settings.setValue("volumeMask",					configLocked->getVolumeMask());
+		settings.setValue("visibleToolbar",				configLocked->getVisibleToolbar());
+		settings.setValue("visibleStatusBar",			configLocked->getVisibleStatusBar());
+		settings.setValue("visibleWaveView",			configLocked->getVisibleWaveView());
+		settings.setValue("pasteMode",					configLocked->getPasteMode());
+		auto& mainTbConfig = configLocked->getMainToolbarConfiguration();
+		settings.setValue("mainToolbarPosition",		mainTbConfig.getPosition());
+		settings.setValue("mainToolbarNumber",			mainTbConfig.getNumber());
+		settings.setValue("hasBreakBeforeMainToolbar",	mainTbConfig.hasBreakBefore());
+		settings.setValue("mainToolbarX",				mainTbConfig.getX());
+		settings.setValue("mainToolbarY",				mainTbConfig.getY());
+		auto& subTbConfig = configLocked->getSubToolbarConfiguration();
+		settings.setValue("subToolbarPosition",			subTbConfig.getPosition());
+		settings.setValue("subToolbarNumber",			subTbConfig.getNumber());
+		settings.setValue("hasBreakBeforesubToolbar",	subTbConfig.hasBreakBefore());
+		settings.setValue("subToolbarX",				subTbConfig.getX());
+		settings.setValue("subToolbarY",				subTbConfig.getY());
 		settings.endGroup();
 
 		// General //
@@ -49,10 +176,12 @@ bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config
 		settings.setValue("retrieveChannelState",	configLocked->getRetrieveChannelState());
 		settings.setValue("enableTranslation",		configLocked->getEnableTranslation());
 		settings.setValue("showFMDetuneAsSigned",	configLocked->getShowFMDetuneAsSigned());
-		settings.setValue("showWaveVisual",			configLocked->getShowWaveVisual());
 		settings.setValue("fill00ToEffectValue",	configLocked->getFill00ToEffectValue());
-		settings.setValue("autosetInstrument",		configLocked->getAutosetInstrument());
 		settings.setValue("moveCursorByHScroll",	configLocked->getMoveCursorByHorizontalScroll());
+		settings.setValue("overwriteUnusedUnedited",	configLocked->getOverwriteUnusedUneditedPropety());
+		settings.setValue("writeOnlyUsedSamples",	configLocked->getWriteOnlyUsedSamples());
+		settings.setValue("reflectInstNumChange",	configLocked->getReflectInstrumentNumberChange());
+		settings.setValue("fixJammingVolume",		configLocked->getFixJammingVolume());
 		settings.endGroup();
 
 		// Edit settings
@@ -64,70 +193,20 @@ bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config
 
 		// Keys
 		settings.beginGroup("Keys");
-		settings.setValue("keyOffKey",
-						  QString::fromUtf8(configLocked->getKeyOffKey().c_str(),
-											static_cast<int>(configLocked->getKeyOffKey().length())));
-		settings.setValue("octaveUpKey",
-						  QString::fromUtf8(configLocked->getOctaveUpKey().c_str(),
-											static_cast<int>(configLocked->getOctaveUpKey().length())));
-		settings.setValue("octaveDownKey",
-						  QString::fromUtf8(configLocked->getOctaveDownKey().c_str(),
-											static_cast<int>(configLocked->getOctaveDownKey().length())));
-		settings.setValue("echoBufferKey",
-						  QString::fromUtf8(configLocked->getEchoBufferKey().c_str(),
-											static_cast<int>(configLocked->getEchoBufferKey().length())));
-		settings.setValue("noteEntryLayout",	static_cast<int>(configLocked->getNoteEntryLayout()));
-		std::unordered_map<std::string, JamKey> customLayoutMapping = configLocked->getCustomLayoutKeys();
-		const std::unordered_map<JamKey, std::string> keyToNameMapping = {
-			{JamKey::LowC,     "lowC"},
-			{JamKey::LowCS,    "lowCS"},
-			{JamKey::LowD,     "lowD"},
-			{JamKey::LowDS,    "lowDS"},
-			{JamKey::LowE,     "lowE"},
-			{JamKey::LowF,     "lowF"},
-			{JamKey::LowFS,    "lowFS"},
-			{JamKey::LowG,     "lowG"},
-			{JamKey::LowGS,    "lowGS"},
-			{JamKey::LowA,     "lowA"},
-			{JamKey::LowAS,    "lowAS"},
-			{JamKey::LowB,     "lowB"},
-			{JamKey::LowC2,   "lowHighC"},
-			{JamKey::LowCS2,  "lowHighCS"},
-			{JamKey::LowD2,   "lowHighD"},
-
-			{JamKey::HighC,    "highC"},
-			{JamKey::HighCS,   "highCS"},
-			{JamKey::HighD,    "highD"},
-			{JamKey::HighDS,   "highDS"},
-			{JamKey::HighE,    "highE"},
-			{JamKey::HighF,    "highF"},
-			{JamKey::HighFS,   "highFS"},
-			{JamKey::HighG,    "highG"},
-			{JamKey::HighGS,   "highGS"},
-			{JamKey::HighA,    "highA"},
-			{JamKey::HighAS,   "highAS"},
-			{JamKey::HighB,    "highB"},
-			{JamKey::HighC2,  "highHighC"},
-			{JamKey::HighCS2, "highHighCS"},
-			{JamKey::HighD2,  "highHighD"}
-		};
-		std::unordered_map<std::string, JamKey>::const_iterator customLayoutMappingIterator = customLayoutMapping.begin();
-		while (customLayoutMappingIterator != customLayoutMapping.end()) {
-			settings.setValue(QString::fromStdString("customLayout_" + keyToNameMapping.at(customLayoutMappingIterator->second)),
-							  QString::fromUtf8(customLayoutMappingIterator->first.c_str(), static_cast<int> (customLayoutMappingIterator->first.length())));
-			customLayoutMappingIterator++;
+		for (const auto& pair : configLocked->getShortcuts()) {
+			settings.setValue("shortcut_" + SHORTCUTS_NAME_MAP_.at(pair.first), utf8ToQString(pair.second));
+		}
+		settings.setValue("noteEntryLayout", static_cast<int>(configLocked->getNoteEntryLayout()));
+		for (const auto& pair : configLocked->getCustomLayoutKeys()) {
+			settings.setValue("customLayout_" + JAM_KEY_NAME_MAP_.at(pair.second), utf8ToQString(pair.first));
 		}
 		settings.endGroup();
 
 		// Sound //
 		settings.beginGroup("Sound");
-		settings.setValue("soundAPI",
-						  QString::fromUtf8(configLocked->getSoundAPI().c_str(),
-											static_cast<int>(configLocked->getSoundAPI().length())));
-		settings.setValue("soundDevice",
-						  QString::fromUtf8(configLocked->getSoundDevice().c_str(),
-											static_cast<int>(configLocked->getSoundDevice().length())));
-                settings.setValue("realChipInterface",	static_cast<int>(configLocked->getRealChipInterface()));
+		settings.setValue("soundAPI", utf8ToQString(configLocked->getSoundAPI()));
+		settings.setValue("soundDevice", utf8ToQString(configLocked->getSoundDevice()));
+		settings.setValue("realChipInterface",	static_cast<int>(configLocked->getRealChipInterface()));
 		settings.setValue("emulator",		configLocked->getEmulator());
 		settings.setValue("sampleRate",   static_cast<int>(configLocked->getSampleRate()));
 		settings.setValue("bufferLength", static_cast<int>(configLocked->getBufferLength()));
@@ -151,7 +230,7 @@ bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config
 		int n = 0;
 		for (auto texts : config.lock()->getFMEnvelopeTexts()) {
 			settings.setArrayIndex(n++);
-			settings.setValue("type", QString::fromUtf8(texts.name.c_str(), static_cast<int>(texts.name.length())));
+			settings.setValue("type", utf8ToQString(texts.name));
 			QStringList typeList;
 			std::transform(texts.texts.begin(), texts.texts.end(), std::back_inserter(typeList),
 						   [](FMEnvelopeTextType type) { return QString::number(static_cast<int>(type)); });
@@ -180,7 +259,7 @@ bool ConfigurationHandler::saveConfiguration(std::weak_ptr<Configuration> config
 bool ConfigurationHandler::loadConfiguration(std::weak_ptr<Configuration> config)
 {
 	try {
-		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ConfigurationHandler::organization, ConfigurationHandler::application);
+		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ConfigurationHandler::ORGANIZATION_, ConfigurationHandler::APPLICATION_);
 		std::shared_ptr<Configuration> configLocked = config.lock();
 
 		// Internal //
@@ -195,10 +274,32 @@ bool ConfigurationHandler::loadConfiguration(std::weak_ptr<Configuration> config
 		configLocked->setInstrumentFMWindowHeight(settings.value("instrumentFMWindowHeight", configLocked->getInstrumentFMWindowHeight()).toInt());
 		configLocked->setInstrumentSSGWindowWidth(settings.value("instrumentSSGWindowWidth", configLocked->getInstrumentSSGWindowWidth()).toInt());
 		configLocked->setInstrumentSSGWindowHeight(settings.value("instrumentSSGWindowHeight", configLocked->getInstrumentSSGWindowHeight()).toInt());
+		configLocked->setInstrumentADPCMWindowWidth(settings.value("instrumentADPCMWindowWidth", configLocked->getInstrumentADPCMWindowWidth()).toInt());
+		configLocked->setInstrumentADPCMWindowHeight(settings.value("instrumentADPCMWindowHeight", configLocked->getInstrumentADPCMWindowHeight()).toInt());
+		configLocked->setInstrumentDrumkitWindowWidth(settings.value("instrumentDrumkitWindowWidth", configLocked->getInstrumentDrumkitWindowWidth()).toInt());
+		configLocked->setInstrumentDrumkitWindowHeight(settings.value("instrumentDrumkitWindowHeight", configLocked->getInstrumentDrumkitWindowHeight()).toInt());
 		configLocked->setFollowMode(settings.value("followMode", configLocked->getFollowMode()).toBool());
 		configLocked->setWorkingDirectory(settings.value("workingDirectory", QString::fromStdString(configLocked->getWorkingDirectory())).toString().toStdString());
 		configLocked->setInstrumentOpenFormat(settings.value("instrumentOpenFormat", configLocked->getInstrumentOpenFormat()).toInt());
 		configLocked->setBankOpenFormat(settings.value("bankOpenFormat", configLocked->getBankOpenFormat()).toInt());
+		configLocked->setInstrumentMask(settings.value("instrumentMask", configLocked->getInstrumentMask()).toBool());
+		configLocked->setVolumeMask(settings.value("volumeMask", configLocked->getVolumeMask()).toBool());
+		configLocked->setVisibleToolbar(settings.value("visibleToolbar", configLocked->getVisibleToolbar()).toBool());
+		configLocked->setVisibleStatusBar(settings.value("visibleStatusBar", configLocked->getVisibleStatusBar()).toBool());
+		configLocked->setVisibleWaveView(settings.value("visibleWaveView", configLocked->getVisibleWaveView()).toBool());
+		configLocked->setPasteMode(static_cast<Configuration::PasteMode>(settings.value("pasteMode", configLocked->getPasteMode()).toInt()));
+		auto& mainTbConfig = configLocked->getMainToolbarConfiguration();
+		mainTbConfig.setPosition(static_cast<Configuration::ToolbarConfiguration::ToolbarPosition>(settings.value("mainToolbarPosition", mainTbConfig.getPosition()).toInt()));
+		mainTbConfig.setNumber(settings.value("mainToolbarNumber", mainTbConfig.getNumber()).toInt());
+		mainTbConfig.setBreakBefore(settings.value("hasBreakBeforeMainToolbar", mainTbConfig.hasBreakBefore()).toBool());
+		mainTbConfig.setX(settings.value("mainToolbarX", mainTbConfig.getX()).toInt());
+		mainTbConfig.setY(settings.value("mainToolbarY", mainTbConfig.getY()).toInt());
+		auto& subTbConfig = configLocked->getSubToolbarConfiguration();
+		subTbConfig.setPosition(static_cast<Configuration::ToolbarConfiguration::ToolbarPosition>(settings.value("subToolbarPosition", subTbConfig.getPosition()).toInt()));
+		subTbConfig.setNumber(settings.value("subToolbarNumber", subTbConfig.getNumber()).toInt());
+		subTbConfig.setBreakBefore(settings.value("hasBreakBeforesubToolbar", subTbConfig.hasBreakBefore()).toBool());
+		subTbConfig.setX(settings.value("subToolbarX", subTbConfig.getX()).toInt());
+		subTbConfig.setY(settings.value("subToolbarY", subTbConfig.getY()).toInt());
 		settings.endGroup();
 
 		// General //
@@ -215,10 +316,20 @@ bool ConfigurationHandler::loadConfiguration(std::weak_ptr<Configuration> config
 		configLocked->setRetrieveChannelState(settings.value("retrieveChannelState", configLocked->getRetrieveChannelState()).toBool());
 		configLocked->setEnableTranslation(settings.value("enableTranslation", configLocked->getEnableTranslation()).toBool());
 		configLocked->setShowFMDetuneAsSigned(settings.value("showFMDetuneAsSigned", configLocked->getShowFMDetuneAsSigned()).toBool());
-		configLocked->setShowWaveVisual(settings.value("showWaveVisual", configLocked->getShowWaveVisual()).toBool());
 		configLocked->setFill00ToEffectValue(settings.value("fill00ToEffectValue", configLocked->getFill00ToEffectValue()).toBool());
-		configLocked->setAutosetInstrument(settings.value("autosetInstrument", configLocked->getAutosetInstrument()).toBool());
 		configLocked->setMoveCursorByHorizontalScroll(settings.value("moveCursorByHScroll", configLocked->getMoveCursorByHorizontalScroll()).toBool());
+		configLocked->setOverwriteUnusedUneditedPropety(settings.value("overwriteUnusedUnedited", configLocked->getOverwriteUnusedUneditedPropety()).toBool());
+		configLocked->setWriteOnlyUsedSamples(settings.value("writeOnlyUsedSamples", configLocked->getWriteOnlyUsedSamples()).toBool());
+		configLocked->setReflectInstrumentNumberChange(settings.value("reflectInstNumChange", configLocked->getReflectInstrumentNumberChange()).toBool());
+		configLocked->setFixJammingVolume(settings.value("fixJammingVolume", configLocked->getFixJammingVolume()).toBool());
+		if (settings.contains("autosetInstrument")) {	// For compatibility before v0.4.0
+			configLocked->setInstrumentMask(!settings.value("autosetInstrument").toBool());
+			settings.remove("autosetInstrument");
+		}
+		if (settings.contains("showWaveVisual")) {	// For compatibility before v0.4.2
+			configLocked->setVisibleWaveView(settings.value("showWaveVisual").toBool());
+			settings.remove("showWaveVisual");
+		}
 		settings.endGroup();
 
 		// Edit settings
@@ -234,58 +345,26 @@ bool ConfigurationHandler::loadConfiguration(std::weak_ptr<Configuration> config
 
 		// Keys
 		settings.beginGroup("Keys");
-		configLocked->setKeyOffKey(settings.value("keyOffKey", QString::fromStdString(configLocked->getKeyOffKey())).toString().toUtf8().toStdString());
-		configLocked->setOctaveUpKey(settings.value("octaveUpKey", QString::fromStdString(configLocked->getOctaveUpKey())).toString().toUtf8().toStdString());
-		configLocked->setOctaveDownKey(settings.value("octaveDownKey", QString::fromStdString(configLocked->getOctaveDownKey())).toString().toUtf8().toStdString());
-		configLocked->setEchoBufferKey(settings.value("echoBufferKey", QString::fromStdString(configLocked->getEchoBufferKey())).toString().toUtf8().toStdString());
+		std::unordered_map<Configuration::ShortcutAction, std::string> shortcuts;
+		for (const auto& pair : SHORTCUTS_NAME_MAP_) {
+			std::string def = configLocked->getShortcuts().at(pair.first);
+			shortcuts[pair.first] = settings.value("shortcut_" + pair.second, utf8ToQString(def)).toString().toUtf8().toStdString();
+		}
+		configLocked->setShortcuts(shortcuts);
 		configLocked->setNoteEntryLayout(static_cast<Configuration::KeyboardLayout>(
 											 settings.value("noteEntryLayout",
 															static_cast<int>(configLocked->getNoteEntryLayout())).toInt()));
 		std::unordered_map<std::string, JamKey> customLayoutNewKeys;
-		const std::unordered_map<std::string, JamKey> nameToKeyMapping = {
-			{"lowC",       JamKey::LowC},
-			{"lowCS",      JamKey::LowCS},
-			{"lowD",       JamKey::LowD},
-			{"lowDS",      JamKey::LowDS},
-			{"lowE",       JamKey::LowE},
-			{"lowF",       JamKey::LowF},
-			{"lowFS",      JamKey::LowFS},
-			{"lowG",       JamKey::LowG},
-			{"lowGS",      JamKey::LowGS},
-			{"lowA",       JamKey::LowA},
-			{"lowAS",      JamKey::LowAS},
-			{"lowB",       JamKey::LowB},
-			{"lowHighC",   JamKey::LowC2},
-			{"lowHighCS",  JamKey::LowCS2},
-			{"lowHighD",   JamKey::LowD2},
-
-			{"highC",      JamKey::HighC},
-			{"highCS",     JamKey::HighCS},
-			{"highD",      JamKey::HighD},
-			{"highDS",     JamKey::HighDS},
-			{"highE",      JamKey::HighE},
-			{"highF",      JamKey::HighF},
-			{"highFS",     JamKey::HighFS},
-			{"highG",      JamKey::HighG},
-			{"highGS",     JamKey::HighGS},
-			{"highA",      JamKey::HighA},
-			{"highAS",     JamKey::HighAS},
-			{"highB",      JamKey::HighB},
-			{"highHighC",  JamKey::HighC2},
-			{"highHighCS", JamKey::HighCS2},
-			{"highHighD",  JamKey::HighD2}
-		};
-		std::unordered_map<std::string, JamKey>::const_iterator nameToKeyMappingIterator = nameToKeyMapping.begin();
-		while (nameToKeyMappingIterator != nameToKeyMapping.end()) {
-			JamKey currentlyWantedJamKey = nameToKeyMappingIterator->second;
+		for (const auto& pair : JAM_KEY_NAME_MAP_) {
+			JamKey currentlyWantedJamKey = pair.first;
 			customLayoutNewKeys[
-						settings.value(QString::fromStdString("customLayout_" + nameToKeyMappingIterator->first),
-									   QString::fromStdString((*std::find_if (configLocked->mappingLayouts.at(Configuration::QWERTY).begin(), configLocked->mappingLayouts.at(Configuration::QWERTY).end(),
-																		[currentlyWantedJamKey](const std::pair<std::string, JamKey>& t) -> bool {
-																		return (t.second) == currentlyWantedJamKey;})
-														 ).first)).toString().toUtf8().toStdString()]
+					settings.value("customLayout_" + pair.second,
+								   QString::fromStdString((*std::find_if (configLocked->mappingLayouts.at(Configuration::QWERTY).begin(),
+																		  configLocked->mappingLayouts.at(Configuration::QWERTY).end(),
+																		  [currentlyWantedJamKey](const std::pair<std::string, JamKey>& t) -> bool {
+				return (t.second) == currentlyWantedJamKey;})
+														   ).first)).toString().toUtf8().toStdString()]
 					= currentlyWantedJamKey;
-			nameToKeyMappingIterator++;
 		}
 		configLocked->setCustomLayoutKeys(customLayoutNewKeys);
 		settings.endGroup();
@@ -294,8 +373,8 @@ bool ConfigurationHandler::loadConfiguration(std::weak_ptr<Configuration> config
 		settings.beginGroup("Sound");
 		configLocked->setSoundAPI(settings.value("soundAPI", QString::fromStdString(configLocked->getSoundAPI())).toString().toUtf8().toStdString());
 		configLocked->setSoundDevice(settings.value("soundDevice", QString::fromStdString(configLocked->getSoundDevice())).toString().toUtf8().toStdString());
-                configLocked->setRealChipInterface(static_cast<RealChipInterface>(
-                                                       settings.value("realChipInterface", static_cast<int>(configLocked->getRealChipInterface())).toInt()));
+		configLocked->setRealChipInterface(static_cast<RealChipInterface>(
+											   settings.value("realChipInterface", static_cast<int>(configLocked->getRealChipInterface())).toInt()));
 		configLocked->setEmulator(settings.value("emulator", configLocked->getEmulator()).toInt());
 		QVariant sampleRateWorkaround;
 		sampleRateWorkaround.setValue(configLocked->getSampleRate());
