@@ -10,9 +10,6 @@ FMEnvelopeSetEditDialog::FMEnvelopeSetEditDialog(std::vector<FMEnvelopeTextType>
 
 	setWindowFlags(windowFlags() ^ Qt::WindowContextHelpButtonHint);
 
-	ui->treeWidget->setColumnCount(2);
-	ui->treeWidget->setHeaderLabels({ tr("Number"), tr("Type") });
-
 	for (size_t i = 0; i < set.size(); ++i) {
 		insertRow(static_cast<int>(i), set.at(i));
 	}
