@@ -14,7 +14,9 @@
 #include <QMessageBox>
 #include "gui/event_guard.hpp"
 #include "gui/jam_layout.hpp"
+#include "gui/instrument_editor/instrument_editor_util.hpp"
 #include "misc.hpp"
+#include "gui/gui_util.hpp"
 
 InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 	QWidget(parent),
@@ -572,6 +574,11 @@ InstrumentEditorFMForm::~InstrumentEditorFMForm()
 	delete ui;
 }
 
+void InstrumentEditorFMForm::setInstrumentNumber(int num)
+{
+	instNum_ = num;
+}
+
 int InstrumentEditorFMForm::getInstrumentNumber() const
 {
 	return instNum_;
@@ -589,7 +596,7 @@ void InstrumentEditorFMForm::setConfiguration(std::weak_ptr<Configuration> confi
 
 	std::vector<QString> names;
 	for (auto texts : config.lock()->getFMEnvelopeTexts()) {
-		names.push_back(QString::fromUtf8(texts.name.c_str(), static_cast<int>(texts.name.length())));
+		names.push_back(utf8ToQString(texts.name));
 	}
 	ui->op1Table->setEnvelopeSetNames(names);
 	ui->op2Table->setEnvelopeSetNames(names);
@@ -621,77 +628,11 @@ void InstrumentEditorFMForm::setColorPalette(std::shared_ptr<ColorPalette> palet
 	ui->ptEditor->setColorPalette(palette);
 }
 
-SequenceType InstrumentEditorFMForm::convertSequenceTypeForData(VisualizedInstrumentMacroEditor::SequenceType type)
-{
-	switch (type) {
-	case VisualizedInstrumentMacroEditor::SequenceType::NoType:
-		return SequenceType::NO_SEQUENCE_TYPE;
-	case VisualizedInstrumentMacroEditor::SequenceType::FixedSequence:
-		return SequenceType::FIXED_SEQUENCE;
-	case VisualizedInstrumentMacroEditor::SequenceType::AbsoluteSequence:
-		return SequenceType::ABSOLUTE_SEQUENCE;
-	case VisualizedInstrumentMacroEditor::SequenceType::RelativeSequence:
-		return SequenceType::RELATIVE_SEQUENCE;
-	default:
-		throw std::invalid_argument("Unexpected SequenceType.");
-	}
-}
-
-VisualizedInstrumentMacroEditor::SequenceType InstrumentEditorFMForm::convertSequenceTypeForUI(SequenceType type)
-{
-	switch (type) {
-	case SequenceType::NO_SEQUENCE_TYPE:
-		return VisualizedInstrumentMacroEditor::SequenceType::NoType;
-	case SequenceType::FIXED_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::FixedSequence;
-	case SequenceType::ABSOLUTE_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::AbsoluteSequence;
-	case SequenceType::RELATIVE_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::RelativeSequence;
-	default:
-		throw std::invalid_argument("Unexpected SequenceType.");
-	}
-}
-
-ReleaseType InstrumentEditorFMForm::convertReleaseTypeForData(VisualizedInstrumentMacroEditor::ReleaseType type)
-{
-	switch (type) {
-	case VisualizedInstrumentMacroEditor::ReleaseType::NO_RELEASE:
-		return ReleaseType::NoRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::FIXED_RELEASE:
-		return ReleaseType::FixedRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::ABSOLUTE_RELEASE:
-		return ReleaseType::AbsoluteRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::RELATIVE_RELEASE:
-		return ReleaseType::RelativeRelease;
-	default:
-		throw std::invalid_argument("Unexpected ReleaseType.");
-	}
-}
-
-VisualizedInstrumentMacroEditor::ReleaseType InstrumentEditorFMForm::convertReleaseTypeForUI(ReleaseType type)
-{
-	switch (type) {
-	case ReleaseType::NoRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::NO_RELEASE;
-	case ReleaseType::FixedRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::FIXED_RELEASE;
-	case ReleaseType::AbsoluteRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::ABSOLUTE_RELEASE;
-	case ReleaseType::RelativeRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::RELATIVE_RELEASE;
-	default:
-		throw std::invalid_argument("Unexpected ReleaseType.");
-	}
-}
-
 void InstrumentEditorFMForm::updateInstrumentParameters()
 {
 	Ui::EventGuard eg(isIgnoreEvent_);
 
-	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
-	auto instFM = dynamic_cast<InstrumentFM*>(inst.get());
-	auto name = QString::fromUtf8(instFM->getName().c_str(), static_cast<int>(instFM->getName().length()));
+	auto name = utf8ToQString(bt_.lock()->getInstrument(instNum_)->getName());
 	setWindowTitle(QString("%1: %2").arg(instNum_, 2, 16, QChar('0')).toUpper().arg(name));
 
 	setInstrumentEnvelopeParameters();
@@ -707,32 +648,13 @@ void InstrumentEditorFMForm::updateInstrumentParameters()
 // MUST DIRECT CONNECTION
 void InstrumentEditorFMForm::keyPressEvent(QKeyEvent *event)
 {
-	// For jam key on
-
-	// Check keys
-	QString seq = QKeySequence(static_cast<int>(event->modifiers()) | event->key()).toString();
-	if (seq == QKeySequence(
-				QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
-								  static_cast<int>(config_.lock()->getOctaveUpKey().length()))).toString()) {
-		emit octaveChanged(true);
-		return;
-	}
-	else if (seq == QKeySequence(
-				 QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
-								   static_cast<int>(config_.lock()->getOctaveDownKey().length()))).toString()) {
-		emit octaveChanged(false);
-		return;
-	}
-
 	// General keys
 	switch (event->key()) {
-	//case Qt::Key_Return:	emit playStatusChanged(0);	break;
-	case Qt::Key_F5:		emit playStatusChanged(1);	break;
-	case Qt::Key_F6:		emit playStatusChanged(2);	break;
-	case Qt::Key_F7:		emit playStatusChanged(3);	break;
-	case Qt::Key_F8:		emit playStatusChanged(-1);	break;
-	case Qt::Key_Escape:	close();					break;
+	case Qt::Key_Escape:
+		close();
+		break;
 	default:
+		// For jam key on
 		if (!event->isAutoRepeat()) {
 			// Musical keyboard
 			Qt::Key qtKey = static_cast<Qt::Key>(event->key());
@@ -906,7 +828,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(int envTypeNum, QSt
 		auto name = config_.lock()->getFMEnvelopeTexts().at(static_cast<size_t>(envTypeNum)).name;
 		QMessageBox::critical(this, tr("Error"),
 							  tr("Did not match the clipboard text format with %1.")
-							  .arg(QString::fromUtf8(name.c_str(), static_cast<int>(name.length()))));
+							  .arg(utf8ToQString(name)));
 		return;
 	}
 
@@ -973,7 +895,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(int envTypeNum, QSt
 		auto name = config_.lock()->getFMEnvelopeTexts().at(static_cast<size_t>(envTypeNum)).name;
 		QMessageBox::critical(this, tr("Error"),
 							  tr("Did not match the clipboard text format with %1.")
-							  .arg(QString::fromUtf8(name.c_str(), static_cast<int>(name.length()))));
+							  .arg(utf8ToQString(name)));
 		return;
 	}
 }
@@ -1295,8 +1217,7 @@ void InstrumentEditorFMForm::on_envGroupBox_customContextMenuRequested(const QPo
 	QMenu* pasteFrom = menu.addMenu(tr("Paste envelope From"));
 	std::vector<FMEnvelopeText> textsSet = config_.lock()->getFMEnvelopeTexts();
 	for (size_t i = 0; i < textsSet.size(); ++i) {
-		QAction* act = pasteFrom->addAction(
-						   QString::fromUtf8(textsSet[i].name.c_str(), static_cast<int>(textsSet[i].name.length())));
+		QAction* act = pasteFrom->addAction(utf8ToQString(textsSet[i].name));
 		act->setData(static_cast<int>(i));
 	}
 	QObject::connect(pasteFrom, &QMenu::triggered,
