@@ -24,6 +24,7 @@ class InstrumentEditorFMForm : public QWidget
 public:
 	InstrumentEditorFMForm(int num, QWidget *parent = nullptr);
 	~InstrumentEditorFMForm() override;
+	void setInstrumentNumber(int num);
 	int getInstrumentNumber() const;
 	void setCore(std::weak_ptr<BambooTracker> core);
 	void setConfiguration(std::weak_ptr<Configuration> config);
@@ -33,14 +34,7 @@ public:
 signals:
 	void jamKeyOnEvent(JamKey key);
 	void jamKeyOffEvent(JamKey key);
-	void octaveChanged(bool upFlag);
 	void modified();
-	/// 0: play song
-	/// 1: play from start
-	/// 2: play pattern
-	/// 3: play from cursor
-	/// -1: stop
-	void playStatusChanged(int stat);
 
 protected:
 	void keyPressEvent(QKeyEvent* event) override;
@@ -57,12 +51,6 @@ private:
 	std::shared_ptr<ColorPalette> palette_;
 	std::weak_ptr<Configuration> config_;
 
-	SequenceType convertSequenceTypeForData(VisualizedInstrumentMacroEditor::SequenceType type);
-	VisualizedInstrumentMacroEditor::SequenceType convertSequenceTypeForUI(SequenceType type);
-
-	ReleaseType convertReleaseTypeForData(VisualizedInstrumentMacroEditor::ReleaseType type);
-	VisualizedInstrumentMacroEditor::ReleaseType convertReleaseTypeForUI(ReleaseType type);
-
 	void updateInstrumentParameters();
 
 	//========== Envelope ==========//
