@@ -7,7 +7,9 @@
 #include "pitch_converter.hpp"
 #include "command_sequence.hpp"
 #include "gui/jam_layout.hpp"
+#include "gui/instrument_editor/instrument_editor_util.hpp"
 #include "misc.hpp"
+#include "gui/gui_util.hpp"
 
 InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	QWidget(parent),
@@ -16,7 +18,7 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 {
 	ui->setupUi(this);
 
-	//========== Wave form ==========//
+	//========== Waveform ==========//
 	ui->waveEditor->setMaximumDisplayedRowCount(7);
 	ui->waveEditor->setDefaultRow(0);
 	ui->waveEditor->AddRow(tr("Sq"), false);
@@ -31,37 +33,37 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::sequenceCommandAdded,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (isModulatedWaveFormSSG(row)) setWaveFormSequenceColumn(col);	// Set square-mask frequency
-			bt_.lock()->addWaveFormSSGSequenceCommand(
+			if (isModulatedWaveformSSG(row)) setWaveformSequenceColumn(col);	// Set square-mask frequency
+			bt_.lock()->addWaveformSSGSequenceCommand(
 						ui->waveNumSpinBox->value(), row, ui->waveEditor->getSequenceDataAt(col));
-			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
+			emit waveformParameterChanged(ui->waveNumSpinBox->value(), instNum_);
 			emit modified();
 		}
 	});
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::sequenceCommandRemoved,
 					 this, [&]() {
 		if (!isIgnoreEvent_) {
-			bt_.lock()->removeWaveFormSSGSequenceCommand(ui->waveNumSpinBox->value());
-			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
+			bt_.lock()->removeWaveformSSGSequenceCommand(ui->waveNumSpinBox->value());
+			emit waveformParameterChanged(ui->waveNumSpinBox->value(), instNum_);
 			emit modified();
 		}
 	});
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::sequenceCommandChanged,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (isModulatedWaveFormSSG(row)) setWaveFormSequenceColumn(col);	// Set square-mask frequency
-			bt_.lock()->setWaveFormSSGSequenceCommand(
+			if (isModulatedWaveformSSG(row)) setWaveformSequenceColumn(col);	// Set square-mask frequency
+			bt_.lock()->setWaveformSSGSequenceCommand(
 						ui->waveNumSpinBox->value(), col, row, ui->waveEditor->getSequenceDataAt(col));
-			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
+			emit waveformParameterChanged(ui->waveNumSpinBox->value(), instNum_);
 			emit modified();
 		}
 	});
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::loopChanged,
 					 this, [&](std::vector<int> begins, std::vector<int> ends, std::vector<int> times) {
 		if (!isIgnoreEvent_) {
-			bt_.lock()->setWaveFormSSGLoops(
+			bt_.lock()->setWaveformSSGLoops(
 						ui->waveNumSpinBox->value(), std::move(begins), std::move(ends), std::move(times));
-			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
+			emit waveformParameterChanged(ui->waveNumSpinBox->value(), instNum_);
 			emit modified();
 		}
 	});
@@ -69,8 +71,8 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 					 this, [&](VisualizedInstrumentMacroEditor::ReleaseType type, int point) {
 		if (!isIgnoreEvent_) {
 			ReleaseType t = convertReleaseTypeForData(type);
-			bt_.lock()->setWaveFormSSGRelease(ui->waveNumSpinBox->value(), t, point);
-			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
+			bt_.lock()->setWaveformSSGRelease(ui->waveNumSpinBox->value(), t, point);
+			emit waveformParameterChanged(ui->waveNumSpinBox->value(), instNum_);
 			emit modified();
 		}
 	});
@@ -305,6 +307,11 @@ InstrumentEditorSSGForm::~InstrumentEditorSSGForm()
 	delete ui;
 }
 
+void InstrumentEditorSSGForm::setInstrumentNumber(int num)
+{
+	instNum_ = num;
+}
+
 int InstrumentEditorSSGForm::getInstrumentNumber() const
 {
 	return instNum_;
@@ -330,80 +337,14 @@ void InstrumentEditorSSGForm::setColorPalette(std::shared_ptr<ColorPalette> pale
 	ui->ptEditor->setColorPalette(palette);
 }
 
-SequenceType InstrumentEditorSSGForm::convertSequenceTypeForData(VisualizedInstrumentMacroEditor::SequenceType type)
-{
-	switch (type) {
-	case VisualizedInstrumentMacroEditor::SequenceType::NoType:
-		return SequenceType::NO_SEQUENCE_TYPE;
-	case VisualizedInstrumentMacroEditor::SequenceType::FixedSequence:
-		return SequenceType::FIXED_SEQUENCE;
-	case VisualizedInstrumentMacroEditor::SequenceType::AbsoluteSequence:
-		return SequenceType::ABSOLUTE_SEQUENCE;
-	case VisualizedInstrumentMacroEditor::SequenceType::RelativeSequence:
-		return SequenceType::RELATIVE_SEQUENCE;
-	default:
-		throw std::invalid_argument("Unexpected SequenceType.");
-	}
-}
-
-VisualizedInstrumentMacroEditor::SequenceType InstrumentEditorSSGForm::convertSequenceTypeForUI(SequenceType type)
-{
-	switch (type) {
-	case SequenceType::NO_SEQUENCE_TYPE:
-		return VisualizedInstrumentMacroEditor::SequenceType::NoType;
-	case SequenceType::FIXED_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::FixedSequence;
-	case SequenceType::ABSOLUTE_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::AbsoluteSequence;
-	case SequenceType::RELATIVE_SEQUENCE:
-		return VisualizedInstrumentMacroEditor::SequenceType::RelativeSequence;
-	default:
-		throw std::invalid_argument("Unexpected SequenceType.");
-	}
-}
-
-ReleaseType InstrumentEditorSSGForm::convertReleaseTypeForData(VisualizedInstrumentMacroEditor::ReleaseType type)
-{
-	switch (type) {
-	case VisualizedInstrumentMacroEditor::ReleaseType::NO_RELEASE:
-		return ReleaseType::NoRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::FIXED_RELEASE:
-		return ReleaseType::FixedRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::ABSOLUTE_RELEASE:
-		return ReleaseType::AbsoluteRelease;
-	case VisualizedInstrumentMacroEditor::ReleaseType::RELATIVE_RELEASE:
-		return ReleaseType::RelativeRelease;
-	default:
-		throw std::invalid_argument("Unexpected ReleaseType.");
-	}
-}
-
-VisualizedInstrumentMacroEditor::ReleaseType InstrumentEditorSSGForm::convertReleaseTypeForUI(ReleaseType type)
-{
-	switch (type) {
-	case ReleaseType::NoRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::NO_RELEASE;
-	case ReleaseType::FixedRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::FIXED_RELEASE;
-	case ReleaseType::AbsoluteRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::ABSOLUTE_RELEASE;
-	case ReleaseType::RelativeRelease:
-		return VisualizedInstrumentMacroEditor::ReleaseType::RELATIVE_RELEASE;
-	default:
-		throw std::invalid_argument("Unexpected ReleaseType.");
-	}
-}
-
 void InstrumentEditorSSGForm::updateInstrumentParameters()
 {
 	Ui::EventGuard eg(isIgnoreEvent_);
 
-	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
-	auto instSSG = dynamic_cast<InstrumentSSG*>(inst.get());
-	auto name = QString::fromUtf8(instSSG->getName().c_str(), static_cast<int>(instSSG->getName().length()));
+	auto name = utf8ToQString(bt_.lock()->getInstrument(instNum_)->getName());
 	setWindowTitle(QString("%1: %2").arg(instNum_, 2, 16, QChar('0')).toUpper().arg(name));
 
-	setInstrumentWaveFormParameters();
+	setInstrumentWaveformParameters();
 	setInstrumentToneNoiseParameters();
 	setInstrumentEnvelopeParameters();
 	setInstrumentArpeggioParameters();
@@ -414,32 +355,13 @@ void InstrumentEditorSSGForm::updateInstrumentParameters()
 // MUST DIRECT CONNECTION
 void InstrumentEditorSSGForm::keyPressEvent(QKeyEvent *event)
 {
-	// For jam key on
-
-	// Check keys
-	QString seq = QKeySequence(static_cast<int>(event->modifiers()) | event->key()).toString();
-	if (seq == QKeySequence(
-				QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
-								  static_cast<int>(config_.lock()->getOctaveUpKey().length()))).toString()) {
-		emit octaveChanged(true);
-		return;
-	}
-	else if (seq == QKeySequence(
-				 QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
-								   static_cast<int>(config_.lock()->getOctaveDownKey().length()))).toString()) {
-		emit octaveChanged(false);
-		return;
-	}
-
 	// General keys
 	switch (event->key()) {
-	//case Qt::Key_Return:	emit playStatusChanged(0);	break;
-	case Qt::Key_F5:		emit playStatusChanged(1);	break;
-	case Qt::Key_F6:		emit playStatusChanged(2);	break;
-	case Qt::Key_F7:		emit playStatusChanged(3);	break;
-	case Qt::Key_F8:		emit playStatusChanged(-1);	break;
-	case Qt::Key_Escape:	close();					break;
+	case Qt::Key_Escape:
+		close();
+		break;
 	default:
+		// For jam key on
 		if (!event->isAutoRepeat()) {
 			// Musical keyboard
 			Qt::Key qtKey = static_cast<Qt::Key>(event->key());
@@ -465,19 +387,19 @@ void InstrumentEditorSSGForm::keyReleaseEvent(QKeyEvent *event)
 	}
 }
 
-//--- Wave form
-void InstrumentEditorSSGForm::setInstrumentWaveFormParameters()
+//--- Waveform
+void InstrumentEditorSSGForm::setInstrumentWaveformParameters()
 {
 	Ui::EventGuard ev(isIgnoreEvent_);
 
 	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
 	auto instSSG = dynamic_cast<InstrumentSSG*>(inst.get());
 
-	ui->waveNumSpinBox->setValue(instSSG->getWaveFormNumber());
+	ui->waveNumSpinBox->setValue(instSSG->getWaveformNumber());
 	ui->waveEditor->clearData();
-	for (auto& com : instSSG->getWaveFormSequence()) {
+	for (auto& com : instSSG->getWaveformSequence()) {
 		QString str("");
-		if (isModulatedWaveFormSSG(com.type)) {
+		if (isModulatedWaveformSSG(com.type)) {
 			if (CommandSequenceUnit::checkDataType(com.data) == CommandSequenceUnit::RATIO) {
 				auto ratio = CommandSequenceUnit::data2ratio(com.data);
 				str = QString("%1/%2").arg(ratio.first).arg(ratio.second);
@@ -488,21 +410,21 @@ void InstrumentEditorSSGForm::setInstrumentWaveFormParameters()
 		}
 		ui->waveEditor->addSequenceCommand(com.type, str, com.data);
 	}
-	for (auto& l : instSSG->getWaveFormLoops()) {
+	for (auto& l : instSSG->getWaveformLoops()) {
 		ui->waveEditor->addLoop(l.begin, l.end, l.times);
 	}
-	ui->waveEditor->setRelease(convertReleaseTypeForUI(instSSG->getWaveFormRelease().type),
-							   instSSG->getWaveFormRelease().begin);
-	if (instSSG->getWaveFormEnabled()) {
+	ui->waveEditor->setRelease(convertReleaseTypeForUI(instSSG->getWaveformRelease().type),
+							   instSSG->getWaveformRelease().begin);
+	if (instSSG->getWaveformEnabled()) {
 		ui->waveEditGroupBox->setChecked(true);
-		onWaveFormNumberChanged();
+		onWaveformNumberChanged();
 	}
 	else {
 		ui->waveEditGroupBox->setChecked(false);
 	}
 }
 
-void InstrumentEditorSSGForm::setWaveFormSequenceColumn(int col)
+void InstrumentEditorSSGForm::setWaveformSequenceColumn(int col)
 {
 	auto button = ui->squareMaskButtonGroup->checkedButton();
 	if (button == ui->squareMaskRawRadioButton) {
@@ -520,10 +442,10 @@ void InstrumentEditorSSGForm::setWaveFormSequenceColumn(int col)
 }
 
 /********** Slots **********/
-void InstrumentEditorSSGForm::onWaveFormNumberChanged()
+void InstrumentEditorSSGForm::onWaveformNumberChanged()
 {
 	// Change users view
-	std::vector<int> users = bt_.lock()->getWaveFormSSGUsers(ui->waveNumSpinBox->value());
+	std::vector<int> users = bt_.lock()->getWaveformSSGUsers(ui->waveNumSpinBox->value());
 	QStringList l;
 	std::transform(users.begin(), users.end(), std::back_inserter(l), [](int n) {
 		return QString("%1").arg(n, 2, 16, QChar('0')).toUpper();
@@ -531,36 +453,36 @@ void InstrumentEditorSSGForm::onWaveFormNumberChanged()
 	ui->waveUsersLineEdit->setText(l.join(","));
 }
 
-void InstrumentEditorSSGForm::onWaveFormParameterChanged(int wfNum)
+void InstrumentEditorSSGForm::onWaveformParameterChanged(int wfNum)
 {
 	if (ui->waveNumSpinBox->value() == wfNum) {
 		Ui::EventGuard eg(isIgnoreEvent_);
-		setInstrumentWaveFormParameters();
+		setInstrumentWaveformParameters();
 	}
 }
 
 void InstrumentEditorSSGForm::on_waveEditGroupBox_toggled(bool arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentSSGWaveFormEnabled(instNum_, arg1);
-		setInstrumentWaveFormParameters();
-		emit waveFormNumberChanged();
+		bt_.lock()->setInstrumentSSGWaveformEnabled(instNum_, arg1);
+		setInstrumentWaveformParameters();
+		emit waveformNumberChanged();
 		emit modified();
 	}
 
-	onWaveFormNumberChanged();
+	onWaveformNumberChanged();
 }
 
 void InstrumentEditorSSGForm::on_waveNumSpinBox_valueChanged(int arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentSSGWaveForm(instNum_, arg1);
-		setInstrumentWaveFormParameters();
-		emit waveFormNumberChanged();
+		bt_.lock()->setInstrumentSSGWaveform(instNum_, arg1);
+		setInstrumentWaveformParameters();
+		emit waveformNumberChanged();
 		emit modified();
 	}
 
-	onWaveFormNumberChanged();
+	onWaveformNumberChanged();
 }
 
 void InstrumentEditorSSGForm::on_squareMaskRawSpinBox_valueChanged(int arg1)
