@@ -22,6 +22,7 @@ class InstrumentEditorSSGForm : public QWidget
 public:
 	InstrumentEditorSSGForm(int num, QWidget *parent = nullptr);
 	~InstrumentEditorSSGForm() override;
+	void setInstrumentNumber(int num);
 	int getInstrumentNumber() const;
 	void setCore(std::weak_ptr<BambooTracker> core);
 	void setConfiguration(std::weak_ptr<Configuration> config);
@@ -30,14 +31,7 @@ public:
 signals:
 	void jamKeyOnEvent(JamKey key);
 	void jamKeyOffEvent(JamKey key);
-	void octaveChanged(bool upFlag);
 	void modified();
-	/// 0: play song
-	/// 1: play from start
-	/// 2: play pattern
-	/// 3: play from cursor
-	/// -1: stop
-	void playStatusChanged(int stat);
 
 protected:
 	void keyPressEvent(QKeyEvent* event) override;
@@ -51,26 +45,20 @@ private:
 	std::weak_ptr<BambooTracker> bt_;
 	std::weak_ptr<Configuration> config_;
 
-	SequenceType convertSequenceTypeForData(VisualizedInstrumentMacroEditor::SequenceType type);
-	VisualizedInstrumentMacroEditor::SequenceType convertSequenceTypeForUI(SequenceType type);
-
-	ReleaseType convertReleaseTypeForData(VisualizedInstrumentMacroEditor::ReleaseType type);
-	VisualizedInstrumentMacroEditor::ReleaseType convertReleaseTypeForUI(ReleaseType type);
-
 	void updateInstrumentParameters();
 
-	//========== Wave form ==========//
+	//========== Waveform ==========//
 signals:
-	void waveFormNumberChanged();
-	void waveFormParameterChanged(int wfNum, int fromInstNum);
+	void waveformNumberChanged();
+	void waveformParameterChanged(int wfNum, int fromInstNum);
 
 public slots:
-	void onWaveFormNumberChanged();
-	void onWaveFormParameterChanged(int wfNum);
+	void onWaveformNumberChanged();
+	void onWaveformParameterChanged(int wfNum);
 
 private:
-	void setInstrumentWaveFormParameters();
-	void setWaveFormSequenceColumn(int col);
+	void setInstrumentWaveformParameters();
+	void setWaveformSequenceColumn(int col);
 
 private slots:
 	void on_waveEditGroupBox_toggled(bool arg1);
