@@ -1,8 +1,10 @@
 #ifndef MAINWINDOW_HPP
 #define MAINWINDOW_HPP
 
+#include <atomic>
 #include <memory>
 #include <cstdint>
+#include <unordered_map>
 #include <QMainWindow>
 #include <QKeyEvent>
 #include <QListWidgetItem>
@@ -19,6 +21,9 @@
 #include <QSpinBox>
 #include <QTimer>
 #include <QLibrary>
+#include <QShortcut>
+#include <QAction>
+#include <QActionGroup>
 #include "configuration.hpp"
 #include "bamboo_tracker.hpp"
 #include "audio_stream.hpp"
@@ -28,7 +33,11 @@
 #include "gui/file_history.hpp"
 #include "gui/effect_list_dialog.hpp"
 #include "gui/keyboard_shortcut_list_dialog.hpp"
+#include "gui/bookmark_manager_form.hpp"
+#include "gui/instrument_selection_dialog.hpp"
+#include "gui/comment_edit_dialog.hpp"
 #include "file_io_error.hpp"
+#include "enum_hash.hpp"
 
 class AbstractBank;
 
@@ -44,8 +53,10 @@ public:
 	MainWindow(std::weak_ptr<Configuration> config, QString filePath, QWidget *parent = nullptr);
 	~MainWindow() override;
 
+	void assignADPCMSamples();
+
 protected:
-	bool eventFilter(QObject *watched, QEvent *event) override;
+	bool eventFilter(QObject* watched, QEvent* event) override;
 	void showEvent(QShowEvent* event) override;
 	void keyPressEvent(QKeyEvent* event) override;
 	void keyReleaseEvent(QKeyEvent* event) override;
@@ -78,8 +89,9 @@ private:
 	// Instrument list
 	std::shared_ptr<InstrumentFormManager> instForms_;
 	void addInstrument();
+	void addDrumkit();
 	void removeInstrument(int row);
-	void editInstrument();
+	void openInstrumentEditor();
 	int findRowFromInstrumentList(int instNum);
 	void renameInstrument();
 	void cloneInstrument();
@@ -90,6 +102,7 @@ private:
 	void importInstrumentsFromBank();
 	void funcImportInstrumentsFromBank(QString file);
 	void exportInstrumentsToBank();
+	void swapInstruments(int row1, int row2);
 
 	// Undo-Redo
 	void undo();
@@ -106,6 +119,8 @@ private:
 	void startPlayFromStart();
 	void startPlayPattern();
 	void startPlayFromCurrentStep();
+	void startPlayFromMarker();
+	void playStep();
 	void stopPlaySong();
 	bool hasLockedWigets_;
 	void lockWidgets(bool isLock);
@@ -134,7 +149,7 @@ private:
 	void showFileIOErrorDialog(const FileIOError& e, const QString sub = "");
 
 	bool isEditedPattern_, isEditedOrder_, isEditedInstList_;
-	bool isSelectedPO_;
+	bool isSelectedPattern_, isSelectedOrder_;
 
 	bool hasShownOnce_;
 
@@ -142,23 +157,41 @@ private:
 
 	bool firstViewUpdateRequest_;
 
-	// Sub tool bar
-	QSpinBox* octave_;
-	QSpinBox *highlight1_, *highlight2_;
+	// Menus
+	std::unique_ptr<QActionGroup> pasteModeGroup_;
 
-	// Status bar
-	QLabel* statusDetail_;
-	QLabel* statusStyle_;
-	QLabel* statusInst_;
-	QLabel* statusOctave_;
-	QLabel* statusIntr_;
-	QLabel* statusMixer_;
-	QLabel* statusBpm_;
-	QLabel* statusPlayPos_;
+	// Toolbar
+	QSpinBox *octave_, *highlight1_, *highlight2_, *volume_;
+	using ToolbarPos = Configuration::ToolbarConfiguration::ToolbarPosition;
+	const std::unordered_map<ToolbarPos, Qt::ToolBarArea> TB_POS_ = {
+		{ToolbarPos::TOP_POS, Qt::TopToolBarArea },
+		{ ToolbarPos::BOTTOM_POS, Qt::BottomToolBarArea },
+		{ ToolbarPos::LEFT_POS, Qt::LeftToolBarArea },
+		{ ToolbarPos::RIGHT_POS, Qt::RightToolBarArea }
+	};
+
+	// Status bars
+	QLabel *statusDetail_, *statusStyle_, *statusInst_, *statusOctave_;
+	QLabel *statusIntr_, *statusMixer_, *statusBpm_, *statusPlayPos_;
+
+	// Shortcuts
+	std::unique_ptr<QAction> octUpSc_, octDownSc_;
+	std::unique_ptr<QShortcut> focusPtnSc_, focusOdrSc_, focusInstSc_, instAddSc_;
+	std::unique_ptr<QAction> playAndStopSc_, playStepSc_, goPrevOdrSc_, goNextOdrSc_;
+	std::unique_ptr<QAction> prevInstSc_, nextInstSc_, incPtnSizeSc_, decPtnSizeSc_;
+	std::unique_ptr<QAction> incEditStepSc_, decEditStepSc_, prevSongSc_, nextSongSc_;
+	std::unique_ptr<QAction> jamVolUpSc_, jamVolDownSc_;
+	void setShortcuts();
 
 	// Dialogs
 	std::unique_ptr<EffectListDialog> effListDiag_;
 	std::unique_ptr<KeyboardShortcutListDialog> shortcutsDiag_;
+	std::unique_ptr<BookmarkManagerForm> bmManForm_;
+	std::unique_ptr<CommentEditDialog> commentDiag_;
+
+	// Bank import
+	std::atomic_bool bankJamMidiCtrl_;
+	std::unique_ptr<InstrumentSelectionDialog> importBankDiag_;
 
 	// Meta methods
 	int tickEventMethod_;
@@ -184,10 +217,9 @@ private:
 	}
 
 private slots:
-	void on_instrumentListWidget_customContextMenuRequested(const QPoint &pos);
-	void on_instrumentListWidget_itemDoubleClicked(QListWidgetItem *item);
-	void onInstrumentListWidgetItemAdded(const QModelIndex& parent, int start, int end);
-	void on_instrumentListWidget_itemSelectionChanged();
+	void on_instrumentList_customContextMenuRequested(const QPoint& pos);
+	void on_instrumentList_itemDoubleClicked(QListWidgetItem* item);
+	void on_instrumentList_itemSelectionChanged();
 	void on_grooveCheckBox_stateChanged(int arg1);
 	void on_actionExit_triggered();
 	void on_actionUndo_triggered();
@@ -198,8 +230,10 @@ private slots:
 	void on_actionDelete_triggered();
 	void updateMenuByPattern();
 	void updateMenuByOrder();
+	void onCurrentTrackChanged();
 	void updateMenuByInstrumentList();
-	void updateMenuByPatternAndOrderSelection(bool isSelected);
+	void updateMenuByPatternSelection(bool isSelected);
+	void updateMenuByOrderSelection(bool isSelected);
 	void on_actionAll_triggered();
 	void on_actionNone_triggered();
 	void on_actionDecrease_Note_triggered();
@@ -264,10 +298,29 @@ private slots:
 	void on_action_Effect_List_triggered();
 	void on_actionShortcuts_triggered();
 	void on_actionExport_To_Bank_File_triggered();
-	void on_actionE_xpand_Effect_Column_triggered();
-	void on_actionS_hrink_Effect_Column_triggered();
 	void on_actionRemove_Duplicate_Instruments_triggered();
 	void on_actionRename_Instrument_triggered();
+	void on_action_Bookmark_Manager_triggered();
+	void on_actionFine_Decrease_Values_triggered();
+	void on_actionFine_Increase_Values_triggered();
+	void on_actionCoarse_D_ecrease_Values_triggered();
+	void on_actionCoarse_I_ncrease_Values_triggered();
+	void on_action_Toggle_Bookmark_triggered();
+	void on_action_Next_Bookmark_triggered();
+	void on_action_Previous_Bookmark_triggered();
+	void on_action_Instrument_Mask_triggered();
+	void on_action_Volume_Mask_triggered();
+	void on_actionSet_Ro_w_Marker_triggered();
+	void on_actionPlay_From_Marker_triggered();
+	void on_action_Go_To_triggered();
+	void on_actionRemove_Unused_ADPCM_Samples_triggered();
+	void on_action_Status_Bar_triggered();
+	void on_action_Toolbar_triggered();
+	void on_actionNew_Drumki_t_triggered();
+	void on_action_Wave_View_triggered(bool checked);
+	void on_action_Transpose_Song_triggered();
+	void on_action_Swap_Tracks_triggered();
+	void on_action_Insert_triggered();
 };
 
 #endif // MAINWINDOW_HPP
