@@ -35,8 +35,6 @@ OrderListEditor::OrderListEditor(QWidget *parent) :
 					 this, [&] { emit orderEdited(); });
 	QObject::connect(ui->panel, &OrderListPanel::selected,
 					 this, [&](bool isSelected) { emit selected(isSelected); });
-	QObject::connect(ui->panel, &OrderListPanel::returnPressed,
-					 this, [&] { emit returnPressed(); });
 
 	auto focusSlot = [&] { ui->panel->setFocus(); };
 
@@ -85,6 +83,11 @@ void OrderListEditor::updatePositionByOrderUpdate(bool isFirstUpdate)
 	ui->panel->updatePositionByOrderUpdate(isFirstUpdate);
 }
 
+void OrderListEditor::updatePositionByPositionJump(bool trackChanged)
+{
+	ui->panel->updatePositionByOrderUpdate(false, true, trackChanged);
+}
+
 void OrderListEditor::copySelectedCells()
 {
 	ui->panel->copySelectedCells();
@@ -226,6 +229,11 @@ void OrderListEditor::onSongLoaded()
 	ui->verticalScrollBar->setMaximum(static_cast<int>(bt_->getOrderSize(song)) - 1);
 }
 
+void OrderListEditor::onShortcutUpdated()
+{
+	ui->panel->onShortcutUpdated();
+}
+
 void OrderListEditor::onPastePressed()
 {
 	ui->panel->onPastePressed();
@@ -266,6 +274,16 @@ void OrderListEditor::onStoppedPlaySong()
 	ui->panel->onStoppedPlaySong();
 }
 
+void OrderListEditor::onGoOrderRequested(bool toNext)
+{
+	ui->panel->onGoOrderRequested(toNext);
+}
+
+void OrderListEditor::onOrderDataGlobalChanged()
+{
+	ui->panel->redrawByPatternChanged();	// Redraw only text
+}
+
 void OrderListEditor::updateHorizontalSliderMaximum()
 {
 	if (!bt_ || !songLoaded_) return;
