@@ -19,6 +19,7 @@
 #include "gui/command/order/order_commands.hpp"
 #include "playback.hpp"
 #include "track.hpp"
+#include "gui/gui_util.hpp"
 
 OrderListPanel::OrderListPanel(QWidget *parent)
 	: QWidget(parent),
@@ -77,6 +78,21 @@ OrderListPanel::OrderListPanel(QWidget *parent)
 
 	setAttribute(Qt::WA_Hover);
 	setContextMenuPolicy(Qt::CustomContextMenu);
+
+	insSc1_ = std::make_unique<QShortcut>(this);
+	insSc1_->setContext(Qt::WidgetShortcut);
+	QObject::connect(insSc1_.get(), &QShortcut::activated, this, &OrderListPanel::insertOrderBelow);
+	insSc2_ = std::make_unique<QShortcut>(this);
+	insSc2_->setContext(Qt::WidgetShortcut);
+	QObject::connect(insSc2_.get(), &QShortcut::activated, this, &OrderListPanel::insertOrderBelow);
+	menuSc_ = std::make_unique<QShortcut>(this);
+	menuSc_->setContext(Qt::WidgetShortcut);
+	QObject::connect(menuSc_.get(), &QShortcut::activated, this, [&] {
+		showContextMenu(
+					curPos_,
+					QPoint(calculateColumnsWidthWithRowNum(leftTrackNum_, curPos_.track), curRowY_ - 8));
+	});
+	onShortcutUpdated();
 }
 
 void OrderListPanel::setCore(std::shared_ptr<BambooTracker> core)
@@ -417,7 +433,7 @@ void OrderListPanel::quickDrawRows(int maxWidth, int trackSize)
 	int halfRowsCnt = viewedRowCnt_ >> 1;
 	int shift = rowFontHeight_ * orderDownCount_;
 
-	/* Move up by 1 step */
+	/* Move up by */
 	QRect srcRect(0, 0, maxWidth, viewedRowsHeight_);
 	textPixmap_->scroll(0, -shift, srcRect);
 	backPixmap_->scroll(0, -shift, srcRect);
@@ -610,14 +626,17 @@ void OrderListPanel::drawHeaders(int maxWidth, int trackSize)
 			break;
 		case SoundSource::DRUM:
 			switch (attrib.channelInSource) {
-			case 0:	str = "BD";	break;
-			case 1:	str = "SD";	break;
+			case 0:	str = "BD";		break;
+			case 1:	str = "SD";		break;
 			case 2:	str = "TOP";	break;
-			case 3:	str = "HH";	break;
+			case 3:	str = "HH";		break;
 			case 4:	str = "TOM";	break;
 			case 5:	str = "RIM";	break;
 			}
 			break;
+		case SoundSource::ADPCM:
+			str = "AP";
+			break;
 		}
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
@@ -706,12 +725,8 @@ void OrderListPanel::moveCursorToRight(int n)
 	entryCnt_ = 0;
 
 	if (!isIgnoreToSlider_) {	// Send to slider
-		if (config_->getMoveCursorByHorizontalScroll()) {
-			emit hScrollBarChangeRequested(curPos_.track);
-		}
-		else {
-			emit hScrollBarChangeRequested(leftTrackNum_);
-		}
+		emit hScrollBarChangeRequested(config_->getMoveCursorByHorizontalScroll() ? curPos_.track
+																				  : leftTrackNum_);
 	}
 
 	if (!isIgnoreToPattern_) emit currentTrackChanged(curPos_.track);	// Send to pattern editor
@@ -784,24 +799,47 @@ void OrderListPanel::changeEditable()
 	repaint();
 }
 
-void OrderListPanel::updatePositionByOrderUpdate(bool isFirstUpdate)
+void OrderListPanel::updatePositionByOrderUpdate(bool isFirstUpdate, bool forceJump, bool trackChanged)
 {	
 	int prev = std::exchange(playingRow_, bt_->getPlayingOrderNumber());
-	if (!config_->getFollowMode() && prev != playingRow_) {	// Repaint only background
+	if (!forceJump && !config_->getFollowMode() && prev != playingRow_) {	// Repaint only background
 		backChanged_ = true;
 		repaint();
 		return;
 	}
 
+	if (trackChanged) {	// Update horizontal position
+		int prevTrack = std::exchange(curPos_.track, bt_->getCurrentTrackAttribute().number);
+		if (prevTrack < curPos_.track) {
+			while (calculateColumnsWidthWithRowNum(leftTrackNum_, curPos_.track) > geometry().width()) {
+				++leftTrackNum_;
+				headerChanged_ = true;
+			}
+		}
+		else {
+			if (curPos_.track < leftTrackNum_) {
+				leftTrackNum_ = curPos_.track;
+				headerChanged_ = true;
+			}
+		}
+
+		updateTracksWidthFromLeftToEnd();
+
+		emit hScrollBarChangeRequested(config_->getMoveCursorByHorizontalScroll() ? curPos_.track
+																				  : leftTrackNum_);
+	}
+
 	int tmp = std::exchange(curPos_.row, bt_->getCurrentOrderNumber());
 	int d = curPos_.row - tmp;
-	if (!d) return;
+	if (d) {
+		emit vScrollBarChangeRequested(curPos_.row, static_cast<int>(bt_->getOrderSize(curSongNum_)) - 1);
 
-	emit vScrollBarChangeRequested(curPos_.row, static_cast<int>(bt_->getOrderSize(curSongNum_)) - 1);
-
-	// Redraw entire area in first update and jumping order
-	orderDownCount_ = (isFirstUpdate || d < 0 || viewedRowCnt_ < d) ? 0 : d;
+		// Redraw entire area in first update and jumping order
+		orderDownCount_ = (isFirstUpdate || d < 0 || (viewedRowCnt_ >> 1) < d) ? 0 : d;
+	}
+	else if (!trackChanged) return;
 
+	entryCnt_ = 0;
 	textChanged_ = true;
 	backChanged_ = true;
 	repaint();
@@ -855,6 +893,7 @@ void OrderListPanel::redrawAll()
 	backChanged_ = true;
 	textChanged_ = true;
 	headerChanged_ = true;
+	orderDownCount_ = 0;	// Prevent quick draw
 	repaint();
 }
 
@@ -1032,8 +1071,10 @@ void OrderListPanel::showContextMenu(const OrderPosition& pos, const QPoint& poi
 	copy->setShortcutVisibleInContextMenu(true);
 	paste->setShortcutVisibleInContextMenu(true);
 #endif
-	duplicate->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_D));
-	clonep->setShortcut(QKeySequence(Qt::ALT + Qt::Key_D));
+	auto shortcuts = config_->getShortcuts();
+	duplicate->setShortcut(strToKeySeq(shortcuts.at(Configuration::DuplicateOrder)));
+	clonep->setShortcut(strToKeySeq(shortcuts.at(Configuration::ClonePatterns)));
+	cloneo->setShortcut(strToKeySeq(shortcuts.at(Configuration::CloneOrder)));
 	copy->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_C));
 	paste->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_V));
 
@@ -1144,6 +1185,13 @@ void OrderListPanel::onSongLoaded()
 	redrawAll();
 }
 
+void OrderListPanel::onShortcutUpdated()
+{
+	insSc1_->setKey(Qt::Key_Insert);
+	insSc2_->setKey(Qt::ALT + Qt::Key_B);
+	menuSc_->setKey(Qt::Key_Menu);
+}
+
 void OrderListPanel::onPastePressed()
 {
 	pasteCopiedCells(curPos_);
@@ -1259,6 +1307,11 @@ void OrderListPanel::onStoppedPlaySong()
 	repaint();
 }
 
+void OrderListPanel::onGoOrderRequested(bool toNext)
+{
+	moveCursorToDown(toNext ? 1 : -1);
+}
+
 /********** Events **********/
 bool OrderListPanel::event(QEvent *event)
 {
@@ -1278,9 +1331,6 @@ bool OrderListPanel::keyPressed(QKeyEvent *event)
 {	
 	/* General Keys */
 	switch (event->key()) {
-	case Qt::Key_Return:
-		emit returnPressed();
-		return true;
 	case Qt::Key_Shift:
 		shiftPressedPos_ = curPos_;
 		return true;
@@ -1355,14 +1405,6 @@ bool OrderListPanel::keyPressed(QKeyEvent *event)
 			else onSelectPressed(0);
 			return true;
 		}
-	case Qt::Key_Insert:
-		insertOrderBelow();
-		return true;
-	case Qt::Key_Menu:
-		showContextMenu(
-					curPos_,
-					QPoint(calculateColumnsWidthWithRowNum(leftTrackNum_, curPos_.track), curRowY_ - 8));
-		return true;
 	default:
 		if (event->modifiers().testFlag(Qt::NoModifier)) {
 			return enterOrder(event->key());
