@@ -16,6 +16,7 @@
 #include <QRect>
 #include <QColor>
 #include <QPoint>
+#include <QShortcut>
 #include <memory>
 #include <atomic>
 #include "bamboo_tracker.hpp"
@@ -35,7 +36,7 @@ public:
 	void setColorPallete(std::shared_ptr<ColorPalette> palette);
 
 	void changeEditable();
-	void updatePositionByOrderUpdate(bool isFirstUpdate);
+	void updatePositionByOrderUpdate(bool isFirstUpdate, bool forceJump = false, bool trackChanged = false);
 	int getScrollableCountByTrack() const;
 
 	void copySelectedCells();
@@ -66,6 +67,7 @@ public slots:
 
 	void onOrderEdited();
 	void onSongLoaded();
+	void onShortcutUpdated();
 
 	void onPastePressed();
 	/// 0: None
@@ -81,6 +83,7 @@ public slots:
 	void onCloneOrderPressed();
 	void onFollowModeChanged();
 	void onStoppedPlaySong();
+	void onGoOrderRequested(bool toNext);
 
 signals:
 	void hScrollBarChangeRequested(int num);
@@ -91,7 +94,6 @@ signals:
 	void orderEdited();
 
 	void selected(bool isSelected);
-	void returnPressed();
 
 protected:
 	virtual bool event(QEvent *event) override;
@@ -156,6 +158,8 @@ private:
 
 	int playingRow_;
 
+	std::unique_ptr<QShortcut> insSc1_, insSc2_, menuSc_;
+
 	void updateSizes();
 	void initDisplay();
 
