@@ -46,10 +46,10 @@ PatternEditor::PatternEditor(QWidget *parent) :
 	});
 	QObject::connect(ui->panel, &PatternEditorPanel::selected,
 					 this, [&](bool isSelected) { emit selected(isSelected); });
-	QObject::connect(ui->panel, &PatternEditorPanel::returnPressed,
-					 this, [&] { emit returnPressed(); });
 	QObject::connect(ui->panel, &PatternEditorPanel::instrumentEntered,
 					 this, [&](int num) { emit instrumentEntered(num); });
+	QObject::connect(ui->panel, &PatternEditorPanel::volumeEntered,
+					 this, [&](int volume) { emit volumeEntered(volume); });
 	QObject::connect(ui->panel, &PatternEditorPanel::effectEntered,
 					 this, [&](QString text) { emit effectEntered(text); });
 
@@ -100,6 +100,16 @@ void PatternEditor::updatePositionByStepUpdate(bool isFirstUpdate)
 	ui->panel->updatePositionByStepUpdate(isFirstUpdate);
 }
 
+void PatternEditor::updatepositionByPositionJump(bool trackChanged)
+{
+	ui->panel->updatePositionByStepUpdate(false, true, trackChanged);
+}
+
+void PatternEditor::changeMarker()
+{
+	ui->panel->changeMarker();
+}
+
 void PatternEditor::copySelectedCells()
 {
 	ui->panel->copySelectedCells();
@@ -232,6 +242,16 @@ void PatternEditor::onDefaultPatternSizeChanged()
 	ui->panel->onDefaultPatternSizeChanged();
 }
 
+void PatternEditor::onShortcutUpdated()
+{
+	ui->panel->onShortcutUpdated();
+}
+
+void PatternEditor::onPatternDataGlobalChanged()
+{
+	ui->panel->redrawByPatternChanged();
+}
+
 void PatternEditor::setPatternHighlight1Count(int count)
 {
 	ui->panel->setPatternHighlight1Count(count);
@@ -279,6 +299,11 @@ void PatternEditor::onPasteOverwritePressed()
 	ui->panel->onPasteOverwritePressed();
 }
 
+void PatternEditor::onPasteInsertPressed()
+{
+	ui->panel->onPasteInsertPressed();
+}
+
 void PatternEditor::onSelectPressed(int type)
 {
 	ui->panel->onSelectPressed(type);
@@ -286,7 +311,14 @@ void PatternEditor::onSelectPressed(int type)
 
 void PatternEditor::onTransposePressed(bool isOctave, bool isIncreased)
 {
-	ui->panel->onTransposePressed(isOctave, isIncreased);
+	int seminote = isOctave ? (isIncreased ? 12 : -12) : (isIncreased ? 1 : -1);
+	ui->panel->onNoteTransposePressed(seminote);
+}
+
+void PatternEditor::onChangeValuesPressed(bool isCoarse, bool isIncreased)
+{
+	int val = isCoarse ? (isIncreased ? 16 : -16) : (isIncreased ? 1 : -1);
+	ui->panel->onChangeValuesPressed(val);
 }
 
 void PatternEditor::onToggleTrackPressed()
@@ -299,16 +331,6 @@ void PatternEditor::onSoloTrackPressed()
 	ui->panel->onSoloTrackPressed(ui->panel->getCurrentTrack());
 }
 
-void PatternEditor::onExpandEffectColumn()
-{
-	ui->panel->onExpandEffectColumnPressed(ui->panel->getCurrentTrack());
-}
-
-void PatternEditor::onShrinkEffectColumn()
-{
-	ui->panel->onShrinkEffectColumnPressed(ui->panel->getCurrentTrack());
-}
-
 void PatternEditor::onExpandPressed()
 {
 	ui->panel->onExpandPressed();
@@ -349,6 +371,11 @@ void PatternEditor::onDuplicateInstrumentsRemoved()
 	ui->panel->redrawByPatternChanged();
 }
 
+void PatternEditor::onPlayStepPressed()
+{
+	ui->panel->onPlayStepPressed();
+}
+
 void PatternEditor::updateHorizontalSliderMaximum()
 {
 	if (!ui->panel->isReadyCore() || !songLoaded_) return;
