@@ -15,6 +15,7 @@
 #include <QUndoStack>
 #include <QString>
 #include <QPoint>
+#include <QShortcut>
 #include <memory>
 #include <vector>
 #include <atomic>
@@ -39,9 +40,11 @@ public:
 
 	void changeEditable();
 	int getFullColmunSize() const;
-	void updatePositionByStepUpdate(bool isFirstUpdate);
+	void updatePositionByStepUpdate(bool isFirstUpdate, bool forceJump = false, bool trackChanged = false);
 	int getScrollableCountByTrack() const;
 
+	void changeMarker();
+
 	void copySelectedCells();
 	void cutSelectedCells();
 
@@ -73,6 +76,7 @@ public slots:
 
 	void onOrderListEdited();
 	void onDefaultPatternSizeChanged();
+	void onShortcutUpdated();
 
 	void setPatternHighlight1Count(int count);
 	void setPatternHighlight2Count(int count);
@@ -84,6 +88,7 @@ public slots:
 	void onPastePressed();
 	void onPasteMixPressed();
 	void onPasteOverwritePressed();
+	void onPasteInsertPressed();
 	/// 0: None
 	/// 1: All
 	/// 2: Row
@@ -91,7 +96,7 @@ public slots:
 	/// 4: Pattern
 	/// 5: Order
 	void onSelectPressed(int type);
-	void onTransposePressed(bool isOctave, bool isIncreased);
+	void onNoteTransposePressed(int seminote);
 	void onToggleTrackPressed(int track);
 	void onSoloTrackPressed(int track);
 	void onUnmuteAllPressed();
@@ -103,6 +108,8 @@ public slots:
 	void onExpandEffectColumnPressed(int trackNum);
 	void onShrinkEffectColumnPressed(int trackNum);
 	void onFollowModeChanged();
+	void onChangeValuesPressed(int value);
+	void onPlayStepPressed();
 
 signals:
 	void hScrollBarChangeRequested(int num);
@@ -112,8 +119,8 @@ signals:
 	void effectColsCompanded(int num, int max);
 
 	void selected(bool isSelected);
-	void returnPressed();
 	void instrumentEntered(int num);
+	void volumeEntered(int volume);
 	void effectEntered(QString text);
 
 protected:
@@ -171,6 +178,7 @@ private:
 	PatternPosition selLeftAbovePos_, selRightBelowPos_;
 	PatternPosition shiftPressedPos_;
 	PatternPosition doubleClickPos_;
+	PatternPosition markerPos_;
 
 	bool isIgnoreToSlider_, isIgnoreToOrder_;
 
@@ -197,9 +205,18 @@ private:
 	std::atomic_bool repaintable_;	// Recurrensive repaint guard
 	std::atomic_int repaintingCnt_;
 
+	// Shortcuts
+	// index 1 is shift pressed
+	std::unique_ptr<QShortcut> upSc_[2], dnSc_[2], pgUpSc_[2], pgDnSc_[2], homeSc_[2], endSc_[2];
+	std::unique_ptr<QShortcut> hlUpSc_[2], hlDnSc_[2], ltSc_[2], rtSc_[2];
+
+	std::unique_ptr<QShortcut> keyOffSc_, echoBufSc_, stepMvUpSc_, stepMvDnSc_;
+	std::unique_ptr<QShortcut> expandColSc_, shrinkColSc_;
+
 	// Meta methods
 	int midiKeyEventMethod_;
 
+	void funcResize();
 	void updateSizes();
 	void initDisplay();
 	void drawPattern(const QRect& rect);
@@ -222,13 +239,18 @@ private:
 	inline void updateTracksWidthFromLeftToEnd()
 	{
 		tracksWidthFromLeftToEnd_ = calculateTracksWidthWithRowNum(
-					leftTrackNum_, static_cast<int>(songStyle_.trackAttribs.size()) - 1);
+										leftTrackNum_, static_cast<int>(songStyle_.trackAttribs.size()) - 1);
 	}
 
 	void moveCursorToRight(int n);
 	void moveViewToRight(int n);
 	void moveCursorToDown(int n);
 
+	inline void checkSelectionByCursorMove(bool isShift) {
+		if (isShift) setSelectedRectangle(shiftPressedPos_, curPos_);
+		else onSelectPressed(0);
+	}
+
 	bool enterToneData(QKeyEvent* event);
 	void setStepKeyOn(Note note, int octave);
 	bool enterInstrumentData(int key);
@@ -250,23 +272,23 @@ private:
 	void deletePreviousStep();
 
 	void eraseSelectedCells();
-	void pasteCopiedCells(const PatternPosition& startPos);
-	void pasteMixCopiedCells(const PatternPosition& startPos);
-	void pasteOverwriteCopiedCells(const PatternPosition& startPos);
-	std::vector<std::vector<std::string> > instantiateCellsFromString(QString str, int& startCol);
+	void pasteCopiedCells(const PatternPosition& cursorPos);
+	void pasteMixCopiedCells(const PatternPosition& cursorPos);
+	void pasteOverwriteCopiedCells(const PatternPosition& cursorPos);
+	void pasteInsertCopiedCells(const PatternPosition& cursorPos);
+	using PatternCells = std::vector<std::vector<std::string>>;
+	PatternCells decodeCells(QString str, int& startCol);
+	PatternPosition getPasteLeftAbovePosition(
+			int pasteCol, const PatternPosition& cursorPos, size_t cellW) const;
+	PatternCells compandPasteCells(const PatternPosition& laPos, const PatternCells& cells);
 
-	void increaseNoteKey(const PatternPosition& startPos, const PatternPosition& endPos);
-	void decreaseNoteKey(const PatternPosition& startPos, const PatternPosition& endPos);
-	void increaseNoteOctave(const PatternPosition& startPos, const PatternPosition& endPos);
-	void decreaseNoteOctave(const PatternPosition& startPos, const PatternPosition& endPos);
+	void transposeNote(const PatternPosition& startPos, const PatternPosition& endPos, int seminote);
+	void changeValuesInPattern(const PatternPosition& startPos, const PatternPosition& endPos, int value);
 
 	void setSelectedRectangle(const PatternPosition& start, const PatternPosition& end);
 	bool isSelectedCell(int trackNum, int colNum, int orderNum, int stepNum);
 
 	void showPatternContextMenu(const PatternPosition& pos, const QPoint& point);
-
-	// Layout decypherer
-	JamKey getJamKeyFromLayoutMapping(Qt::Key key);
 };
 
 #endif // PATTERN_EDITOR_PANEL_HPP
