@@ -1,11 +1,11 @@
 #include "instrument.hpp"
 #include <utility>
+#include <algorithm>
 
-AbstractInstrument::AbstractInstrument(int number, SoundSource source, std::string name, InstrumentsManager* owner)
+AbstractInstrument::AbstractInstrument(int number, std::string name, InstrumentsManager* owner)
 	: owner_(owner),
 	  number_(number),
-	  name_(name),
-	  source_(source)
+	  name_(name)
 {}
 
 int AbstractInstrument::getNumber() const
@@ -18,11 +18,6 @@ void AbstractInstrument::setNumber(int n)
 	number_ = n;
 }
 
-SoundSource AbstractInstrument::getSoundSource() const
-{
-	return source_;
-}
-
 std::string AbstractInstrument::getName() const
 {
 	return name_;
@@ -41,7 +36,7 @@ bool AbstractInstrument::isRegisteredWithManager() const
 /****************************************/
 
 InstrumentFM::InstrumentFM(int number, std::string name, InstrumentsManager* owner) :
-	AbstractInstrument(number, SoundSource::FM, name, owner),
+	AbstractInstrument(number, name, owner),
 	envNum_(0),
 	lfoEnabled_(false),
 	lfoNum_(0)
@@ -163,6 +158,16 @@ InstrumentFM::InstrumentFM(int number, std::string name, InstrumentsManager* own
 	};
 }
 
+SoundSource InstrumentFM::getSoundSource() const
+{
+	return SoundSource::FM;
+}
+
+InstrumentType InstrumentFM::getType() const
+{
+	return InstrumentType::FM;
+}
+
 std::unique_ptr<AbstractInstrument> InstrumentFM::clone()
 {
 	std::unique_ptr<InstrumentFM> c = std::make_unique<InstrumentFM>(number_, name_, owner_);
@@ -371,7 +376,7 @@ std::unique_ptr<CommandSequence::Iterator> InstrumentFM::getPitchSequenceIterato
 /****************************************/
 
 InstrumentSSG::InstrumentSSG(int number, std::string name, InstrumentsManager* owner)
-	: AbstractInstrument(number, SoundSource::SSG, name, owner),
+	: AbstractInstrument(number, name, owner),
 	  wfEnabled_(false),
 	  wfNum_(0),
 	  tnEnabled_(false),
@@ -385,11 +390,21 @@ InstrumentSSG::InstrumentSSG(int number, std::string name, InstrumentsManager* o
 {
 }
 
+SoundSource InstrumentSSG::getSoundSource() const
+{
+	return SoundSource::SSG;
+}
+
+InstrumentType InstrumentSSG::getType() const
+{
+	return InstrumentType::SSG;
+}
+
 std::unique_ptr<AbstractInstrument> InstrumentSSG::clone()
 {
 	std::unique_ptr<InstrumentSSG> c = std::make_unique<InstrumentSSG>(number_, name_, owner_);
-	c->setWaveFormEnabled(wfEnabled_);
-	c->setWaveFormNumber(wfNum_);
+	c->setWaveformEnabled(wfEnabled_);
+	c->setWaveformNumber(wfNum_);
 	c->setToneNoiseEnabled(tnEnabled_);
 	c->setToneNoiseNumber(tnNum_);
 	c->setEnvelopeEnabled(envEnabled_);
@@ -401,44 +416,44 @@ std::unique_ptr<AbstractInstrument> InstrumentSSG::clone()
 	return std::move(c);
 }
 
-void InstrumentSSG::setWaveFormEnabled(bool enabled)
+void InstrumentSSG::setWaveformEnabled(bool enabled)
 {
 	wfEnabled_ = enabled;
 }
 
-bool InstrumentSSG::getWaveFormEnabled() const
+bool InstrumentSSG::getWaveformEnabled() const
 {
 	return wfEnabled_;
 }
 
-void InstrumentSSG::setWaveFormNumber(int n)
+void InstrumentSSG::setWaveformNumber(int n)
 {
 	wfNum_ = n;
 }
 
-int InstrumentSSG::getWaveFormNumber() const
+int InstrumentSSG::getWaveformNumber() const
 {
 	return wfNum_;
 }
 
-std::vector<CommandSequenceUnit> InstrumentSSG::getWaveFormSequence() const
+std::vector<CommandSequenceUnit> InstrumentSSG::getWaveformSequence() const
 {
-	return owner_->getWaveFormSSGSequence(wfNum_);
+	return owner_->getWaveformSSGSequence(wfNum_);
 }
 
-std::vector<Loop> InstrumentSSG::getWaveFormLoops() const
+std::vector<Loop> InstrumentSSG::getWaveformLoops() const
 {
-	return owner_->getWaveFormSSGLoops(wfNum_);
+	return owner_->getWaveformSSGLoops(wfNum_);
 }
 
-Release InstrumentSSG::getWaveFormRelease() const
+Release InstrumentSSG::getWaveformRelease() const
 {
-	return owner_->getWaveFormSSGRelease(wfNum_);
+	return owner_->getWaveformSSGRelease(wfNum_);
 }
 
-std::unique_ptr<CommandSequence::Iterator> InstrumentSSG::getWaveFormSequenceIterator() const
+std::unique_ptr<CommandSequence::Iterator> InstrumentSSG::getWaveformSequenceIterator() const
 {
-	return owner_->getWaveFormSSGIterator(wfNum_);
+	return owner_->getWaveformSSGIterator(wfNum_);
 }
 
 void InstrumentSSG::setToneNoiseEnabled(bool enabled)
@@ -610,3 +625,307 @@ std::unique_ptr<CommandSequence::Iterator> InstrumentSSG::getPitchSequenceIterat
 {
 	return owner_->getPitchSSGIterator(ptNum_);
 }
+
+/****************************************/
+
+InstrumentADPCM::InstrumentADPCM(int number, std::string name, InstrumentsManager* owner)
+	: AbstractInstrument(number, name, owner),
+	  sampNum_(0),
+	  envEnabled_(false),
+	  envNum_(0),
+	  arpEnabled_(false),
+	  arpNum_(0),
+	  ptEnabled_(false),
+	  ptNum_(0)
+{
+}
+
+SoundSource InstrumentADPCM::getSoundSource() const
+{
+	return SoundSource::ADPCM;
+}
+
+InstrumentType InstrumentADPCM::getType() const
+{
+	return InstrumentType::ADPCM;
+}
+
+std::unique_ptr<AbstractInstrument> InstrumentADPCM::clone()
+{
+	std::unique_ptr<InstrumentADPCM> c = std::make_unique<InstrumentADPCM>(number_, name_, owner_);
+	c->setSampleNumber(sampNum_);
+	c->setEnvelopeEnabled(envEnabled_);
+	c->setEnvelopeNumber(envNum_);
+	c->setArpeggioEnabled(arpEnabled_);
+	c->setArpeggioNumber(arpNum_);
+	c->setPitchEnabled(ptEnabled_);
+	c->setPitchNumber(ptNum_);
+	return std::move(c);
+}
+
+void InstrumentADPCM::setSampleNumber(int n)
+{
+	sampNum_ = n;
+}
+
+int InstrumentADPCM::getSampleNumber() const
+{
+	return sampNum_;
+}
+
+int InstrumentADPCM::getSampleRootKeyNumber() const
+{
+	return owner_->getSampleADPCMRootKeyNumber(sampNum_);
+}
+
+int InstrumentADPCM::getSampleRootDeltaN() const
+{
+	return owner_->getSampleADPCMRootDeltaN(sampNum_);
+}
+
+bool InstrumentADPCM::isSampleRepeatable() const
+{
+	return owner_->isSampleADPCMRepeatable(sampNum_);
+}
+
+std::vector<uint8_t> InstrumentADPCM::getRawSample() const
+{
+	return owner_->getSampleADPCMRawSample(sampNum_);
+}
+
+size_t InstrumentADPCM::getSampleStartAddress() const
+{
+	return owner_->getSampleADPCMStartAddress(sampNum_);
+}
+
+size_t InstrumentADPCM::getSampleStopAddress() const
+{
+	return owner_->getSampleADPCMStopAddress(sampNum_);
+}
+
+void InstrumentADPCM::setEnvelopeEnabled(bool enabled)
+{
+	envEnabled_ = enabled;
+}
+
+bool InstrumentADPCM::getEnvelopeEnabled() const
+{
+	return envEnabled_;
+}
+
+void InstrumentADPCM::setEnvelopeNumber(int n)
+{
+	envNum_ = n;
+}
+
+int InstrumentADPCM::getEnvelopeNumber() const
+{
+	return envNum_;
+}
+
+std::vector<CommandSequenceUnit> InstrumentADPCM::getEnvelopeSequence() const
+{
+	return owner_->getEnvelopeADPCMSequence(envNum_);
+}
+
+std::vector<Loop> InstrumentADPCM::getEnvelopeLoops() const
+{
+	return owner_->getEnvelopeADPCMLoops(envNum_);
+}
+
+Release InstrumentADPCM::getEnvelopeRelease() const
+{
+	return owner_->getEnvelopeADPCMRelease(envNum_);
+}
+
+std::unique_ptr<CommandSequence::Iterator> InstrumentADPCM::getEnvelopeSequenceIterator() const
+{
+	return owner_->getEnvelopeADPCMIterator(envNum_);
+}
+
+void InstrumentADPCM::setArpeggioEnabled(bool enabled)
+{
+	arpEnabled_ = enabled;
+}
+
+bool InstrumentADPCM::getArpeggioEnabled() const
+{
+	return arpEnabled_;
+}
+
+void InstrumentADPCM::setArpeggioNumber(int n)
+{
+	arpNum_ = n;
+}
+
+int InstrumentADPCM::getArpeggioNumber() const
+{
+	return arpNum_;
+}
+
+SequenceType InstrumentADPCM::getArpeggioType() const
+{
+	return owner_->getArpeggioADPCMType(arpNum_);
+}
+
+std::vector<CommandSequenceUnit> InstrumentADPCM::getArpeggioSequence() const
+{
+	return owner_->getArpeggioADPCMSequence(arpNum_);
+}
+
+std::vector<Loop> InstrumentADPCM::getArpeggioLoops() const
+{
+	return owner_->getArpeggioADPCMLoops(arpNum_);
+}
+
+Release InstrumentADPCM::getArpeggioRelease() const
+{
+	return owner_->getArpeggioADPCMRelease(arpNum_);
+}
+
+std::unique_ptr<CommandSequence::Iterator> InstrumentADPCM::getArpeggioSequenceIterator() const
+{
+	return owner_->getArpeggioADPCMIterator(arpNum_);
+}
+
+void InstrumentADPCM::setPitchEnabled(bool enabled)
+{
+	ptEnabled_ = enabled;
+}
+
+bool InstrumentADPCM::getPitchEnabled() const
+{
+	return ptEnabled_;
+}
+
+void InstrumentADPCM::setPitchNumber(int n)
+{
+	ptNum_ = n;
+}
+
+int InstrumentADPCM::getPitchNumber() const
+{
+	return ptNum_;
+}
+
+SequenceType InstrumentADPCM::getPitchType() const
+{
+	return owner_->getPitchADPCMType(ptNum_);
+}
+
+std::vector<CommandSequenceUnit> InstrumentADPCM::getPitchSequence() const
+{
+	return owner_->getPitchADPCMSequence(ptNum_);
+}
+
+std::vector<Loop> InstrumentADPCM::getPitchLoops() const
+{
+	return owner_->getPitchADPCMLoops(ptNum_);
+}
+
+Release InstrumentADPCM::getPitchRelease() const
+{
+	return owner_->getPitchADPCMRelease(ptNum_);
+}
+
+std::unique_ptr<CommandSequence::Iterator> InstrumentADPCM::getPitchSequenceIterator() const
+{
+	return owner_->getPitchADPCMIterator(ptNum_);
+}
+
+/****************************************/
+
+InstrumentDrumkit::InstrumentDrumkit(int number, std::string name, InstrumentsManager* owner)
+	: AbstractInstrument(number, name, owner)
+{
+}
+
+SoundSource InstrumentDrumkit::getSoundSource() const
+{
+	return SoundSource::ADPCM;
+}
+
+InstrumentType InstrumentDrumkit::getType() const
+{
+	return InstrumentType::Drumkit;
+}
+
+std::unique_ptr<AbstractInstrument> InstrumentDrumkit::clone()
+{
+	std::unique_ptr<InstrumentDrumkit> c = std::make_unique<InstrumentDrumkit>(number_, name_, owner_);
+
+	for (const auto& pair : kit_) {
+		c->setSampleEnabled(pair.first, true);
+		c->setSampleNumber(pair.first, pair.second.sampNum);
+		c->setPitch(pair.first, pair.second.pitch);
+	}
+	return std::move(c);
+}
+
+std::vector<int> InstrumentDrumkit::getAssignedKeys() const
+{
+	std::vector<int> keys(kit_.size());
+	std::transform(kit_.begin(), kit_.end(), keys.begin(), [](const auto& pair) { return pair.first; });
+	return keys;
+}
+
+void InstrumentDrumkit::setSampleEnabled(int key, bool enabled)
+{
+	if (enabled) kit_[key] = { 0, 0 };
+	else kit_.erase(key);
+}
+
+bool InstrumentDrumkit::getSampleEnabled(int key) const
+{
+	return kit_.count(key);
+}
+
+void InstrumentDrumkit::setSampleNumber(int key, int n)
+{
+	if (kit_.count(key)) kit_.at(key).sampNum = n;
+}
+
+int InstrumentDrumkit::getSampleNumber(int key) const
+{
+	return kit_.at(key).sampNum;
+}
+
+int InstrumentDrumkit::getSampleRootKeyNumber(int key) const
+{
+	return owner_->getSampleADPCMRootKeyNumber(kit_.at(key).sampNum);
+}
+
+int InstrumentDrumkit::getSampleRootDeltaN(int key) const
+{
+	return owner_->getSampleADPCMRootDeltaN(kit_.at(key).sampNum);
+}
+
+bool InstrumentDrumkit::isSampleRepeatable(int key) const
+{
+	return owner_->isSampleADPCMRepeatable(kit_.at(key).sampNum);
+}
+
+std::vector<uint8_t> InstrumentDrumkit::getRawSample(int key) const
+{
+	return owner_->getSampleADPCMRawSample(kit_.at(key).sampNum);
+}
+
+size_t InstrumentDrumkit::getSampleStartAddress(int key) const
+{
+	return owner_->getSampleADPCMStartAddress(kit_.at(key).sampNum);
+}
+
+size_t InstrumentDrumkit::getSampleStopAddress(int key) const
+{
+	return owner_->getSampleADPCMStopAddress(kit_.at(key).sampNum);
+}
+
+void InstrumentDrumkit::setPitch(int key, int pitch)
+{
+	if (kit_.count(key)) kit_.at(key).pitch = pitch;
+}
+
+int InstrumentDrumkit::getPitch(int key) const
+{
+	return kit_.at(key).pitch;
+}
