@@ -8,26 +8,31 @@
 #include "instrument.hpp"
 #include "envelope_fm.hpp"
 #include "lfo_fm.hpp"
+#include "waveform_adpcm.hpp"
 #include "command_sequence.hpp"
 #include "enum_hash.hpp"
 #include "misc.hpp"
 
+enum class InstrumentType;
 class AbstractInstrument;
 class InstrumentFM;
 enum class FMEnvelopeParameter;
 class EnvelopeFM;
 class InstrumentSSG;
+class InstrumentADPCM;
+class InstrumentDrumkit;
 
 class InstrumentsManager
 {
 public:
-	InstrumentsManager();
+	explicit InstrumentsManager(bool unedited);
 
-	void addInstrument(int instNum, SoundSource source, std::string name);
+	void addInstrument(int instNum, InstrumentType type, std::string name);
 	void addInstrument(std::unique_ptr<AbstractInstrument> inst);
 	std::unique_ptr<AbstractInstrument> removeInstrument(int instNum);
 	void cloneInstrument(int cloneInstNum, int resInstNum);
 	void deepCloneInstrument(int cloneInstNum, int resInstNum);
+	void swapInstruments(int inst1Num, int inst2Num);
 	std::shared_ptr<AbstractInstrument> getInstrumentSharedPtr(int instNum);
 	void clearAll();
 	std::vector<int> getInstrumentIndices() const;
@@ -44,21 +49,23 @@ public:
 
 	std::vector<std::vector<int>> checkDuplicateInstruments() const;
 
+	void setPropertyFindMode(bool unedited);
+
 private:
 	std::array<std::shared_ptr<AbstractInstrument>, 128> insts_;
+	bool regardingUnedited_;
 
 	//----- FM methods -----
 public:
 	void setInstrumentFMEnvelope(int instNum, int envNum);
-    int getInstrumentFMEnvelope(int instNum) const;
+	int getInstrumentFMEnvelope(int instNum) const;
 	void setEnvelopeFMParameter(int envNum, FMEnvelopeParameter param, int value);
 	int getEnvelopeFMParameter(int envNum, FMEnvelopeParameter param) const;
 	void setEnvelopeFMOperatorEnabled(int envNum, int opNum, bool enabled);
 	bool getEnvelopeFMOperatorEnabled(int envNum, int opNum) const;
 	std::vector<int> getEnvelopeFMUsers(int envNum) const;
 	std::vector<int> getEnvelopeFMEntriedIndices() const;
-	int findFirstFreeEnvelopeFM() const;
-	int findFirstFreePlainEnvelopeFM() const;
+	int findFirstAssignableEnvelopeFM() const;
 
 	void setInstrumentFMLFOEnabled(int instNum, bool enabled);
 	bool getInstrumentFMLFOEnabled(int instNum) const;
@@ -68,8 +75,7 @@ public:
 	int getLFOFMparameter(int lfoNum, FMLFOParameter param) const;
 	std::vector<int> getLFOFMUsers(int lfoNum) const;
 	std::vector<int> getLFOFMEntriedIndices() const;
-	int findFirstFreeLFOFM() const;
-	int findFirstFreePlainLFOFM() const;
+	int findFirstAssignableLFOFM() const;
 
 	void setInstrumentFMOperatorSequenceEnabled(int instNum, FMEnvelopeParameter param, bool enabled);
 	bool getInstrumentFMOperatorSequenceEnabled(int instNum, FMEnvelopeParameter param) const;
@@ -86,8 +92,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getOperatorSequenceFMIterator(FMEnvelopeParameter param, int opSeqNum) const;
 	std::vector<int> getOperatorSequenceFMUsers(FMEnvelopeParameter param, int opSeqNum) const;
 	std::vector<int> getOperatorSequenceFMEntriedIndices(FMEnvelopeParameter param) const;
-	int findFirstFreeOperatorSequenceFM(FMEnvelopeParameter param) const;
-	int findFirstFreePlainOperatorSequenceFM(FMEnvelopeParameter param) const;
+	int findFirstAssignableOperatorSequenceFM(FMEnvelopeParameter param) const;
 
 	void setInstrumentFMArpeggioEnabled(int instNum, FMOperatorType op, bool enabled);
 	bool getInstrumentFMArpeggioEnabled(int instNum, FMOperatorType op) const;
@@ -106,8 +111,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getArpeggioFMIterator(int arpNum) const;
 	std::vector<int> getArpeggioFMUsers(int arpNum) const;
 	std::vector<int> getArpeggioFMEntriedIndices() const;
-	int findFirstFreeArpeggioFM() const;
-	int findFirstFreePlainArpeggioFM() const;
+	int findFirstAssignableArpeggioFM() const;
 
 	void setInstrumentFMPitchEnabled(int instNum, FMOperatorType op, bool enabled);
 	bool getInstrumentFMPitchEnabled(int instNum, FMOperatorType op) const;
@@ -126,8 +130,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getPitchFMIterator(int ptNum) const;
 	std::vector<int> getPitchFMUsers(int ptNum) const;
 	std::vector<int> getPitchFMEntriedIndices() const;
-	int findFirstFreePitchFM() const;
-	int findFirstFreePlainPitchFM() const;
+	int findFirstAssignablePitchFM() const;
 
 	void setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorType op, bool enabled);
 
@@ -138,8 +141,8 @@ private:
 	std::array<std::shared_ptr<CommandSequence>, 128> arpFM_;
 	std::array<std::shared_ptr<CommandSequence>, 128> ptFM_;
 
-	std::vector<FMEnvelopeParameter> envFMParams_;
-	std::vector<FMOperatorType> fmOpTypes_;
+	static const FMEnvelopeParameter ENV_FM_PARAMS_[38];
+	static const FMOperatorType FM_OP_TYPES_[5];
 
 	int cloneFMEnvelope(int srcNum);
 	int cloneFMLFO(int srcNum);
@@ -147,27 +150,26 @@ private:
 	int cloneFMArpeggio(int srcNum);
 	int cloneFMPitch(int srcNum);
 
-	bool equalPropertiesFM(std::shared_ptr<InstrumentFM> a, std::shared_ptr<InstrumentFM> b) const;
+	bool equalPropertiesFM(std::shared_ptr<AbstractInstrument> a, std::shared_ptr<AbstractInstrument> b) const;
 
 	//----- SSG methods -----
 public:
-	void setInstrumentSSGWaveFormEnabled(int instNum, bool enabled);
-	bool getInstrumentSSGWaveFormEnabled(int instNum) const;
-	void setInstrumentSSGWaveForm(int instNum, int wfNum);
-	int getInstrumentSSGWaveForm(int instNum);
-	void addWaveFormSSGSequenceCommand(int wfNum, int type, int data);
-	void removeWaveFormSSGSequenceCommand(int wfNum);
-	void setWaveFormSSGSequenceCommand(int wfNum, int cnt, int type, int data);
-	std::vector<CommandSequenceUnit> getWaveFormSSGSequence(int wfNum);
-	void setWaveFormSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
-	std::vector<Loop> getWaveFormSSGLoops(int wfNum) const;
-	void setWaveFormSSGRelease(int wfNum, ReleaseType type, int begin);
-	Release getWaveFormSSGRelease(int wfNum) const;
-	std::unique_ptr<CommandSequence::Iterator> getWaveFormSSGIterator(int wfNum) const;
-	std::vector<int> getWaveFormSSGUsers(int wfNum) const;
-	std::vector<int> getWaveFormSSGEntriedIndices() const;
-	int findFirstFreeWaveFormSSG() const;
-	int findFirstFreePlainWaveFormSSG() const;
+	void setInstrumentSSGWaveformEnabled(int instNum, bool enabled);
+	bool getInstrumentSSGWaveformEnabled(int instNum) const;
+	void setInstrumentSSGWaveform(int instNum, int wfNum);
+	int getInstrumentSSGWaveform(int instNum);
+	void addWaveformSSGSequenceCommand(int wfNum, int type, int data);
+	void removeWaveformSSGSequenceCommand(int wfNum);
+	void setWaveformSSGSequenceCommand(int wfNum, int cnt, int type, int data);
+	std::vector<CommandSequenceUnit> getWaveformSSGSequence(int wfNum);
+	void setWaveformSSGLoops(int wfNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	std::vector<Loop> getWaveformSSGLoops(int wfNum) const;
+	void setWaveformSSGRelease(int wfNum, ReleaseType type, int begin);
+	Release getWaveformSSGRelease(int wfNum) const;
+	std::unique_ptr<CommandSequence::Iterator> getWaveformSSGIterator(int wfNum) const;
+	std::vector<int> getWaveformSSGUsers(int wfNum) const;
+	std::vector<int> getWaveformSSGEntriedIndices() const;
+	int findFirstAssignableWaveformSSG() const;
 
 	void setInstrumentSSGToneNoiseEnabled(int instNum, bool enabled);
 	bool getInstrumentSSGToneNoiseEnabled(int instNum) const;
@@ -184,8 +186,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getToneNoiseSSGIterator(int tnNum) const;
 	std::vector<int> getToneNoiseSSGUsers(int tnNum) const;
 	std::vector<int> getToneNoiseSSGEntriedIndices() const;
-	int findFirstFreeToneNoiseSSG() const;
-	int findFirstFreePlainToneNoiseSSG() const;
+	int findFirstAssignableToneNoiseSSG() const;
 
 	void setInstrumentSSGEnvelopeEnabled(int instNum, bool enabled);
 	bool getInstrumentSSGEnvelopeEnabled(int instNum) const;
@@ -202,8 +203,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getEnvelopeSSGIterator(int envNum) const;
 	std::vector<int> getEnvelopeSSGUsers(int envNum) const;
 	std::vector<int> getEnvelopeSSGEntriedIndices() const;
-	int findFirstFreeEnvelopeSSG() const;
-	int findFirstFreePlainEnvelopeSSG() const;
+	int findFirstAssignableEnvelopeSSG() const;
 
 	void setInstrumentSSGArpeggioEnabled(int instNum, bool enabled);
 	bool getInstrumentSSGArpeggioEnabled(int instNum) const;
@@ -222,8 +222,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getArpeggioSSGIterator(int arpNum) const;
 	std::vector<int> getArpeggioSSGUsers(int arpNum) const;
 	std::vector<int> getArpeggioSSGEntriedIndices() const;
-	int findFirstFreeArpeggioSSG() const;
-	int findFirstFreePlainArpeggioSSG() const;
+	int findFirstAssignableArpeggioSSG() const;
 
 	void setInstrumentSSGPitchEnabled(int instNum, bool enabled);
 	bool getInstrumentSSGPitchEnabled(int instNum) const;
@@ -242,8 +241,7 @@ public:
 	std::unique_ptr<CommandSequence::Iterator> getPitchSSGIterator(int ptNum) const;
 	std::vector<int> getPitchSSGUsers(int ptNum) const;
 	std::vector<int> getPitchSSGEntriedIndices() const;
-	int findFirstFreePitchSSG() const;
-	int findFirstFreePlainPitchSSG() const;
+	int findFirstAssignablePitchSSG() const;
 
 private:
 	std::array<std::shared_ptr<CommandSequence>, 128> wfSSG_;
@@ -252,11 +250,114 @@ private:
 	std::array<std::shared_ptr<CommandSequence>, 128> arpSSG_;
 	std::array<std::shared_ptr<CommandSequence>, 128> ptSSG_;
 
-	int cloneSSGWaveForm(int srcNum);
+	int cloneSSGWaveform(int srcNum);
 	int cloneSSGToneNoise(int srcNum);
 	int cloneSSGEnvelope(int srcNum);
 	int cloneSSGArpeggio(int srcNum);
 	int cloneSSGPitch(int srcNum);
 
-	bool equalPropertiesSSG(std::shared_ptr<InstrumentSSG> a, std::shared_ptr<InstrumentSSG> b) const;
+	bool equalPropertiesSSG(std::shared_ptr<AbstractInstrument> a, std::shared_ptr<AbstractInstrument> b) const;
+
+	//----- ADPCM methods -----
+public:
+	void setInstrumentADPCMSample(int instNum, int sampNum);
+	int getInstrumentADPCMSample(int instNum);
+	void setSampleADPCMRootKeyNumber(int sampNum, int n);
+	int getSampleADPCMRootKeyNumber(int sampNum) const;
+	void setSampleADPCMRootDeltaN(int sampNum, int dn);
+	int getSampleADPCMRootDeltaN(int sampNum) const;
+	void setSampleADPCMRepeatEnabled(int sampNum, bool enabled);
+	bool isSampleADPCMRepeatable(int sampNum) const;
+	void storeSampleADPCMRawSample(int sampNum, std::vector<uint8_t> sample);
+	void clearSampleADPCMRawSample(int sampNum);
+	std::vector<uint8_t> getSampleADPCMRawSample(int sampNum) const;
+	void setSampleADPCMStartAddress(int sampNum, size_t addr);
+	size_t getSampleADPCMStartAddress(int sampNum) const;
+	void setSampleADPCMStopAddress(int sampNum, size_t addr);
+	size_t getSampleADPCMStopAddress(int sampNum) const;
+	std::vector<int> getSampleADPCMUsers(int sampNum) const;
+	std::vector<int> getSampleADPCMEntriedIndices() const;
+	std::vector<int> getSampleADPCMValidIndices() const;
+	void clearUnusedSamplesADPCM();
+	int findFirstAssignableSampleADPCM(int startIndex = 0) const;
+
+	void setInstrumentADPCMEnvelopeEnabled(int instNum, bool enabled);
+	bool getInstrumentADPCMEnvelopeEnabled(int instNum) const;
+	void setInstrumentADPCMEnvelope(int instNum, int envNum);
+	int getInstrumentADPCMEnvelope(int instNum);
+	void addEnvelopeADPCMSequenceCommand(int envNum, int type, int data);
+	void removeEnvelopeADPCMSequenceCommand(int envNum);
+	void setEnvelopeADPCMSequenceCommand(int envNum, int cnt, int type, int data);
+	std::vector<CommandSequenceUnit> getEnvelopeADPCMSequence(int envNum);
+	void setEnvelopeADPCMLoops(int envNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	std::vector<Loop> getEnvelopeADPCMLoops(int envNum) const;
+	void setEnvelopeADPCMRelease(int envNum, ReleaseType type, int begin);
+	Release getEnvelopeADPCMRelease(int envNum) const;
+	std::unique_ptr<CommandSequence::Iterator> getEnvelopeADPCMIterator(int envNum) const;
+	std::vector<int> getEnvelopeADPCMUsers(int envNum) const;
+	std::vector<int> getEnvelopeADPCMEntriedIndices() const;
+	int findFirstAssignableEnvelopeADPCM() const;
+
+	void setInstrumentADPCMArpeggioEnabled(int instNum, bool enabled);
+	bool getInstrumentADPCMArpeggioEnabled(int instNum) const;
+	void setInstrumentADPCMArpeggio(int instNum, int arpNum);
+	int getInstrumentADPCMArpeggio(int instNum);
+	void setArpeggioADPCMType(int arpNum, SequenceType type);
+	SequenceType getArpeggioADPCMType(int arpNum) const;
+	void addArpeggioADPCMSequenceCommand(int arpNum, int type, int data);
+	void removeArpeggioADPCMSequenceCommand(int arpNum);
+	void setArpeggioADPCMSequenceCommand(int arpNum, int cnt, int type, int data);
+	std::vector<CommandSequenceUnit> getArpeggioADPCMSequence(int arpNum);
+	void setArpeggioADPCMLoops(int arpNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	std::vector<Loop> getArpeggioADPCMLoops(int arpNum) const;
+	void setArpeggioADPCMRelease(int arpNum, ReleaseType type, int begin);
+	Release getArpeggioADPCMRelease(int arpNum) const;
+	std::unique_ptr<CommandSequence::Iterator> getArpeggioADPCMIterator(int arpNum) const;
+	std::vector<int> getArpeggioADPCMUsers(int arpNum) const;
+	std::vector<int> getArpeggioADPCMEntriedIndices() const;
+	int findFirstAssignableArpeggioADPCM() const;
+
+	void setInstrumentADPCMPitchEnabled(int instNum, bool enabled);
+	bool getInstrumentADPCMPitchEnabled(int instNum) const;
+	void setInstrumentADPCMPitch(int instNum, int ptNum);
+	int getInstrumentADPCMPitch(int instNum);
+	void setPitchADPCMType(int ptNum, SequenceType type);
+	SequenceType getPitchADPCMType(int ptNum) const;
+	void addPitchADPCMSequenceCommand(int ptNum, int type, int data);
+	void removePitchADPCMSequenceCommand(int ptNum);
+	void setPitchADPCMSequenceCommand(int ptNum, int cnt, int type, int data);
+	std::vector<CommandSequenceUnit> getPitchADPCMSequence(int ptNum);
+	void setPitchADPCMLoops(int ptNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
+	std::vector<Loop> getPitchADPCMLoops(int ptNum) const;
+	void setPitchADPCMRelease(int ptNum, ReleaseType type, int begin);
+	Release getPitchADPCMRelease(int ptNum) const;
+	std::unique_ptr<CommandSequence::Iterator> getPitchADPCMIterator(int ptNum) const;
+	std::vector<int> getPitchADPCMUsers(int ptNum) const;
+	std::vector<int> getPitchADPCMEntriedIndices() const;
+	int findFirstAssignablePitchADPCM() const;
+
+private:
+	std::array<std::shared_ptr<SampleADPCM>, 128> sampADPCM_;
+	std::array<std::shared_ptr<CommandSequence>, 128> envADPCM_;
+	std::array<std::shared_ptr<CommandSequence>, 128> arpADPCM_;
+	std::array<std::shared_ptr<CommandSequence>, 128> ptADPCM_;
+
+	int cloneADPCMSample(int srcNum);
+	int cloneADPCMEnvelope(int srcNum);
+	int cloneADPCMArpeggio(int srcNum);
+	int cloneADPCMPitch(int srcNum);
+
+	bool equalPropertiesADPCM(std::shared_ptr<AbstractInstrument> a, std::shared_ptr<AbstractInstrument> b) const;
+
+	//----- Drumkit methods -----
+public:
+	void setInstrumentDrumkitSamplesEnabled(int instNum, int key, bool enabled);
+	bool getInstrumentDrumkitSamplesEnabled(int instNum, int key) const;
+	void setInstrumentDrumkitSamples(int instNum, int key, int sampNum);
+	int getInstrumentDrumkitSamples(int instNum, int key);
+
+	void setInstrumentDrumkitPitch(int instNum, int key, int pitch);
+
+private:
+	bool equalPropertiesDrumkit(std::shared_ptr<AbstractInstrument> a, std::shared_ptr<AbstractInstrument> b) const;
 };
