@@ -30,8 +30,8 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 			std::string name = inst->getName();
 			ctr.appendUint32(name.length());
 			if (!name.empty()) ctr.appendString(name);
-			switch (inst->getSoundSource()) {
-			case SoundSource::FM:
+			switch (inst->getType()) {
+			case InstrumentType::FM:
 			{
 				ctr.appendUint8(0x00);
 				auto instFM = std::dynamic_pointer_cast<InstrumentFM>(inst);
@@ -62,12 +62,12 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 				}
 				break;
 			}
-			case SoundSource::SSG:
+			case InstrumentType::SSG:
 			{
 				ctr.appendUint8(0x01);
 				auto instSSG = std::dynamic_pointer_cast<InstrumentSSG>(inst);
-				uint8_t tmp = static_cast<uint8_t>(instSSG->getWaveFormNumber());
-				ctr.appendUint8(instSSG->getWaveFormEnabled() ? tmp : (0x80 | tmp));
+				uint8_t tmp = static_cast<uint8_t>(instSSG->getWaveformNumber());
+				ctr.appendUint8(instSSG->getWaveformEnabled() ? tmp : (0x80 | tmp));
 				tmp = static_cast<uint8_t>(instSSG->getToneNoiseNumber());
 				ctr.appendUint8(instSSG->getToneNoiseEnabled() ? tmp : (0x80 | tmp));
 				tmp = static_cast<uint8_t>(instSSG->getEnvelopeNumber());
@@ -78,9 +78,33 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 				ctr.appendUint8(instSSG->getPitchEnabled() ? tmp : (0x80 | tmp));
 				break;
 			}
-			case SoundSource::DRUM:
+			case InstrumentType::ADPCM:
+			{
+				ctr.appendUint8(0x02);
+				auto instADPCM = std::dynamic_pointer_cast<InstrumentADPCM>(inst);
+				ctr.appendUint8(static_cast<uint8_t>(instADPCM->getSampleNumber()));
+				uint8_t tmp = static_cast<uint8_t>(instADPCM->getEnvelopeNumber());
+				ctr.appendUint8(instADPCM->getEnvelopeEnabled() ? tmp : (0x80 | tmp));
+				tmp = static_cast<uint8_t>(instADPCM->getArpeggioNumber());
+				ctr.appendUint8(instADPCM->getArpeggioEnabled() ? tmp : (0x80 | tmp));
+				tmp = static_cast<uint8_t>(instADPCM->getPitchNumber());
+				ctr.appendUint8(instADPCM->getPitchEnabled() ? tmp : (0x80 | tmp));
 				break;
 			}
+			case InstrumentType::Drumkit:
+			{
+				ctr.appendUint8(0x03);
+				auto instKit = std::dynamic_pointer_cast<InstrumentDrumkit>(inst);
+				std::vector<int> keys = instKit->getAssignedKeys();
+				ctr.appendUint8(keys.size());
+				for (const int& key : keys) {
+					ctr.appendUint8(static_cast<uint8_t>(key));
+					ctr.appendUint8(static_cast<uint8_t>(instKit->getSampleNumber(key)));
+					ctr.appendInt8(instKit->getPitch(key));
+				}
+				break;
+			}
+			}
 			ctr.writeUint32(iOfs, ctr.size() - iOfs);
 		}
 	}
@@ -383,10 +407,10 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 		}
 	}
 
-	// SSG wave form
+	// SSG waveform
 	std::vector<int> wfSSGIdcs;
-	for (auto& idx : instMan.lock()->getWaveFormSSGEntriedIndices()) {
-		std::vector<int> users = instMan.lock()->getWaveFormSSGUsers(idx);
+	for (auto& idx : instMan.lock()->getWaveformSSGEntriedIndices()) {
+		std::vector<int> users = instMan.lock()->getWaveformSSGUsers(idx);
 		std::vector<int> intersection;
 		std::set_intersection(users.begin(), users.end(), instNums.begin(), instNums.end(), std::back_inserter(intersection));
 		if (!intersection.empty()) wfSSGIdcs.push_back(idx);
@@ -398,20 +422,20 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 			ctr.appendUint8(static_cast<uint8_t>(idx));
 			size_t ofs = ctr.size();
 			ctr.appendUint16(0);	// Dummy offset
-			auto seq = instMan.lock()->getWaveFormSSGSequence(idx);
+			auto seq = instMan.lock()->getWaveformSSGSequence(idx);
 			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
 			for (auto& com : seq) {
 				ctr.appendUint16(static_cast<uint16_t>(com.type));
 				ctr.appendInt32(static_cast<int32_t>(com.data));
 			}
-			auto loop = instMan.lock()->getWaveFormSSGLoops(idx);
+			auto loop = instMan.lock()->getWaveformSSGLoops(idx);
 			ctr.appendUint16(static_cast<uint16_t>(loop.size()));
 			for (auto& l : loop) {
 				ctr.appendUint16(static_cast<uint16_t>(l.begin));
 				ctr.appendUint16(static_cast<uint16_t>(l.end));
 				ctr.appendUint8(static_cast<uint8_t>(l.times));
 			}
-			auto release = instMan.lock()->getWaveFormSSGRelease(idx);
+			auto release = instMan.lock()->getWaveformSSGRelease(idx);
 			switch (release.type) {
 			case ReleaseType::NoRelease:
 				ctr.appendUint8(0x00);
@@ -650,6 +674,195 @@ void BankIO::saveBank(BinaryContainer& ctr, std::vector<int> instNums,
 		}
 	}
 
+	// ADPCM sample
+	std::vector<int> sampADPCMIdcs;
+	for (auto& idx : instMan.lock()->getSampleADPCMEntriedIndices()) {
+		std::vector<int> users = instMan.lock()->getSampleADPCMUsers(idx);
+		std::vector<int> intersection;
+		std::set_intersection(users.begin(), users.end(), instNums.begin(), instNums.end(), std::back_inserter(intersection));
+		if (!intersection.empty()) sampADPCMIdcs.push_back(idx);
+	}
+	if (!sampADPCMIdcs.empty()) {
+		ctr.appendUint8(0x40);
+		ctr.appendUint8(static_cast<uint8_t>(sampADPCMIdcs.size()));
+		for (auto& idx : sampADPCMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint32(0);	// Dummy offset
+			ctr.appendUint8(static_cast<uint8_t>(instMan.lock()->getSampleADPCMRootKeyNumber(idx)));
+			ctr.appendUint16(static_cast<uint16_t>(instMan.lock()->getSampleADPCMRootDeltaN(idx)));
+			ctr.appendUint8(static_cast<uint8_t>(instMan.lock()->isSampleADPCMRepeatable(idx)));
+			std::vector<uint8_t> samples = instMan.lock()->getSampleADPCMRawSample(idx);
+			ctr.appendUint32(samples.size());
+			ctr.appendVector(std::move(samples));
+			ctr.writeUint32(ofs, ctr.size() - ofs);
+		}
+	}
+
+	// ADPCM envelope
+	std::vector<int> envADPCMIdcs;
+	for (auto& idx : instMan.lock()->getEnvelopeADPCMEntriedIndices()) {
+		std::vector<int> users = instMan.lock()->getEnvelopeADPCMUsers(idx);
+		std::vector<int> intersection;
+		std::set_intersection(users.begin(), users.end(), instNums.begin(), instNums.end(), std::back_inserter(intersection));
+		if (!intersection.empty()) envADPCMIdcs.push_back(idx);
+	}
+	if (!envADPCMIdcs.empty()) {
+		ctr.appendUint8(0x41);
+		ctr.appendUint8(static_cast<uint8_t>(envADPCMIdcs.size()));
+		for (auto& idx : envADPCMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instMan.lock()->getEnvelopeADPCMSequence(idx);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& com : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(com.type));
+				ctr.appendInt32(static_cast<int32_t>(com.data));
+			}
+			auto loop = instMan.lock()->getEnvelopeADPCMLoops(idx);
+			ctr.appendUint16(static_cast<uint16_t>(loop.size()));
+			for (auto& l : loop) {
+				ctr.appendUint16(static_cast<uint16_t>(l.begin));
+				ctr.appendUint16(static_cast<uint16_t>(l.end));
+				ctr.appendUint8(static_cast<uint8_t>(l.times));
+			}
+			auto release = instMan.lock()->getEnvelopeADPCMRelease(idx);
+
+			switch (release.type) {
+			case ReleaseType::NoRelease:
+				ctr.appendUint8(0x00);
+				// If release.type is NO_RELEASE, then release.begin == -1 so omit to save it.
+				break;
+			case ReleaseType::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			}
+			ctr.appendUint8(0);	// Skip sequence type
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
+	}
+
+	// ADPCM arpeggio
+	std::vector<int> arpADPCMIdcs;
+	for (auto& idx : instMan.lock()->getArpeggioADPCMEntriedIndices()) {
+		std::vector<int> users = instMan.lock()->getArpeggioADPCMUsers(idx);
+		std::vector<int> intersection;
+		std::set_intersection(users.begin(), users.end(), instNums.begin(), instNums.end(), std::back_inserter(intersection));
+		if (!intersection.empty()) arpADPCMIdcs.push_back(idx);
+	}
+	if (!arpADPCMIdcs.empty()) {
+		ctr.appendUint8(0x42);
+		ctr.appendUint8(static_cast<uint8_t>(arpADPCMIdcs.size()));
+		for (auto& idx : arpADPCMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instMan.lock()->getArpeggioADPCMSequence(idx);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& com : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(com.type));
+			}
+			auto loop = instMan.lock()->getArpeggioADPCMLoops(idx);
+			ctr.appendUint16(static_cast<uint16_t>(loop.size()));
+			for (auto& l : loop) {
+				ctr.appendUint16(static_cast<uint16_t>(l.begin));
+				ctr.appendUint16(static_cast<uint16_t>(l.end));
+				ctr.appendUint8(static_cast<uint8_t>(l.times));
+			}
+			auto release = instMan.lock()->getArpeggioADPCMRelease(idx);
+			switch (release.type) {
+			case ReleaseType::NoRelease:
+				ctr.appendUint8(0x00);
+				// If release.type is NO_RELEASE, then release.begin == -1 so omit to save it.
+				break;
+			case ReleaseType::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			}
+			switch (instMan.lock()->getArpeggioADPCMType(idx)) {
+			case SequenceType::ABSOLUTE_SEQUENCE:	ctr.appendUint8(0x00);	break;
+			case SequenceType::FIXED_SEQUENCE:		ctr.appendUint8(0x01);	break;
+			case SequenceType::RELATIVE_SEQUENCE:	ctr.appendUint8(0x02);	break;
+			default:												break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
+	}
+
+	// ADPCM pitch
+	std::vector<int> ptADPCMIdcs;
+	for (auto& idx : instMan.lock()->getPitchADPCMEntriedIndices()) {
+		std::vector<int> users = instMan.lock()->getPitchADPCMUsers(idx);
+		std::vector<int> intersection;
+		std::set_intersection(users.begin(), users.end(), instNums.begin(), instNums.end(), std::back_inserter(intersection));
+		if (!intersection.empty()) ptADPCMIdcs.push_back(idx);
+	}
+	if (!ptADPCMIdcs.empty()) {
+		ctr.appendUint8(0x43);
+		ctr.appendUint8(static_cast<uint8_t>(ptADPCMIdcs.size()));
+		for (auto& idx : ptADPCMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instMan.lock()->getPitchADPCMSequence(idx);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& com : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(com.type));
+			}
+			auto loop = instMan.lock()->getPitchADPCMLoops(idx);
+			ctr.appendUint16(static_cast<uint16_t>(loop.size()));
+			for (auto& l : loop) {
+				ctr.appendUint16(static_cast<uint16_t>(l.begin));
+				ctr.appendUint16(static_cast<uint16_t>(l.end));
+				ctr.appendUint8(static_cast<uint8_t>(l.times));
+			}
+			auto release = instMan.lock()->getPitchADPCMRelease(idx);
+			switch (release.type) {
+			case ReleaseType::NoRelease:
+				ctr.appendUint8(0x00);
+				// If release.type is NO_RELEASE, then release.begin == -1 so omit to save it.
+				break;
+			case ReleaseType::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			case ReleaseType::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.begin));
+				break;
+			}
+			switch (instMan.lock()->getPitchADPCMType(idx)) {
+			case SequenceType::ABSOLUTE_SEQUENCE:	ctr.appendUint8(0x00);	break;
+			case SequenceType::RELATIVE_SEQUENCE:	ctr.appendUint8(0x02);	break;
+			default:												break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
+	}
+
 	ctr.writeUint32(instPropOfs, ctr.size() - instPropOfs);
 
 	ctr.writeUint32(eofOfs, ctr.size() - eofOfs);
@@ -660,6 +873,8 @@ AbstractBank* BankIO::loadBank(BinaryContainer& ctr, std::string path)
 	std::string ext = FileIO::getExtension(path);
 	if (ext.compare("wopn") == 0) return BankIO::loadWOPNFile(ctr);
 	if (ext.compare("btb") == 0) return BankIO::loadBTBFile(ctr);
+	if (ext.compare("ppc") == 0) return BankIO::loadPPCFile(ctr);
+	if (ext.compare("pvi") == 0) return BankIO::loadPVIFile(ctr);
 	throw FileInputError(FileIO::FileType::Bank);
 }
 
@@ -732,3 +947,61 @@ AbstractBank* BankIO::loadWOPNFile(BinaryContainer& ctr)
 	wopn.release();
 	return bank;
 }
+
+AbstractBank* BankIO::loadPPCFile(BinaryContainer& ctr)
+{
+	size_t globCsr = 0;
+	if (ctr.readString(globCsr, 30) != "ADPCM DATA for  PMD ver.4.4-  ")
+		throw FileCorruptionError(FileIO::FileType::Bank);
+	globCsr += 30;
+	uint16_t nextAddr = ctr.readUint16(globCsr);
+	if ((nextAddr - 0x26u) * 0x20u + 0x420u != ctr.size())	// File size check
+		throw FileCorruptionError(FileIO::FileType::Bank);
+	globCsr += 2;
+
+	size_t sampOffs = globCsr + 256 * 4;
+	if (ctr.size() < sampOffs) throw FileCorruptionError(FileIO::FileType::Bank);
+
+	std::vector<int> ids;
+	std::vector<std::vector<uint8_t>> samples;
+	BankIO::extractADPCMSamples(ctr, globCsr, sampOffs, 256, ids, samples);
+
+	return new PpcBank(std::move(ids), std::move(samples));
+}
+
+AbstractBank* BankIO::loadPVIFile(BinaryContainer& ctr)
+{
+	size_t globCsr = 0;
+	if (ctr.readString(globCsr, 4) != "PVI2")
+		throw FileCorruptionError(FileIO::FileType::Bank);
+	globCsr += 0x10;
+
+	size_t sampOffs = globCsr + 128 * 4;
+	if (ctr.size() < sampOffs) throw FileCorruptionError(FileIO::FileType::Bank);
+
+	std::vector<int> ids;
+	std::vector<std::vector<uint8_t>> samples;
+	BankIO::extractADPCMSamples(ctr, globCsr, sampOffs, 128, ids, samples);
+
+	return new PviBank(std::move(ids), std::move(samples));
+}
+
+void BankIO::extractADPCMSamples(const BinaryContainer& ctr, size_t addrPos, size_t sampOffs, int maxCnt,
+								 std::vector<int>& ids, std::vector<std::vector<uint8_t>>& samples)
+{
+	size_t ofs = 0;
+	for (int i = 0; i < maxCnt; ++i) {
+		uint16_t start = ctr.readUint16(addrPos);
+		addrPos += 2;
+		uint16_t stop = ctr.readUint16(addrPos);
+		addrPos += 2;
+
+		if (stop && start <= stop) {
+			if (ids.empty()) ofs = start;
+			ids.push_back(i);
+			size_t st = sampOffs + static_cast<size_t>((start - ofs) << 5);
+			size_t sampSize = std::min(static_cast<size_t>((stop + 1 - start) << 5), ctr.size() - st);
+			samples.push_back(ctr.getSubcontainer(st, sampSize).toVector());
+		}
+	}
+}
