@@ -16,7 +16,12 @@ public:
 	static AbstractBank* loadBank(BinaryContainer& ctr, std::string path);
 	static AbstractBank* loadBTBFile(BinaryContainer& ctr);
 	static AbstractBank* loadWOPNFile(BinaryContainer& ctr);
+	static AbstractBank* loadPPCFile(BinaryContainer& ctr);
+	static AbstractBank* loadPVIFile(BinaryContainer& ctr);
 
 private:
 	BankIO();
+
+	static void extractADPCMSamples(const BinaryContainer& ctr, size_t addrPos, size_t sampOffs, int maxCnt,
+									std::vector<int>& ids, std::vector<std::vector<uint8_t>>& samples);
 };
