@@ -1,6 +1,8 @@
 #pragma once
 
 #include <utility>
+#include <algorithm>
+#include <cmath>
 #include <stdexcept>
 
 #if !defined(DECL_MAYBE_UNUSED) && defined(__GNUC__)
@@ -13,7 +15,8 @@ enum class SoundSource : int
 {
 	FM = 1,
 	SSG = 2,
-	DRUM = 4
+	DRUM = 4,
+	ADPCM = 8
 };
 
 enum class SongType : int
@@ -52,7 +55,7 @@ enum class FMOperatorType : int
 	All, Op1, Op2, Op3, Op4
 };
 
-enum class SSGWaveFormType : int
+enum class SSGWaveformType : int
 {
 	UNSET = -1,
 	SQUARE = 0,
@@ -177,29 +180,42 @@ inline static size_t getFMChannelCount(SongType type)
 }
 
 DECL_MAYBE_UNUSED
-inline static bool isModulatedWaveFormSSG(SSGWaveFormType type)
+inline static bool isModulatedWaveformSSG(SSGWaveformType type)
 {
 	switch (type) {
-	case SSGWaveFormType::SQUARE:
-	case SSGWaveFormType::TRIANGLE:
-	case SSGWaveFormType::SAW:
-	case SSGWaveFormType::INVSAW:
+	case SSGWaveformType::SQUARE:
+	case SSGWaveformType::TRIANGLE:
+	case SSGWaveformType::SAW:
+	case SSGWaveformType::INVSAW:
 		return false;
-	case SSGWaveFormType::SQM_TRIANGLE:
-	case SSGWaveFormType::SQM_SAW:
-	case SSGWaveFormType::SQM_INVSAW:
+	case SSGWaveformType::SQM_TRIANGLE:
+	case SSGWaveformType::SQM_SAW:
+	case SSGWaveformType::SQM_INVSAW:
 		return true;
 	default:
-		throw std::invalid_argument("Invalid SSGWaveFormType");
+		throw std::invalid_argument("Invalid SSGWaveformType");
 	}
 }
 
 DECL_MAYBE_UNUSED
-inline static bool isModulatedWaveFormSSG(int id)
+inline static bool isModulatedWaveformSSG(int id)
 {
 	try {
-		return isModulatedWaveFormSSG(static_cast<SSGWaveFormType>(id));
+		return isModulatedWaveformSSG(static_cast<SSGWaveformType>(id));
 	} catch (...) {
 		throw std::invalid_argument("Invalid id");
 	}
 }
+
+template <typename T>
+DECL_MAYBE_UNUSED
+inline const T& clamp(const T& value, const T& low, const T& high)
+{
+	return std::min(std::max(value, low), high);
+}
+
+DECL_MAYBE_UNUSED
+inline static int calcADPCMDeltaN(unsigned int rate)
+{
+	return static_cast<int>(std::round((rate << 16) / 55500.));
+}
