@@ -15,6 +15,11 @@ class Track
 public:
 	Track(int number, SoundSource source, int channelInSource, int defPattenSize);
 	Track(const Track& other);
+	Track& operator=(const Track& other);
+	Track(Track&& other) noexcept;
+	Track& operator=(Track&& other) noexcept;
+
+	void setAttribute(int number, SoundSource source, int channelInSource);
 	TrackAttribute getAttribute() const;
 	OrderData getOrderData(int order);
 	size_t getOrderSize() const;
@@ -38,6 +43,8 @@ public:
 	void clearUnusedPatterns();
 	void replaceDuplicateInstrumentsInPatterns(std::unordered_map<int, int> map);
 
+	void transpose(int seminotes, std::vector<int> excludeInsts);
+
 private:
 	std::unique_ptr<TrackAttribute> attrib_;
 
