@@ -26,15 +26,15 @@ struct ToneDetail
 	int pitch;
 };
 
-struct ToneNoise
+struct SSGToneNoise
 {
-	bool isTone_, isNoise_;
+	bool isTone, isNoise;
 	int noisePeriod_;
 };
 
-struct WaveForm
+struct SSGWaveform
 {
-	SSGWaveFormType type;
+	SSGWaveformType type;
 	int data;	// Same format with CommandSequenceUnit::data
 };
 
@@ -53,6 +53,9 @@ public:
 	void sendRegisterAddress(int bank, int address);
 	void sendRegisterValue(int value);
 
+	// DRAM
+	size_t getDRAMSize() const;
+
 	// Update register states after tick process
 	void updateRegisterStates();
 
@@ -72,6 +75,10 @@ public:
 	void setMode(SongType mode);
 	SongType getMode() const;
 
+	// Mute
+	void setMuteState(SoundSource src, int chInSrc, bool isMute);
+	bool isMute(SoundSource src, int chInSrc);
+
 	// Stream details
 	int getRate() const;
 	void setRate(int rate);
@@ -88,11 +95,24 @@ private:
 
 	std::vector<RegisterUnit> registerSetBuf_;
 
+	std::unique_ptr<int16_t[]> outputHistory_;
+	size_t outputHistoryIndex_;
+	std::unique_ptr<int16_t[]> outputHistoryReady_;
+	std::mutex outputHistoryReadyMutex_;
+
 	void initChip();
 
 	void fillOutputHistory(const int16_t* outputs, size_t nSamples);
 	void transferReadyHistory();
 
+	void checkRealToneByArpeggio(int seqPos, const std::unique_ptr<SequenceIteratorInterface>& arpIt,
+								 const std::deque<ToneDetail>& baseTone, ToneDetail& keyTone, bool& needToneSet);
+	void checkPortamento(const std::unique_ptr<SequenceIteratorInterface>& arpIt, int prtm, bool hasKeyOnBefore,
+						 bool isTonePrtm, const std::deque<ToneDetail>& baseTone, ToneDetail& keyTone,
+						 bool& needToneSet);
+	void checkRealToneByPitch(int seqPos, const std::unique_ptr<CommandSequence::Iterator>& ptIt,
+							  int& sumPitch, bool& needToneSet);
+
 	/*----- FM -----*/
 public:
 	// Key on-off
@@ -101,7 +121,7 @@ public:
 	void keyOffFM(int ch, bool isJam = false);
 	void updateEchoBufferFM(int ch, int octave, Note note, int pitch);
 
-	// Set Instrument
+	// Set instrument
 	void setInstrumentFM(int ch, std::shared_ptr<InstrumentFM> inst);
 	void updateInstrumentFM(int instNum);
 	void updateInstrumentFMEnvelopeParameter(int envNum, FMEnvelopeParameter param);
@@ -115,10 +135,8 @@ public:
 	void setTemporaryVolumeFM(int ch, int volume);
 	void setMasterVolumeFM(double dB);
 
-	// Set pan
-	void setPanFM(int ch, int value);
-
 	// Set effect
+	void setPanFM(int ch, int value);
 	void setArpeggioEffectFM(int ch, int second, int third);
 	void setPortamentoEffectFM(int ch, int depth, bool isTonePortamento = false);
 	void setVibratoEffectFM(int ch, int period, int depth);
@@ -138,10 +156,6 @@ public:
 	// For state retrieve
 	void haltSequencesFM(int ch);
 
-	// Mute
-	void setMuteFMState(int ch, bool isMuteFM);
-	bool isMuteFM(int ch);
-
 	// Chip details
 	bool isKeyOnFM(int ch) const;
 	bool isTonePortamentoFM(int ch) const;
@@ -151,7 +165,7 @@ public:
 private:
 	std::shared_ptr<InstrumentFM> refInstFM_[6];
 	std::unique_ptr<EnvelopeFM> envFM_[6];
-	bool isKeyOnFM_[9];
+	bool isKeyOnFM_[9], hasKeyOnBeforeFM_[9];
 	uint8_t fmOpEnables_[6];
 	std::deque<ToneDetail> baseToneFM_[9];
 	ToneDetail keyToneFM_[9];
@@ -186,11 +200,12 @@ private:
 
 	void initFM();
 
-	int toInternalFMChannel(int ch) const;
-	uint8_t getFMKeyOnOffChannelMask(int ch) const;
+	void setMuteFMState(int ch, bool isMuteFM);
+	bool isMuteFM(int ch);
+
 	uint32_t getFMChannelOffset(int ch, bool forPitch = false) const;
 	FMOperatorType toChannelOperatorType(int ch) const;
-	std::vector<FMEnvelopeParameter> getFMEnvelopeParametersForOperator(FMOperatorType op) const;
+	const std::unordered_map<FMOperatorType, std::vector<FMEnvelopeParameter>> FM_ENV_PARAMS_OP_;
 
 	void updateFMVolume(int ch);
 
@@ -207,70 +222,72 @@ private:
 
 	void checkOperatorSequenceFM(int ch, int type);
 	void checkVolumeEffectFM(int ch);
-	void checkRealToneFMByArpeggio(int ch, int seqPos);
-	void checkPortamentoFM(int ch);
-	void checkRealToneFMByPitch(int ch, int seqPos);
+
+	inline void checkRealToneFMByArpeggio(int ch, int seqPos)
+	{
+		checkRealToneByArpeggio(seqPos, arpItFM_[ch], baseToneFM_[ch], keyToneFM_[ch], needToneSetFM_[ch]);
+	}
+
+	inline void checkPortamentoFM(int ch)
+	{
+		checkPortamento(arpItFM_[ch], prtmFM_[ch], hasKeyOnBeforeFM_[ch], isTonePrtmFM_[ch], baseToneFM_[ch],
+						keyToneFM_[ch], needToneSetFM_[ch]);
+	}
+
+	inline void checkRealToneFMByPitch(int ch, int seqPos)
+	{
+		checkRealToneByPitch(seqPos, ptItFM_[ch], sumPitchFM_[ch], needToneSetFM_[ch]);
+	}
+
 	void writePitchFM(int ch);
 
 	void setInstrumentFMProperties(int ch);
 
-	static bool isCarrier(int op, int al);
 	static std::vector<int> getOperatorsInLevel(int level, int al);
 
-	inline FMEnvelopeParameter getParameterTL(int op) const
+	const bool CARRIER_JUDGE_[4][8] /* [operator][algorithm] */ = {
+		{ false, false, false, false, false, false, false, true },
+		{ false, false, false, false, true, true, true, true },
+		{ false, false, false, false, false, true, true, true },
+		{ true, true, true, true, true, true, true, true },
+	};
+	const uint8_t FM_KEYOFF_MASK_[6] = { 0, 1, 2, 4, 5, 6 };
+	const std::unordered_map<int, uint8_t> FM3_KEY_OFF_MASK_ = { { 2, 0xe }, { 6, 0xd }, { 7, 0xb }, { 8, 0x7 }	};
+	const FMEnvelopeParameter PARAM_TL_[4] = {
+		FMEnvelopeParameter::TL1, FMEnvelopeParameter::TL2, FMEnvelopeParameter::TL3, FMEnvelopeParameter::TL4
+	};
+	const FMEnvelopeParameter PARAM_ML_[4] = {
+		FMEnvelopeParameter::ML1, FMEnvelopeParameter::ML2, FMEnvelopeParameter::ML3, FMEnvelopeParameter::ML4
+	};
+	const FMEnvelopeParameter PARAM_AR_[4] = {
+		FMEnvelopeParameter::AR1, FMEnvelopeParameter::AR2, FMEnvelopeParameter::AR3, FMEnvelopeParameter::AR4
+	};
+	const FMEnvelopeParameter PARAM_DR_[4] = {
+		FMEnvelopeParameter::DR1, FMEnvelopeParameter::DR2, FMEnvelopeParameter::DR3, FMEnvelopeParameter::DR4
+	};
+	const FMEnvelopeParameter PARAM_RR_[4] = {
+		FMEnvelopeParameter::RR1, FMEnvelopeParameter::RR2, FMEnvelopeParameter::RR3, FMEnvelopeParameter::RR4
+	};
+
+	inline int toInternalFMChannel(int ch) const
 	{
-		switch (op) {
-		case 0:	return FMEnvelopeParameter::TL1;
-		case 1:	return FMEnvelopeParameter::TL2;
-		case 2:	return FMEnvelopeParameter::TL3;
-		case 3:	return FMEnvelopeParameter::TL4;
-		default:	throw std::invalid_argument("Invalid operator value.");
-		}
+		if (0 <= ch && ch < 6) return ch;
+		else if (mode_ == SongType::FM3chExpanded && 6 <= ch && ch < 9) return 2;
+		else throw std::out_of_range("Out of channel range.");
 	}
 
-	inline FMEnvelopeParameter getParameterML(int op) const
+	inline uint8_t getFMKeyOnOffChannelMask(int ch) const
 	{
-		switch (op) {
-		case 0:	return FMEnvelopeParameter::ML1;
-		case 1:	return FMEnvelopeParameter::ML2;
-		case 2:	return FMEnvelopeParameter::ML3;
-		case 3:	return FMEnvelopeParameter::ML4;
-		default:	throw std::invalid_argument("Invalid operator value.");
-		}
+		return FM_KEYOFF_MASK_[toInternalFMChannel(ch)];
 	}
 
-	inline FMEnvelopeParameter getParameterAR(int op) const
+	inline uint8_t getFM3SlotValidStatus() const
 	{
-		switch (op) {
-		case 0:	return FMEnvelopeParameter::AR1;
-		case 1:	return FMEnvelopeParameter::AR2;
-		case 2:	return FMEnvelopeParameter::AR3;
-		case 3:	return FMEnvelopeParameter::AR4;
-		default:	throw std::invalid_argument("Invalid operator value.");
-		}
+		return fmOpEnables_[2] & (static_cast<uint8_t>(isKeyOnFM_[2]) | (static_cast<uint8_t>(isKeyOnFM_[6]) << 1)
+				| (static_cast<uint8_t>(isKeyOnFM_[7]) << 2) | (static_cast<uint8_t>(isKeyOnFM_[8]) << 3));
 	}
 
-	inline FMEnvelopeParameter getParameterDR(int op) const
-	{
-		switch (op) {
-		case 0:	return FMEnvelopeParameter::DR1;
-		case 1:	return FMEnvelopeParameter::DR2;
-		case 2:	return FMEnvelopeParameter::DR3;
-		case 3:	return FMEnvelopeParameter::DR4;
-		default:	throw std::invalid_argument("Invalid operator value.");
-		}
-	}
-
-	inline FMEnvelopeParameter getParameterRR(int op) const
-	{
-		switch (op) {
-		case 0:	return FMEnvelopeParameter::RR1;
-		case 1:	return FMEnvelopeParameter::RR2;
-		case 2:	return FMEnvelopeParameter::RR3;
-		case 3:	return FMEnvelopeParameter::RR4;
-		default:	throw std::invalid_argument("Invalid operator value.");
-		}
-	}
+	inline bool isCarrier(int op, int al) { return CARRIER_JUDGE_[op][al]; }
 
 	inline uint8_t calculateTL(int ch, uint8_t data) const
 	{
@@ -286,7 +303,7 @@ public:
 	void keyOffSSG(int ch, bool isJam = false);
 	void updateEchoBufferSSG(int ch, int octave, Note note, int pitch);
 
-	// Set Instrument
+	// Set instrument
 	void setInstrumentSSG(int ch, std::shared_ptr<InstrumentSSG> inst);
 	void updateInstrumentSSG(int instNum);
 
@@ -309,13 +326,8 @@ public:
 	void setHardEnvelopePeriod(int ch, bool high, int period);
 	void setAutoEnvelopeSSG(int ch, int shift, int shape);
 
-	// For state retrieve
 	void haltSequencesSSG(int ch);
 
-	// Mute
-	void setMuteSSGState(int ch, bool isMuteFM);
-	bool isMuteSSG(int ch);
-
 	// Chip details
 	bool isKeyOnSSG(int ch) const;
 	bool isTonePortamentoSSG(int ch) const;
@@ -323,12 +335,12 @@ public:
 
 private:
 	std::shared_ptr<InstrumentSSG> refInstSSG_[3];
-	bool isKeyOnSSG_[3];
+	bool isKeyOnSSG_[3], hasKeyOnBeforeSSG_[9];
 	uint8_t mixerSSG_;
 	std::deque<ToneDetail> baseToneSSG_[3];
 	ToneDetail keyToneSSG_[3];
 	int sumPitchSSG_[3];
-	ToneNoise tnSSG_[3];
+	SSGToneNoise tnSSG_[3];
 	int baseVolSSG_[3], tmpVolSSG_[3];
 	bool isBuzzEffSSG_[3];
 	bool isHardEnvSSG_[3];
@@ -340,7 +352,7 @@ private:
 	bool needToneSetSSG_[3];
 	bool needSqMaskFreqSetSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> wfItSSG_[3];
-	WaveForm wfSSG_[3];
+	SSGWaveform wfSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> envItSSG_[3];
 	CommandSequenceUnit envSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> tnItSSG_[3];
@@ -360,27 +372,44 @@ private:
 	int toneNoiseMixSSG_[3];
 	int noisePitchSSG_;
 	int hardEnvPeriodHighSSG_, hardEnvPeriodLowSSG_;
-	std::unique_ptr<int16_t[]> outputHistory_;
-	size_t outputHistoryIndex_;
-	std::unique_ptr<int16_t[]> outputHistoryReady_;
-	std::mutex outputHistoryReadyMutex_;
+	const int AUTO_ENV_SHAPE_TYPE_[15] = { 17, 17, 17, 21, 21, 21, 21, 16, 17, 18, 19, 20, 21, 22, 23 };
 
 	void initSSG();
 
+	void setMuteSSGState(int ch, bool isMuteFM);
+	bool isMuteSSG(int ch);
+
 	void setFrontSSGSequences(int ch);
 	void releaseStartSSGSequences(int ch);
 	void tickEventSSG(int ch);
 
-	void writeWaveFormSSGToRegister(int ch, int seqPos);
-	void writeSquareWaveForm(int ch);
+	void writeWaveformSSGToRegister(int ch, int seqPos);
+	void writeSquareWaveform(int ch);
 
 	void writeToneNoiseSSGToRegister(int ch, int seqPos);
 	void writeToneNoiseSSGToRegisterNoReference(int ch);
 
 	void writeEnvelopeSSGToRegister(int ch, int seqPos);
-	void checkRealToneSSGByArpeggio(int ch, int seqPos);
-	void checkPortamentoSSG(int ch);
-	void checkRealToneSSGByPitch(int ch, int seqPos);
+
+	inline uint8_t SSGToneFlag(int ch) { return (1 << ch); }
+	inline uint8_t SSGNoiseFlag(int ch) { return (8 << ch); }
+
+	inline void checkRealToneSSGByArpeggio(int ch, int seqPos)
+	{
+		checkRealToneByArpeggio(seqPos, arpItSSG_[ch], baseToneSSG_[ch], keyToneSSG_[ch], needToneSetSSG_[ch]);
+	}
+
+	inline void checkPortamentoSSG(int ch)
+	{
+		checkPortamento(arpItSSG_[ch], prtmSSG_[ch], hasKeyOnBeforeSSG_[ch], isTonePrtmSSG_[ch], baseToneSSG_[ch],
+						keyToneSSG_[ch], needToneSetSSG_[ch]);
+	}
+
+	inline void checkRealToneSSGByPitch(int ch, int seqPos)
+	{
+		checkRealToneByPitch(seqPos, ptItSSG_[ch], sumPitchSSG_[ch], needToneSetSSG_[ch]);
+	}
+
 	void writePitchSSG(int ch);
 	void writeAutoEnvelopePitchSSG(int ch, double tonePitch);
 	void writeSquareMaskPitchSSG(int ch, double tonePitch, bool isTriangle);
@@ -403,13 +432,9 @@ public:
 	void setMasterVolumeDrum(int volume);
 	void setTemporaryVolumeDrum(int ch, int volume);
 
-	// Set pan
+	// Set effect
 	void setPanDrum(int ch, int value);
 
-	// Mute
-	void setMuteDrumState(int ch, bool isMute);
-	bool isMuteDrum(int ch);
-
 private:
 	uint8_t keyOnFlagDrum_, keyOffFlagDrum_;
 	int volDrum_[6], mVolDrum_, tmpVolDrum_[6];
@@ -420,5 +445,116 @@ private:
 
 	void initDrum();
 
+	void setMuteDrumState(int ch, bool isMute);
+	bool isMuteDrum(int ch);
+
 	void updateKeyOnOffStatusDrum();
+
+	/*----- ADPCM/Drumkit -----*/
+public:
+	// Key on-off
+	void keyOnADPCM(Note note, int octave, int pitch, bool isJam = false);
+	void keyOnADPCM(int echoBuf);
+	void keyOffADPCM(bool isJam = false);
+	void updateEchoBufferADPCM(int octave, Note note, int pitch);
+
+	// Set instrument
+	void setInstrumentADPCM(std::shared_ptr<InstrumentADPCM> inst);
+	void updateInstrumentADPCM(int instNum);
+	void setInstrumentDrumkit(std::shared_ptr<InstrumentDrumkit> inst);
+	void updateInstrumentDrumkit(int instNum, int key);
+	void clearSamplesADPCM();
+	/// return: [0]: start address, [1]: stop address
+	std::vector<size_t> storeSampleADPCM(std::vector<uint8_t> sample);
+
+	// Set volume
+	void setVolumeADPCM(int volume);
+	void setTemporaryVolumeADPCM(int volume);
+
+	// Set effect
+	void setPanADPCM(int value);
+	void setArpeggioEffectADPCM(int second, int third);
+	void setPortamentoEffectADPCM(int depth, bool isTonePortamento = false);
+	void setVibratoEffectADPCM(int period, int depth);
+	void setTremoloEffectADPCM(int period, int depth);
+	void setVolumeSlideADPCM(int depth, bool isUp);
+	void setDetuneADPCM(int pitch);
+	void setNoteSlideADPCM(int speed, int seminote);
+	void setTransposeEffectADPCM(int seminote);
+
+	// For state retrieve
+	void haltSequencesADPCM();
+
+	// Chip details
+	bool isKeyOnADPCM() const;
+	bool isTonePortamentoADPCM() const;
+	ToneDetail getADPCMTone() const;
+	size_t getADPCMStoredSize() const;
+
+private:
+	std::shared_ptr<InstrumentADPCM> refInstADPCM_;
+	std::shared_ptr<InstrumentDrumkit> refInstKit_;
+	bool isKeyOnADPCM_, hasKeyOnBeforeADPCM_;
+	std::deque<ToneDetail> baseToneADPCM_;
+	ToneDetail keyToneADPCM_;
+	int sumPitchADPCM_;
+	int baseVolADPCM_, tmpVolADPCM_;
+	uint8_t panADPCM_;
+	bool isMuteADPCM_;
+	bool hasPreSetTickEventADPCM_;
+	bool needEnvSetADPCM_;
+	bool needToneSetADPCM_;
+	size_t dramSize_;
+	size_t startAddrADPCM_, stopAddrADPCM_;	// By 32 bytes
+	size_t storePointADPCM_;	// Move by 32 bytes
+	std::unique_ptr<CommandSequence::Iterator> envItADPCM_;
+	std::unique_ptr<CommandSequence::Iterator> tnItADPCM_;
+	std::unique_ptr<SequenceIteratorInterface> arpItADPCM_;
+	std::unique_ptr<CommandSequence::Iterator> ptItADPCM_;
+	bool isArpEffADPCM_;
+	int prtmADPCM_;
+	bool isTonePrtmADPCM_;
+	std::unique_ptr<WavingEffectIterator> vibItADPCM_;
+	std::unique_ptr<WavingEffectIterator> treItADPCM_;
+	int volSldADPCM_, sumVolSldADPCM_;
+	int detuneADPCM_;
+	std::unique_ptr<NoteSlideEffectIterator> nsItADPCM_;
+	int sumNoteSldADPCM_;
+	bool noteSldADPCMSetFlag_;
+	int transposeADPCM_;
+	bool hasStartRequestedKit_;
+
+	void initADPCM();
+
+	void setMuteADPCMState(bool isMuteFM);
+	bool isMuteADPCM();
+
+	void setFrontADPCMSequences();
+	void releaseStartADPCMSequences();
+	void tickEventADPCM();
+
+	void writeEnvelopeADPCMToRegister(int seqPos);
+
+	inline void checkRealToneADPCMByArpeggio(int seqPos)
+	{
+		checkRealToneByArpeggio(seqPos, arpItADPCM_, baseToneADPCM_, keyToneADPCM_, needToneSetADPCM_);
+	}
+
+	inline void checkPortamentoADPCM()
+	{
+		checkPortamento(arpItADPCM_, prtmADPCM_, hasKeyOnBeforeADPCM_, isTonePrtmADPCM_, baseToneADPCM_,
+						keyToneADPCM_, needToneSetADPCM_);
+	}
+
+	inline void checkRealToneADPCMByPitch(int seqPos)
+	{
+		checkRealToneByPitch(seqPos, ptItADPCM_, sumPitchADPCM_, needToneSetADPCM_);
+	}
+
+	void writePitchADPCM();
+	void writePitchADPCMToRegister(int pitchDiff, int rtDeltaN);
+
+	void setRealVolumeADPCM();
+
+	void triggerSamplePlayADPCM(size_t startAddress, size_t stopAddress, bool repeatable);
 };
