@@ -15,7 +15,6 @@ PlaybackManager::PlaybackManager(std::shared_ptr<OPNAController> opnaCtrl,
 	  playOrderNum_(-1),
 	  playStepNum_(-1),
 	  playState_(0),
-	  managerState_(PlaybackState::Stop),
 	  isFindNextStep_(false),
 	  isRetrieveChannel_(isRetrieveChannel)
 {
@@ -35,9 +34,9 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	/* opna mode is changed in BambooTracker class */
 
 	size_t fmch = getFMChannelCount(songStyle_.type);
-	isNoteDelayFM_ = std::vector<bool>(fmch);
-	keyOnBasedEffsFM_ = std::vector<std::unordered_map<EffectType, int>>(fmch);
-	stepBeginBasedEffsFM_ = std::vector<std::unordered_map<EffectType, int>>(fmch);
+	isNoteDelay_[SoundSource::FM] = std::vector<bool>(fmch);
+	keyOnBasedEffs_[SoundSource::FM] = EffectMemorySource(fmch);
+	stepBeginBasedEffs_[SoundSource::FM] = EffectMemorySource(fmch);
 	ntDlyCntFM_ = std::vector<int>(fmch);
 	ntCutDlyCntFM_ = std::vector<int>(fmch);
 	volDlyCntFM_ = std::vector<int>(fmch);
@@ -45,9 +44,9 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	tposeDlyCntFM_ = std::vector<int>(fmch);
 	tposeDlyValueFM_ = std::vector<int>(fmch);
 
-	isNoteDelaySSG_ = std::vector<bool>(3);
-	keyOnBasedEffsSSG_ = std::vector<std::unordered_map<EffectType, int>>(3);
-	stepBeginBasedEffsSSG_ = std::vector<std::unordered_map<EffectType, int>>(3);
+	isNoteDelay_[SoundSource::SSG] = std::vector<bool>(3);
+	keyOnBasedEffs_[SoundSource::SSG] = EffectMemorySource(3);
+	stepBeginBasedEffs_[SoundSource::SSG] = EffectMemorySource(3);
 	ntDlyCntSSG_ = std::vector<int>(3);
 	ntCutDlyCntSSG_ = std::vector<int>(3);
 	volDlyCntSSG_ = std::vector<int>(3);
@@ -55,13 +54,21 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	tposeDlyCntSSG_ = std::vector<int>(3);
 	tposeDlyValueSSG_ = std::vector<int>(3);
 
-	isNoteDelayDrum_ = std::vector<bool>(6);
-	keyOnBasedEffsDrum_ = std::vector<std::unordered_map<EffectType, int>>(6);
-	stepBeginBasedEffsDrum_ = std::vector<std::unordered_map<EffectType, int>>(6);
+	isNoteDelay_[SoundSource::DRUM] = std::vector<bool>(6);
+	keyOnBasedEffs_[SoundSource::DRUM] = EffectMemorySource(6);
+	stepBeginBasedEffs_[SoundSource::DRUM] = EffectMemorySource(6);
 	ntDlyCntDrum_ = std::vector<int>(6);
 	ntCutDlyCntDrum_ = std::vector<int>(6);
 	volDlyCntDrum_ = std::vector<int>(6);
 	volDlyValueDrum_ = std::vector<int>(6, -1);
+
+	isNoteDelay_[SoundSource::ADPCM] = std::vector<bool>(1);
+	keyOnBasedEffs_[SoundSource::ADPCM] = EffectMemorySource(1);
+	stepBeginBasedEffs_[SoundSource::ADPCM] = EffectMemorySource(1);
+	ntDlyCntADPCM_ = 0;
+	ntCutDlyCntADPCM_ = 0;
+	volDlyCntADPCM_ = 0;
+	volDlyValueADPCM_ = -1;
 }
 
 /********** Play song **********/
@@ -71,7 +78,6 @@ void PlaybackManager::startPlaySong(int order)
 
 	startPlay();
 	playState_ = 0x01;
-	managerState_ = PlaybackState::PlaySong;
 	playStepNum_ = 0;
 	playOrderNum_ = order;
 	findNextStep();
@@ -84,7 +90,6 @@ void PlaybackManager::startPlayFromStart()
 
 	startPlay();
 	playState_ = 0x01;
-	managerState_ = PlaybackState::PlayFromStart;
 	playOrderNum_ = 0;
 	playStepNum_ = 0;
 	findNextStep();
@@ -96,26 +101,53 @@ void PlaybackManager::startPlayPattern(int order)
 
 	startPlay();
 	playState_ = 0x11;
-	managerState_ = PlaybackState::PlayPattern;
 	playStepNum_ = 0;
 	playOrderNum_ = order;
 	findNextStep();
 	if (isRetrieveChannel_) retrieveChannelStates();
 }
 
-void PlaybackManager::startPlayFromCurrentStep(int order, int step)
+void PlaybackManager::startPlayFromPosition(int order, int step)
 {
 	std::lock_guard<std::mutex> lock(mutex_);
 
 	startPlay();
 	playState_ = 0x01;
-	managerState_ = PlaybackState::PlayFromCurrentStep;
 	playOrderNum_ = order;
 	playStepNum_ = step;
 	findNextStep();
 	if (isRetrieveChannel_) retrieveChannelStates();
 }
 
+void PlaybackManager::playStep(int order, int step)
+{
+	std::lock_guard<std::mutex> lock(mutex_);
+
+	bool isPlaying = isPlayingStep();
+	if (!isPlaying) {
+		opnaCtrl_->reset();
+
+		Song& song = mod_.lock()->getSong(curSongNum_);
+		tickCounter_.lock()->setTempo(song.getTempo());
+		tickCounter_.lock()->setSpeed(song.getSpeed());
+		tickCounter_.lock()->setGroove(mod_.lock()->getGroove(song.getGroove()).getSequence());
+		tickCounter_.lock()->setGrooveTrigger(song.isUsedTempo() ? GrooveTrigger::Invalid
+																 : GrooveTrigger::ValidByGlobal);
+	}
+	tickCounter_.lock()->resetCount();
+	tickCounter_.lock()->setPlayState(true);
+
+	clearEffectMaps();
+	clearNoteDelayCounts();
+	clearDelayBeyondStepCounts();
+
+	playState_ = 0x20;
+	playOrderNum_ = order;
+	playStepNum_ = step;
+
+	if (!isPlaying && isRetrieveChannel_) retrieveChannelStates();
+}
+
 void PlaybackManager::startPlay()
 {
 	opnaCtrl_->reset();
@@ -147,7 +179,6 @@ void PlaybackManager::stopPlay()
 
 	tickCounter_.lock()->setPlayState(false);
 	playState_ = 0;
-	managerState_ = PlaybackState::Stop;
 	playOrderNum_ = -1;
 	playStepNum_ = -1;
 }
@@ -157,9 +188,9 @@ bool PlaybackManager::isPlaySong() const
 	return ((playState_ & 0x01) > 0);
 }
 
-PlaybackState PlaybackManager::getPlaybackState() const
+bool PlaybackManager::isPlayingStep() const
 {
-	return managerState_;
+	return ((playState_ & 0x20) > 0);
 }
 
 int PlaybackManager::getPlayingOrderNumber() const
@@ -189,7 +220,7 @@ int PlaybackManager::streamCountUp()
 			stepProcess();
 			if (!isFindNextStep_) findNextStep();
 		}
-		else {
+		else if (!isPlayingStep()) {
 			stopPlay();
 		}
 	}
@@ -205,13 +236,18 @@ int PlaybackManager::streamCountUp()
 bool PlaybackManager::stepDown()
 {
 	if (playState_ & 0x02) {	// Foward current step
-		if (nextReadOrder_ == -1) {
-			isFindNextStep_ = false;
+		if (isPlayingStep()) {
 			return false;
 		}
 		else {
-			playOrderNum_ = nextReadOrder_;
-			playStepNum_ = nextReadStep_;
+			if (nextReadOrder_ == -1) {
+				isFindNextStep_ = false;
+				return false;
+			}
+			else {
+				playOrderNum_ = nextReadOrder_;
+				playStepNum_ = nextReadStep_;
+			}
 		}
 	}
 	else {	// First read
@@ -286,41 +322,20 @@ void PlaybackManager::stepProcess()
 		auto& step = song.getTrack(attrib.number)
 					 .getPatternFromOrderNumber(playOrderNum_).getStep(playStepNum_);
 		size_t uch = static_cast<size_t>(attrib.channelInSource);
+		keyOnBasedEffs_[attrib.source].at(uch).clear();
+		bool (PlaybackManager::*storeEffectToMap)(int, Effect);
 		switch (attrib.source) {
-		case SoundSource::FM:
-		{
-			keyOnBasedEffsFM_.at(uch).clear();
-			bool isDelay = false;
-			for (int i = 0; i < 4; ++i) {
-				Effect&& eff = Effect::makeEffectData(SoundSource::FM, step.getEffectID(i), step.getEffectValue(i));
-				isDelay |= storeEffectToMapFM(attrib.channelInSource, std::move(eff));
-			}
-			isNoteDelayFM_.at(uch) = isDelay;
-			break;
-		}
-		case SoundSource::SSG:
-		{
-			keyOnBasedEffsSSG_.at(uch).clear();
-			bool isDelay = false;
-			for (int i = 0; i < 4; ++i) {
-				Effect&& eff = Effect::makeEffectData(SoundSource::SSG, step.getEffectID(i), step.getEffectValue(i));
-				isDelay |= storeEffectToMapSSG(attrib.channelInSource, std::move(eff));
-			}
-			isNoteDelaySSG_.at(uch) = isDelay;
-			break;
-		}
-		case SoundSource::DRUM:
-		{
-			keyOnBasedEffsDrum_.at(uch).clear();
-			bool isDelay = false;
-			for (int i = 0; i < 4; ++i) {
-				Effect&& eff = Effect::makeEffectData(SoundSource::DRUM, step.getEffectID(i), step.getEffectValue(i));
-				isDelay |= storeEffectToMapDrum(attrib.channelInSource, std::move(eff));
-			}
-			isNoteDelayDrum_.at(uch) = isDelay;
-			break;
+		case SoundSource::FM:		storeEffectToMap = &PlaybackManager::storeEffectToMapFM;	break;
+		case SoundSource::SSG:		storeEffectToMap = &PlaybackManager::storeEffectToMapSSG;	break;
+		case SoundSource::DRUM:		storeEffectToMap = &PlaybackManager::storeEffectToMapDrum;	break;
+		case SoundSource::ADPCM:	storeEffectToMap = &PlaybackManager::storeEffectToMapADPCM;	break;
 		}
+		bool isDelay = false;
+		for (int i = 0; i < 4; ++i) {
+			Effect&& eff = Effect::makeEffectData(attrib.source, step.getEffectID(i), step.getEffectValue(i));
+			isDelay |= (this->*storeEffectToMap)(attrib.channelInSource, std::move(eff));
 		}
+		isNoteDelay_[attrib.source].at(uch) = isDelay;
 	}
 
 	// Execute step events
@@ -330,7 +345,7 @@ void PlaybackManager::stepProcess()
 					 .getPatternFromOrderNumber(playOrderNum_).getStep(playStepNum_);
 		switch (attrib.source) {
 		case SoundSource::FM:
-			if (isNoteDelayFM_.at(attrib.channelInSource)) {
+			if (isNoteDelay_[SoundSource::FM].at(attrib.channelInSource)) {
 				// Set effect
 				executeStoredEffectsFM(attrib.channelInSource);
 				checkFMNoteDelayAndEnvelopeReset(step, attrib.channelInSource);
@@ -341,7 +356,7 @@ void PlaybackManager::stepProcess()
 			}
 			break;
 		case SoundSource::SSG:
-			if (isNoteDelaySSG_.at(attrib.channelInSource)) {
+			if (isNoteDelay_[SoundSource::SSG].at(attrib.channelInSource)) {
 				// Set effect
 				executeStoredEffectsSSG(attrib.channelInSource);
 				opnaCtrl_->tickEvent(SoundSource::SSG, attrib.channelInSource);
@@ -351,7 +366,7 @@ void PlaybackManager::stepProcess()
 			}
 			break;
 		case SoundSource::DRUM:
-			if (isNoteDelayDrum_.at(attrib.channelInSource)) {
+			if (isNoteDelay_[SoundSource::DRUM].at(attrib.channelInSource)) {
 				// Set effect
 				executeStoredEffectsDrum(attrib.channelInSource);
 				opnaCtrl_->tickEvent(SoundSource::DRUM, attrib.channelInSource);
@@ -360,6 +375,16 @@ void PlaybackManager::stepProcess()
 				executeDrumStepEvents(step, attrib.channelInSource);
 			}
 			break;
+		case SoundSource::ADPCM:
+			if (isNoteDelay_[SoundSource::ADPCM].at(attrib.channelInSource)) {
+				// Set effect
+				executeStoredEffectsADPCM();
+				opnaCtrl_->tickEvent(SoundSource::ADPCM, attrib.channelInSource);
+			}
+			else {
+				executeADPCMStepEvents(step);
+			}
+			break;
 		}
 	}
 	opnaCtrl_->updateRegisterStates();	// Update for other changes
@@ -507,6 +532,61 @@ void PlaybackManager::executeDrumStepEvents(Step& step, int ch, bool calledByNot
 	}
 }
 
+void PlaybackManager::executeADPCMStepEvents(Step& step, bool calledByNoteDelay)
+{
+	int noteNum = step.getNoteNumber();
+	if (!calledByNoteDelay && noteNum != -1) clearADPCMDelayBeyondStepCounts();	// Except no key event
+
+	// Set volume
+	int vol = step.getVolume();
+	if (0 <= vol && vol < 0x100) {
+		opnaCtrl_->setVolumeADPCM(vol);
+	}
+
+	// Set instrument
+	if (step.getInstrumentNumber() != -1) {
+		auto inst = instMan_.lock()->getInstrumentSharedPtr(step.getInstrumentNumber());
+		if (inst->getType() == InstrumentType::ADPCM)
+			opnaCtrl_->setInstrumentADPCM(std::dynamic_pointer_cast<InstrumentADPCM>(inst));
+		else if (inst->getType() == InstrumentType::Drumkit)
+			opnaCtrl_->setInstrumentDrumkit(std::dynamic_pointer_cast<InstrumentDrumkit>(inst));
+	}
+
+	// Set effect
+	executeStoredEffectsADPCM();
+
+	// Set key
+	switch (noteNum) {
+	case -1:	// None
+		if (!calledByNoteDelay) {	// When this is called by note delay, tick event will be updated in readTick
+			checkADPCMDelayEventsInTick(step);
+			opnaCtrl_->tickEvent(SoundSource::ADPCM, 0);
+		}
+		break;
+	case -2:	// Key off
+		opnaCtrl_->keyOffADPCM();
+		break;
+	case -3:	// Echo 0
+		opnaCtrl_->keyOnADPCM(0);
+		break;
+	case -4:	// Echo 1
+		opnaCtrl_->keyOnADPCM(1);
+		break;
+	case -5:	// Echo 2
+		opnaCtrl_->keyOnADPCM(2);
+		break;
+	case -6:	// Echo 3
+		opnaCtrl_->keyOnADPCM(3);
+		break;
+	default:	// Key on
+	{
+		std::pair<int, Note> octNote = noteNumberToOctaveAndNote(step.getNoteNumber());
+		opnaCtrl_->keyOnADPCM(octNote.second, octNote.first, 0);
+		break;
+	}
+	}
+}
+
 bool PlaybackManager::executeStoredEffectsGlobal()
 {
 	bool changedNextPos = false;
@@ -578,7 +658,7 @@ bool PlaybackManager::storeEffectToMapFM(int ch, Effect eff)
 	case EffectType::RegisterAddress1:
 	case EffectType::RegisterValue:
 	case EffectType::Brightness:
-		keyOnBasedEffsFM_.at(static_cast<size_t>(ch))[eff.type] = eff.value;
+		keyOnBasedEffs_[SoundSource::FM].at(static_cast<size_t>(ch))[eff.type] = eff.value;
 		return false;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
@@ -586,7 +666,7 @@ bool PlaybackManager::storeEffectToMapFM(int ch, Effect eff)
 		return false;
 	case EffectType::NoteDelay:
 		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			stepBeginBasedEffsFM_.at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
+			stepBeginBasedEffs_[SoundSource::FM].at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
 			return true;
 		}
 		return false;
@@ -606,7 +686,8 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 	bool isNoteDelay = false;
 
 	// Read step beginning based effects
-	for (const auto& eff : stepBeginBasedEffsFM_.at(uch)) {
+	auto& stepBeginBasedEffs = stepBeginBasedEffs_[SoundSource::FM].at(uch);
+	for (const auto& eff : stepBeginBasedEffs) {
 		switch (eff.first) {
 		case EffectType::NoteDelay:
 			ntDlyCntFM_.at(uch) = eff.second;
@@ -616,11 +697,12 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 			break;
 		}
 	}
-	stepBeginBasedEffsFM_.at(uch).clear();
+	stepBeginBasedEffs.clear();
 
 	// Read note on and step beginning based effects
 	if (!isNoteDelay) {
-		for (const auto& eff : keyOnBasedEffsFM_.at(uch)) {
+		auto& keyOnBasedEffs = keyOnBasedEffs_[SoundSource::FM].at(uch);
+		for (auto& eff : keyOnBasedEffs) {
 			switch (eff.first) {
 			case EffectType::Arpeggio:
 				opnaCtrl_->setArpeggioEffectFM(ch, eff.second >> 4, eff.second & 0x0f);
@@ -738,7 +820,7 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 				break;
 			}
 		}
-		keyOnBasedEffsFM_.at(uch).clear();
+		keyOnBasedEffs.clear();
 	}
 }
 
@@ -766,7 +848,7 @@ bool PlaybackManager::storeEffectToMapSSG(int ch, Effect eff)
 	case EffectType::RegisterAddress0:
 	case EffectType::RegisterAddress1:
 	case EffectType::RegisterValue:
-		keyOnBasedEffsSSG_.at(static_cast<size_t>(ch))[eff.type] = eff.value;
+		keyOnBasedEffs_[SoundSource::SSG].at(static_cast<size_t>(ch))[eff.type] = eff.value;
 		return false;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
@@ -774,7 +856,7 @@ bool PlaybackManager::storeEffectToMapSSG(int ch, Effect eff)
 		return false;
 	case EffectType::NoteDelay:
 		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			stepBeginBasedEffsSSG_.at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
+			stepBeginBasedEffs_[SoundSource::SSG].at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
 			return true;
 		}
 		return false;
@@ -794,7 +876,8 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 	bool isNoteDelay = false;
 
 	// Read step beginning based effects
-	for (const auto& eff : stepBeginBasedEffsSSG_.at(uch)) {
+	auto& stepBeginBasedEffs = stepBeginBasedEffs_[SoundSource::SSG].at(uch);
+	for (const auto& eff : stepBeginBasedEffs) {
 		switch (eff.first) {
 		case EffectType::NoteDelay:
 			ntDlyCntSSG_.at(uch) = eff.second;
@@ -804,11 +887,12 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 			break;
 		}
 	}
-	stepBeginBasedEffsSSG_.at(uch).clear();
+	stepBeginBasedEffs.clear();
 
 	// Read note on and step beginning based effects
 	if (!isNoteDelay) {
-		for (const auto& eff : keyOnBasedEffsSSG_.at(uch)) {
+		auto& keyOnBasedEffs = keyOnBasedEffs_[SoundSource::SSG].at(uch);
+		for (const auto& eff : keyOnBasedEffs) {
 			switch (eff.first) {
 			case EffectType::Arpeggio:
 				opnaCtrl_->setArpeggioEffectSSG(ch, eff.second >> 4, eff.second & 0x0f);
@@ -895,7 +979,7 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 				break;
 			}
 		}
-		keyOnBasedEffsSSG_.at(uch).clear();
+		keyOnBasedEffs.clear();
 	}
 }
 
@@ -909,7 +993,7 @@ bool PlaybackManager::storeEffectToMapDrum(int ch, Effect eff)
 	case EffectType::RegisterAddress0:
 	case EffectType::RegisterAddress1:
 	case EffectType::RegisterValue:
-		keyOnBasedEffsDrum_.at(static_cast<size_t>(ch))[eff.type] = eff.value;
+		keyOnBasedEffs_[SoundSource::DRUM].at(static_cast<size_t>(ch))[eff.type] = eff.value;
 		return false;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
@@ -917,7 +1001,7 @@ bool PlaybackManager::storeEffectToMapDrum(int ch, Effect eff)
 		return false;
 	case EffectType::NoteDelay:
 		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			stepBeginBasedEffsDrum_.at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
+			stepBeginBasedEffs_[SoundSource::DRUM].at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
 			return true;
 		}
 		return false;
@@ -937,7 +1021,8 @@ void PlaybackManager::executeStoredEffectsDrum(int ch)
 	bool isNoteDelay = false;
 
 	// Read step beginning based effects
-	for (const auto& eff : stepBeginBasedEffsDrum_.at(uch)) {
+	auto& stepBeginBasedEffs = stepBeginBasedEffs_[SoundSource::DRUM].at(uch);
+	for (const auto& eff : stepBeginBasedEffs) {
 		switch (eff.first) {
 		case EffectType::NoteDelay:
 			ntDlyCntDrum_.at(uch) = eff.second;
@@ -947,11 +1032,12 @@ void PlaybackManager::executeStoredEffectsDrum(int ch)
 			break;
 		}
 	}
-	stepBeginBasedEffsDrum_.at(uch).clear();
+	stepBeginBasedEffs.clear();
 
-	// Read key on  and step beginning based effects
+	// Read key on and step beginning based effects
 	if (!isNoteDelay) {
-		for (const auto& eff : keyOnBasedEffsDrum_.at(uch)) {
+		auto& keyOnBasedEffs = keyOnBasedEffs_[SoundSource::DRUM].at(uch);
+		for (const auto& eff : keyOnBasedEffs) {
 			switch (eff.first) {
 			case EffectType::Pan:
 				if (-1 < eff.second && eff.second < 4) opnaCtrl_->setPanDrum(ch, eff.second);
@@ -990,7 +1076,149 @@ void PlaybackManager::executeStoredEffectsDrum(int ch)
 				break;
 			}
 		}
-		keyOnBasedEffsDrum_.at(uch).clear();
+		keyOnBasedEffs.clear();
+	}
+}
+
+bool PlaybackManager::storeEffectToMapADPCM(int ch, Effect eff)
+{
+	switch (eff.type) {
+	case EffectType::Arpeggio:
+	case EffectType::PortamentoUp:
+	case EffectType::PortamentoDown:
+	case EffectType::TonePortamento:
+	case EffectType::Vibrato:
+	case EffectType::Tremolo:
+	case EffectType::Pan:
+	case EffectType::VolumeSlide:
+	case EffectType::Detune:
+	case EffectType::NoteSlideUp:
+	case EffectType::NoteSlideDown:
+	case EffectType::NoteCut:
+	case EffectType::TransposeDelay:
+	case EffectType::VolumeDelay:
+	case EffectType::RegisterAddress0:
+	case EffectType::RegisterAddress1:
+	case EffectType::RegisterValue:
+		keyOnBasedEffs_[SoundSource::ADPCM].at(static_cast<size_t>(ch))[eff.type] = eff.value;
+		return false;
+	case EffectType::SpeedTempoChange:
+	case EffectType::Groove:
+		stepBeginBasedEffsGlobal_[eff.type] = eff.value;
+		return false;
+	case EffectType::NoteDelay:
+		if (eff.value < tickCounter_.lock()->getSpeed()) {
+			stepBeginBasedEffs_[SoundSource::ADPCM].at(static_cast<size_t>(ch))[EffectType::NoteDelay] = eff.value;
+			return true;
+		}
+		return false;
+	case EffectType::PositionJump:
+	case EffectType::SongEnd:
+	case EffectType::PatternBreak:
+		stepEndBasedEffsGlobal_[eff.type] = eff.value;
+		return false;
+	default:
+		return false;
+	}
+}
+
+void PlaybackManager::executeStoredEffectsADPCM()
+{
+	bool isNoteDelay = false;
+
+	// Read step beginning based effects
+	auto& stepBeginBasedEffs = stepBeginBasedEffs_[SoundSource::ADPCM].front();
+	for (const auto& eff : stepBeginBasedEffs) {
+		switch (eff.first) {
+		case EffectType::NoteDelay:
+			ntDlyCntADPCM_ = eff.second;
+			isNoteDelay = true;
+			break;
+		default:
+			break;
+		}
+	}
+	stepBeginBasedEffs.clear();
+
+	// Read note on and step beginning based effects
+	if (!isNoteDelay) {
+		auto& keyOnBasedEffs = keyOnBasedEffs_[SoundSource::ADPCM].front();
+		for (const auto& eff : keyOnBasedEffs) {
+			switch (eff.first) {
+			case EffectType::Arpeggio:
+				opnaCtrl_->setArpeggioEffectADPCM(eff.second >> 4, eff.second & 0x0f);
+				break;
+			case EffectType::PortamentoUp:
+				opnaCtrl_->setPortamentoEffectADPCM(eff.second);
+				break;
+			case EffectType::PortamentoDown:
+				opnaCtrl_->setPortamentoEffectADPCM(-eff.second);
+				break;
+			case EffectType::TonePortamento:
+				opnaCtrl_->setPortamentoEffectADPCM(eff.second, true);
+				break;
+			case EffectType::Vibrato:
+				opnaCtrl_->setVibratoEffectADPCM(eff.second >> 4, eff.second & 0x0f);
+				break;
+			case EffectType::Tremolo:
+				opnaCtrl_->setTremoloEffectADPCM(eff.second >> 4, eff.second & 0x0f);
+				break;
+			case EffectType::Pan:
+				if (-1 < eff.second && eff.second < 4) opnaCtrl_->setPanADPCM(eff.second);
+				break;
+			case EffectType::VolumeSlide:
+			{
+				int hi = eff.second >> 4;
+				int low = eff.second & 0x0f;
+				if (hi && !low) opnaCtrl_->setVolumeSlideADPCM(hi, true);	// Slide up
+				else if (!hi) opnaCtrl_->setVolumeSlideADPCM(low, false);	// Slide down
+				break;
+			}
+			case EffectType::Detune:
+				opnaCtrl_->setDetuneADPCM(eff.second - 0x80);
+				break;
+			case EffectType::NoteSlideUp:
+				opnaCtrl_->setNoteSlideADPCM(eff.second >> 4, eff.second & 0x0f);
+				break;
+			case EffectType::NoteSlideDown:
+				opnaCtrl_->setNoteSlideADPCM(eff.second >> 4, -(eff.second & 0x0f));
+				break;
+			case EffectType::NoteCut:
+				ntCutDlyCntADPCM_ = eff.second;
+				break;
+			case EffectType::TransposeDelay:
+				tposeDlyCntADPCM_ = (eff.second & 0x70) >> 4;
+				tposeDlyValueADPCM_ = ((eff.second & 0x80) ? -1 : 1) * (eff.second & 0x0f);
+				break;
+			case EffectType::VolumeDelay:
+			{
+				int count = eff.second >> 8;
+				if (count > 0) {
+					volDlyCntADPCM_ = count;
+					volDlyValueADPCM_ = eff.second & 0x00ff;
+				}
+				break;
+			}
+			case EffectType::RegisterAddress0:
+			{
+				if (-1 < eff.second && eff.second < 0x6c) opnaCtrl_->sendRegisterAddress(0, eff.second);
+				break;
+			}
+			case EffectType::RegisterAddress1:
+			{
+				if (-1 < eff.second && eff.second < 0x6c) opnaCtrl_->sendRegisterAddress(1, eff.second);
+				break;
+			}
+			case EffectType::RegisterValue:
+			{
+				if (-1 < eff.second) opnaCtrl_->sendRegisterValue(eff.second);
+				break;
+			}
+			default:
+				break;
+			}
+		}
+		keyOnBasedEffs.clear();
 	}
 }
 
@@ -1056,12 +1284,13 @@ void PlaybackManager::tickProcess(int rest)
 						.getPatternFromOrderNumber(playOrderNum_).getStep(playStepNum_);
 		int ch = attrib.channelInSource;
 		switch (attrib.source) {
-		case SoundSource::FM:	checkFMDelayEventsInTick(curStep, ch);		break;
-		case SoundSource::SSG:	checkSSGDelayEventsInTick(curStep, ch);		break;
-		case SoundSource::DRUM:	checkDrumDelayEventsInTick(curStep, ch);	break;
+		case SoundSource::FM:		checkFMDelayEventsInTick(curStep, ch);		break;
+		case SoundSource::SSG:		checkSSGDelayEventsInTick(curStep, ch);		break;
+		case SoundSource::DRUM:		checkDrumDelayEventsInTick(curStep, ch);	break;
+		case SoundSource::ADPCM:	checkADPCMDelayEventsInTick(curStep);		break;
 		}
 
-		if (rest == 1 && nextReadOrder_ != -1 && attrib.source == SoundSource::FM) {
+		if (rest == 1 && nextReadOrder_ != -1 && attrib.source == SoundSource::FM && !isPlayingStep()) {
 			// Channel envelope reset before next key on
 			auto& step = song.getTrack(attrib.number)
 						 .getPatternFromOrderNumber(nextReadOrder_).getStep(nextReadStep_);
@@ -1162,19 +1391,33 @@ void PlaybackManager::checkDrumDelayEventsInTick(Step& step, int ch)
 		executeDrumStepEvents(step, ch, true);
 }
 
+void PlaybackManager::checkADPCMDelayEventsInTick(Step& step)
+{
+	// Check volume delay
+	if (!volDlyCntADPCM_)
+		opnaCtrl_->setTemporaryVolumeADPCM(volDlyValueADPCM_);
+	// Check note cut
+	if (!ntCutDlyCntADPCM_)
+		opnaCtrl_->keyOffADPCM();
+	// Check transpose delay
+	if (!tposeDlyCntADPCM_)
+		opnaCtrl_->setTransposeEffectADPCM(tposeDlyValueADPCM_);
+	// Check note delay
+	if (!ntDlyCntADPCM_)
+		executeADPCMStepEvents(step, true);
+}
+
 void PlaybackManager::clearEffectMaps()
 {
 	stepBeginBasedEffsGlobal_.clear();
 	stepEndBasedEffsGlobal_.clear();
 
-	for (auto& map : keyOnBasedEffsFM_) map.clear();
-	for (auto& map : stepBeginBasedEffsFM_) map.clear();
-
-	for (auto& map : keyOnBasedEffsSSG_) map.clear();
-	for (auto& map : stepBeginBasedEffsSSG_) map.clear();
-
-	for (auto& map : keyOnBasedEffsDrum_) map.clear();
-	for (auto& map : stepBeginBasedEffsDrum_) map.clear();
+	for (auto& maps: keyOnBasedEffs_) {
+		for (EffectMemory& map : maps.second) map.clear();
+	}
+	for (auto& maps: stepBeginBasedEffs_) {
+		for (EffectMemory& map : maps.second) map.clear();
+	}
 }
 
 void PlaybackManager::clearNoteDelayCounts()
@@ -1182,6 +1425,7 @@ void PlaybackManager::clearNoteDelayCounts()
 	std::fill(ntDlyCntFM_.begin(), ntDlyCntFM_.end(), -1);
 	std::fill(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), -1);
 	std::fill(ntDlyCntDrum_.begin(), ntDlyCntDrum_.end(), -1);
+	ntDlyCntADPCM_ = -1;
 }
 
 void PlaybackManager::clearDelayBeyondStepCounts()
@@ -1201,6 +1445,12 @@ void PlaybackManager::clearDelayBeyondStepCounts()
 	std::fill(ntCutDlyCntDrum_.begin(), ntCutDlyCntDrum_.end(), -1);
 	std::fill(volDlyCntDrum_.begin(), volDlyCntDrum_.end(), -1);
 	std::fill(volDlyValueDrum_.begin(), volDlyValueDrum_.end(), -1);
+
+	ntCutDlyCntADPCM_ = -1;
+	volDlyCntADPCM_ = -1;
+	volDlyValueADPCM_ = -1;
+	tposeDlyCntADPCM_ = -1;
+	tposeDlyValueADPCM_ = 0;
 }
 
 void PlaybackManager::clearFMDelayBeyondStepCounts(int ch)
@@ -1231,20 +1481,33 @@ void PlaybackManager::clearDrumDelayBeyondStepCounts(int ch)
 	volDlyValueDrum_.at(uch) = -1;
 }
 
+void PlaybackManager::clearADPCMDelayBeyondStepCounts()
+{
+	ntCutDlyCntADPCM_ = -1;
+	volDlyCntADPCM_ = -1;
+	volDlyValueADPCM_ = -1;
+	tposeDlyCntADPCM_ = -1;
+	tposeDlyValueADPCM_ = 0;
+}
+
 void PlaybackManager::updateDelayEventCounts()
 {
 	auto f = [](int x) { return (x == -1) ? x : --x; };
 	std::transform(ntDlyCntFM_.begin(), ntDlyCntFM_.end(), ntDlyCntFM_.begin(), f);
 	std::transform(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), ntDlyCntSSG_.begin(), f);
 	std::transform(ntDlyCntDrum_.begin(), ntDlyCntDrum_.end(), ntDlyCntDrum_.begin(), f);
+	--ntDlyCntADPCM_;
 	std::transform(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), ntCutDlyCntFM_.begin(), f);
 	std::transform(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), ntCutDlyCntSSG_.begin(), f);
 	std::transform(ntCutDlyCntDrum_.begin(), ntCutDlyCntDrum_.end(), ntCutDlyCntDrum_.begin(), f);
+	--ntCutDlyCntADPCM_;
 	std::transform(volDlyCntFM_.begin(), volDlyCntFM_.end(), volDlyCntFM_.begin(), f);
 	std::transform(volDlyCntSSG_.begin(), volDlyCntSSG_.end(), volDlyCntSSG_.begin(), f);
 	std::transform(volDlyCntDrum_.begin(), volDlyCntDrum_.end(), volDlyCntDrum_.begin(), f);
+	--volDlyCntADPCM_;
 	std::transform(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), tposeDlyCntFM_.begin(), f);
 	std::transform(tposeDlyCntSSG_.begin(), tposeDlyCntSSG_.end(), tposeDlyCntSSG_.begin(), f);
+	--tposeDlyCntADPCM_;
 }
 
 void PlaybackManager::checkPlayPosition(int maxStepSize)
@@ -1264,8 +1527,10 @@ void PlaybackManager::retrieveChannelStates()
 {
 	size_t fmch = getFMChannelCount(songStyle_.type);
 
-	std::vector<int> tonesCntFM(fmch, 0), tonesCntSSG(3, 0);
+	std::vector<int> tonesCntFM(fmch), tonesCntSSG(3);
+	int tonesCntADPCM = 0;
 	std::vector<std::vector<int>> toneFM(fmch), toneSSG(3);
+	std::vector<int> toneADPCM = std::vector<int>(3, -1);
 	for (size_t i = 0; i < fmch; ++i) {
 		toneFM.at(i) = std::vector<int>(3, -1);
 	}
@@ -1282,6 +1547,9 @@ void PlaybackManager::retrieveChannelStates()
 	std::vector<bool> isSetVibSSG(3, false), isSetTreSSG(3, false), isSetVolSldSSG(3, false), isSetDtnSSG(3, false);
 	std::vector<bool> isSetTNMixSSG(3, false);
 	std::vector<bool> isSetVolDrum(6, false), isSetPanDrum(6, false);
+	bool isSetInstADPCM(false), isSetVolADPCM(false), isSetArpADPCM(false), isSetPrtADPCM(false);
+	bool isSetVibADPCM(false), isSetTreADPCM(false), isSetPanADPCM(false), isSetVolSldADPCM(false);
+	bool isSetDtnADPCM = false;
 	bool isSetMVolDrum = false;
 	bool isSetNoisePitchSSG = false;
 	bool isSetHardEnvPeriodHighSSG = false;
@@ -1381,11 +1649,13 @@ void PlaybackManager::retrieveChannelStates()
 						break;
 					case EffectType::SpeedTempoChange:
 						if (!(speedStates & 0x4)) {
-							if (eff.value < 0x20 && !(speedStates & 0x1)) {	// Speed change
-								speedStates |= 0x1;
-								if (isPrevPos) effSpeedChange(eff.value);
+							if (eff.value < 0x20) {				// Speed change
+								if (!(speedStates & 0x1)) {
+									speedStates |= 0x1;
+									if (isPrevPos) effSpeedChange(eff.value);
+								}
 							}
-							else if (!(speedStates & 0x2)) {			// Tempo change
+							else if (!(speedStates & 0x2)) {	// Tempo change
 								speedStates |= 0x2;
 								if (isPrevPos) effTempoChange(eff.value);
 							}
@@ -1568,11 +1838,13 @@ void PlaybackManager::retrieveChannelStates()
 						break;
 					case EffectType::SpeedTempoChange:
 						if (!(speedStates & 0x4)) {
-							if (eff.value < 0x20 && !(speedStates & 0x1)) {	// Speed change
-								speedStates |= 0x1;
-								if (isPrevPos) effSpeedChange(eff.value);
+							if (eff.value < 0x20) {				// Speed change
+								if (!(speedStates & 0x1)) {
+									speedStates |= 0x1;
+									if (isPrevPos) effSpeedChange(eff.value);
+								}
 							}
-							else if (!(speedStates & 0x2)) {			// Tempo change
+							else if (!(speedStates & 0x2)) {	// Tempo change
 								speedStates |= 0x2;
 								if (isPrevPos) effTempoChange(eff.value);
 							}
@@ -1664,11 +1936,13 @@ void PlaybackManager::retrieveChannelStates()
 						break;
 					case EffectType::SpeedTempoChange:
 						if (!(speedStates & 0x4)) {
-							if (eff.value < 0x20 && !(speedStates & 0x1)) {	// Speed change
-								speedStates |= 0x1;
-								if (isPrevPos) effSpeedChange(eff.value);
+							if (eff.value < 0x20) {				// Speed change
+								if (!(speedStates & 0x1)) {
+									speedStates |= 0x1;
+									if (isPrevPos) effSpeedChange(eff.value);
+								}
 							}
-							else if (!(speedStates & 0x2)) {			// Tempo change
+							else if (!(speedStates & 0x2)) {	// Tempo change
 								speedStates |= 0x2;
 								if (isPrevPos) effTempoChange(eff.value);
 							}
@@ -1692,6 +1966,128 @@ void PlaybackManager::retrieveChannelStates()
 				}
 				break;
 			}
+			case SoundSource::ADPCM:
+			{
+				// Volume
+				int vol = step.getVolume();
+				if (!isSetVolADPCM && 0 <= vol && vol < 0x100) {
+					isSetVolADPCM = true;
+					if (isPrevPos)
+						opnaCtrl_->setVolumeADPCM(step.getVolume());
+				}
+				// Instrument
+				if (!isSetInstADPCM && step.getInstrumentNumber() != -1) {
+					if (auto inst = std::dynamic_pointer_cast<InstrumentADPCM>(
+								instMan_.lock()->getInstrumentSharedPtr(step.getInstrumentNumber()))) {
+						isSetInstADPCM = true;
+						if (isPrevPos)
+							opnaCtrl_->setInstrumentADPCM(inst);
+					}
+				}
+				// Effects
+				for (int i = 3; i > -1; --i) {
+					Effect eff = Effect::makeEffectData(SoundSource::ADPCM, step.getEffectID(i), step.getEffectValue(i));
+					switch (eff.type) {
+					case EffectType::Arpeggio:
+						if (!isSetArpADPCM) {
+							isSetArpADPCM = true;
+							if (isPrevPos) opnaCtrl_->setArpeggioEffectADPCM(eff.value >> 4, eff.value & 0x0f);
+						}
+						break;
+					case EffectType::PortamentoUp:
+						if (!isSetPrtADPCM) {
+							isSetPrtADPCM = true;
+							if (isPrevPos) opnaCtrl_->setPortamentoEffectADPCM(eff.value);
+						}
+						break;
+					case EffectType::PortamentoDown:
+						if (!isSetPrtADPCM) {
+							isSetPrtADPCM = true;
+							if (isPrevPos) opnaCtrl_->setPortamentoEffectADPCM(-eff.value);
+						}
+						break;
+					case EffectType::TonePortamento:
+						if (!isSetPrtADPCM) {
+							isSetPrtADPCM = true;
+							if (isPrevPos) opnaCtrl_->setPortamentoEffectADPCM(eff.value, true);
+						}
+						break;
+					case EffectType::Vibrato:
+						if (!isSetVibADPCM) {
+							isSetVibADPCM = true;
+							if (isPrevPos) opnaCtrl_->setVibratoEffectADPCM(eff.value >> 4, eff.value & 0x0f);
+						}
+						break;
+					case EffectType::Tremolo:
+						if (!isSetTreADPCM) {
+							isSetTreADPCM = true;
+							if (isPrevPos) opnaCtrl_->setTremoloEffectADPCM(eff.value >> 4, eff.value & 0x0f);
+						}
+						break;
+					case EffectType::Pan:
+						if (-1 < eff.value && eff.value < 4 && !isSetPanADPCM) {
+							isSetPanADPCM = true;
+							if (isPrevPos) opnaCtrl_->setPanADPCM(eff.value);
+						}
+						break;
+					case EffectType::VolumeSlide:
+						if (!isSetVolSldADPCM) {
+							isSetVolSldADPCM = true;
+							if (isPrevPos) {
+								int hi = eff.value >> 4;
+								int low = eff.value & 0x0f;
+								if (hi && !low) opnaCtrl_->setVolumeSlideADPCM(hi, true);	// Slide up
+								else if (!hi) opnaCtrl_->setVolumeSlideADPCM(low, false);	// Slide down
+							}
+						}
+						break;
+					case EffectType::SpeedTempoChange:
+						if (!(speedStates & 0x4)) {
+							if (eff.value < 0x20 && !(speedStates & 0x1)) {	// Speed change
+								speedStates |= 0x1;
+								if (isPrevPos) effSpeedChange(eff.value);
+							}
+							else if (!(speedStates & 0x2)) {			// Tempo change
+								speedStates |= 0x2;
+								if (isPrevPos) effTempoChange(eff.value);
+							}
+						}
+						break;
+					case EffectType::Groove:
+						if (eff.value < static_cast<int>(mod_.lock()->getGrooveCount()) && !speedStates) {
+							speedStates |= 0x4;
+							if (isPrevPos) effGrooveChange(eff.value);
+						}
+						break;
+					case EffectType::Detune:
+						if (!isSetDtnADPCM) {
+							isSetDtnADPCM = true;
+							if (isPrevPos) opnaCtrl_->setDetuneADPCM(eff.value - 0x80);
+						}
+						break;
+					default:
+						break;
+					}
+				}
+				// Tone
+				int t = step.getNoteNumber();
+				if (isPrevPos && t != -1 && t != -2) {
+					--tonesCntADPCM;
+					for (auto it2 = toneADPCM.rbegin();
+						 it2 != toneADPCM.rend(); ++it2) {
+						if (*it2 == -1 || *it2 == tonesCntADPCM) {
+							if (t >= 0) {
+								*it2 = t;
+							}
+							else if (t < -2) {
+								*it2 = tonesCntADPCM - t + 2;
+							}
+							break;
+						}
+					}
+				}
+				break;
+			}
 			}
 		}
 
@@ -1722,6 +2118,13 @@ void PlaybackManager::retrieveChannelStates()
 		}
 		opnaCtrl_->haltSequencesSSG(static_cast<int>(ch));
 	}
+	for (size_t i = 0; i < 3; ++i) {
+		if (toneADPCM.at(i) >= 0) {
+			std::pair<int, Note> octNote = noteNumberToOctaveAndNote(toneADPCM[i]);
+			opnaCtrl_->updateEchoBufferADPCM(octNote.first, octNote.second, 0);
+		}
+	}
+	opnaCtrl_->haltSequencesADPCM();
 }
 
 size_t PlaybackManager::getOrderSize(int songNum) const
